/***********************************************************************************************
* [IttyBitty_GPIO_portpins.h]
*
* This file is part of the Itty Bitty Arduino library.
* Copyright  2016 Thomas J. Biuso III  ALL RIGHTS RESERVED...WHATEVER THAT MEANS.
* RELEASED UNDER THE GPL v3.0 LICENSE; SEE <LICENSE> FILE WITHIN DISTRIBUTION ROOT FOR TERMS.
***********************************************************************************************/

#ifndef _ITTYBITTY_GPIO_PORTPINS_H
#define _ITTYBITTY_GPIO_PORTPINS_H


#include "IttyBitty_GPIO.h"


#pragma region GENERAL PIN MANIPULATION MACROs

#define PIN_REF(port_ref, pin_number) reinterpret_cast<PVBYTE>(port_ref)[pin_number]

#define PORT_PIN_PAIR(port_ref, pin_number) port_ref, pin_number	// NOTE: For use with XXXXXX_PAIR bit manipulation macros

#define CHECK_PIN(port_ref, pin_number)				CHECK_SET(port_ref, pin_number)
#define CHECK_PIN_SET(port_ref, pin_number)			CHECK_PIN(port_ref, pin_number)
#define CHECK_PIN_UNSET(port_ref, pin_number)		CHECK_UNSET(port_ref, pin_number)

#define SET_PIN(port_ref, pin_number)				SET_BIT(port_ref, pin_number)
#define CLEAR_PIN(port_ref, pin_number)				CLEAR_BIT(port_ref, pin_number)
#define TOGGLE_PIN(port_ref, pin_number)			TOGGLE_BIT(port_ref, pin_number)

#define PIN_IN_CHECK(port_letter, pin_number)		CHECK_SET(PIN##port_letter, pin_number)
#define PIN_IN_CHECK_SET(port_letter, pin_number)	CHECK_SET(PIN##port_letter, pin_number)
#define PIN_IN_CHECK_UNSET(port_letter, pin_number)	CHECK_UNSET(PIN##port_letter, pin_number)

#define PIN_OUT_SET(port_letter, pin_number)		SET_BIT(PORT##port_letter, pin_number)
#define PIN_OUT_CLEAR(port_letter, pin_number)		CLEAR_BIT(PORT##port_letter, pin_number)
#define PIN_OUT_TOGGLE(port_letter, pin_number)		TOGGLE_BIT(PIN##port_letter, pin_number)

#define PIN_RESET(port_letter, pin_number)		\
	SET_PIN(PORT##port_letter, pin_number);		\
	SET_PIN(DDR##port_letter, pin_number);		\
	CLEAR_PIN(PORT##port_letter, pin_number);	\
	CLEAR_PIN(DDR##port_letter, pin_number)

#define PIN_GET_MODE(port_letter, pin_number)	\
	((PinMode)(CHECK_SET(DDR##port_letter, pin_number) OR CHECK_SET(PORT##port_letter, pin_number) SHL 0b1))

#define PIN_SET_MODE(port_letter, pin_number, mode)	\
	if (MASK((BYTE)(IttyBitty::PinMode)mode, OUTPUT)) SET_PIN(DDR##port_letter, pin_number); else CLEAR_PIN(DDR##port_letter, pin_number);		\
	if (MASK((BYTE)(IttyBitty::PinMode)mode, INPUT_PULLUP)) SET_PIN(PORT##port_letter, pin_number); else CLEAR_PIN(PORT##port_letter, pin_number)

#pragma endregion


#pragma region PIN-SPECIFIC MACROs

#ifdef PORTA

	#define DDRA_0 PIN_REF(DDRA, 0)
	#define DDRA_1 PIN_REF(DDRA, 1)
	#define DDRA_2 PIN_REF(DDRA, 2)
	#define DDRA_3 PIN_REF(DDRA, 3)
	#define DDRA_4 PIN_REF(DDRA, 4)
	#define DDRA_5 PIN_REF(DDRA, 5)
	#define DDRA_6 PIN_REF(DDRA, 6)
	#define DDRA_7 PIN_REF(DDRA, 7)

	#define PORTA_0 PIN_REF(PORTA, 0)
	#define PORTA_1 PIN_REF(PORTA, 1)
	#define PORTA_2 PIN_REF(PORTA, 2)
	#define PORTA_3 PIN_REF(PORTA, 3)
	#define PORTA_4 PIN_REF(PORTA, 4)
	#define PORTA_5 PIN_REF(PORTA, 5)
	#define PORTA_6 PIN_REF(PORTA, 6)
	#define PORTA_7 PIN_REF(PORTA, 7)

	#define PINA_0 PIN_REF(PINA, 0)
	#define PINA_1 PIN_REF(PINA, 1)
	#define PINA_2 PIN_REF(PINA, 2)
	#define PINA_3 PIN_REF(PINA, 3)
	#define PINA_4 PIN_REF(PINA, 4)
	#define PINA_5 PIN_REF(PINA, 5)
	#define PINA_6 PIN_REF(PINA, 6)
	#define PINA_7 PIN_REF(PINA, 7)


	#define DDRA0_PAIR() PORT_PIN_PAIR(DDRA, 0)
	#define DDRA1_PAIR() PORT_PIN_PAIR(DDRA, 1)
	#define DDRA2_PAIR() PORT_PIN_PAIR(DDRA, 2)
	#define DDRA3_PAIR() PORT_PIN_PAIR(DDRA, 3)
	#define DDRA4_PAIR() PORT_PIN_PAIR(DDRA, 4)
	#define DDRA5_PAIR() PORT_PIN_PAIR(DDRA, 5)
	#define DDRA6_PAIR() PORT_PIN_PAIR(DDRA, 6)
	#define DDRA7_PAIR() PORT_PIN_PAIR(DDRA, 7)

	#define PORTA0_PAIR() PORT_PIN_PAIR(PORTA, 0)
	#define PORTA1_PAIR() PORT_PIN_PAIR(PORTA, 1)
	#define PORTA2_PAIR() PORT_PIN_PAIR(PORTA, 2)
	#define PORTA3_PAIR() PORT_PIN_PAIR(PORTA, 3)
	#define PORTA4_PAIR() PORT_PIN_PAIR(PORTA, 4)
	#define PORTA5_PAIR() PORT_PIN_PAIR(PORTA, 5)
	#define PORTA6_PAIR() PORT_PIN_PAIR(PORTA, 6)
	#define PORTA7_PAIR() PORT_PIN_PAIR(PORTA, 7)

	#define PINA0_PAIR() PORT_PIN_PAIR(PINA, 0)
	#define PINA1_PAIR() PORT_PIN_PAIR(PINA, 1)
	#define PINA2_PAIR() PORT_PIN_PAIR(PINA, 2)
	#define PINA3_PAIR() PORT_PIN_PAIR(PINA, 3)
	#define PINA4_PAIR() PORT_PIN_PAIR(PINA, 4)
	#define PINA5_PAIR() PORT_PIN_PAIR(PINA, 5)
	#define PINA6_PAIR() PORT_PIN_PAIR(PINA, 6)
	#define PINA7_PAIR() PORT_PIN_PAIR(PINA, 7)


	#define DDRA0_CHECK() CHECK_PIN(DDRA, 0)
	#define DDRA1_CHECK() CHECK_PIN(DDRA, 1)
	#define DDRA2_CHECK() CHECK_PIN(DDRA, 2)
	#define DDRA3_CHECK() CHECK_PIN(DDRA, 3)
	#define DDRA4_CHECK() CHECK_PIN(DDRA, 4)
	#define DDRA5_CHECK() CHECK_PIN(DDRA, 5)
	#define DDRA6_CHECK() CHECK_PIN(DDRA, 6)
	#define DDRA7_CHECK() CHECK_PIN(DDRA, 7)

	#define PORTA0_CHECK() CHECK_PIN(PORTA, 0)
	#define PORTA1_CHECK() CHECK_PIN(PORTA, 1)
	#define PORTA2_CHECK() CHECK_PIN(PORTA, 2)
	#define PORTA3_CHECK() CHECK_PIN(PORTA, 3)
	#define PORTA4_CHECK() CHECK_PIN(PORTA, 4)
	#define PORTA5_CHECK() CHECK_PIN(PORTA, 5)
	#define PORTA6_CHECK() CHECK_PIN(PORTA, 6)
	#define PORTA7_CHECK() CHECK_PIN(PORTA, 7)

	#define PINA0_CHECK() CHECK_PIN(PINA, 0)
	#define PINA1_CHECK() CHECK_PIN(PINA, 1)
	#define PINA2_CHECK() CHECK_PIN(PINA, 2)
	#define PINA3_CHECK() CHECK_PIN(PINA, 3)
	#define PINA4_CHECK() CHECK_PIN(PINA, 4)
	#define PINA5_CHECK() CHECK_PIN(PINA, 5)
	#define PINA6_CHECK() CHECK_PIN(PINA, 6)
	#define PINA7_CHECK() CHECK_PIN(PINA, 7)


	#define DDRA0_IS_SET() CHECK_SET_PIN(DDRA, 0)
	#define DDRA1_IS_SET() CHECK_SET_PIN(DDRA, 1)
	#define DDRA2_IS_SET() CHECK_SET_PIN(DDRA, 2)
	#define DDRA3_IS_SET() CHECK_SET_PIN(DDRA, 3)
	#define DDRA4_IS_SET() CHECK_SET_PIN(DDRA, 4)
	#define DDRA5_IS_SET() CHECK_SET_PIN(DDRA, 5)
	#define DDRA6_IS_SET() CHECK_SET_PIN(DDRA, 6)
	#define DDRA7_IS_SET() CHECK_SET_PIN(DDRA, 7)

	#define PORTA0_IS_SET() CHECK_SET_PIN(PORTA, 0)
	#define PORTA1_IS_SET() CHECK_SET_PIN(PORTA, 1)
	#define PORTA2_IS_SET() CHECK_SET_PIN(PORTA, 2)
	#define PORTA3_IS_SET() CHECK_SET_PIN(PORTA, 3)
	#define PORTA4_IS_SET() CHECK_SET_PIN(PORTA, 4)
	#define PORTA5_IS_SET() CHECK_SET_PIN(PORTA, 5)
	#define PORTA6_IS_SET() CHECK_SET_PIN(PORTA, 6)
	#define PORTA7_IS_SET() CHECK_SET_PIN(PORTA, 7)

	#define PINA0_IS_SET() CHECK_SET_PIN(PINA, 0)
	#define PINA1_IS_SET() CHECK_SET_PIN(PINA, 1)
	#define PINA2_IS_SET() CHECK_SET_PIN(PINA, 2)
	#define PINA3_IS_SET() CHECK_SET_PIN(PINA, 3)
	#define PINA4_IS_SET() CHECK_SET_PIN(PINA, 4)
	#define PINA5_IS_SET() CHECK_SET_PIN(PINA, 5)
	#define PINA6_IS_SET() CHECK_SET_PIN(PINA, 6)
	#define PINA7_IS_SET() CHECK_SET_PIN(PINA, 7)


	#define DDRA0_IS_UNSET() CHECK_UNSET_PIN(DDRA, 0)
	#define DDRA1_IS_UNSET() CHECK_UNSET_PIN(DDRA, 1)
	#define DDRA2_IS_UNSET() CHECK_UNSET_PIN(DDRA, 2)
	#define DDRA3_IS_UNSET() CHECK_UNSET_PIN(DDRA, 3)
	#define DDRA4_IS_UNSET() CHECK_UNSET_PIN(DDRA, 4)
	#define DDRA5_IS_UNSET() CHECK_UNSET_PIN(DDRA, 5)
	#define DDRA6_IS_UNSET() CHECK_UNSET_PIN(DDRA, 6)
	#define DDRA7_IS_UNSET() CHECK_UNSET_PIN(DDRA, 7)

	#define PORTA0_IS_UNSET() CHECK_UNSET_PIN(PORTA, 0)
	#define PORTA1_IS_UNSET() CHECK_UNSET_PIN(PORTA, 1)
	#define PORTA2_IS_UNSET() CHECK_UNSET_PIN(PORTA, 2)
	#define PORTA3_IS_UNSET() CHECK_UNSET_PIN(PORTA, 3)
	#define PORTA4_IS_UNSET() CHECK_UNSET_PIN(PORTA, 4)
	#define PORTA5_IS_UNSET() CHECK_UNSET_PIN(PORTA, 5)
	#define PORTA6_IS_UNSET() CHECK_UNSET_PIN(PORTA, 6)
	#define PORTA7_IS_UNSET() CHECK_UNSET_PIN(PORTA, 7)

	#define PINA0_IS_UNSET() CHECK_UNSET_PIN(PINA, 0)
	#define PINA1_IS_UNSET() CHECK_UNSET_PIN(PINA, 1)
	#define PINA2_IS_UNSET() CHECK_UNSET_PIN(PINA, 2)
	#define PINA3_IS_UNSET() CHECK_UNSET_PIN(PINA, 3)
	#define PINA4_IS_UNSET() CHECK_UNSET_PIN(PINA, 4)
	#define PINA5_IS_UNSET() CHECK_UNSET_PIN(PINA, 5)
	#define PINA6_IS_UNSET() CHECK_UNSET_PIN(PINA, 6)
	#define PINA7_IS_UNSET() CHECK_UNSET_PIN(PINA, 7)


	#define DDRA0_SET() SET_PIN(DDRA, 0)
	#define DDRA1_SET() SET_PIN(DDRA, 1)
	#define DDRA2_SET() SET_PIN(DDRA, 2)
	#define DDRA3_SET() SET_PIN(DDRA, 3)
	#define DDRA4_SET() SET_PIN(DDRA, 4)
	#define DDRA5_SET() SET_PIN(DDRA, 5)
	#define DDRA6_SET() SET_PIN(DDRA, 6)
	#define DDRA7_SET() SET_PIN(DDRA, 7)

	#define PORTA0_SET() SET_PIN(PORTA, 0)
	#define PORTA1_SET() SET_PIN(PORTA, 1)
	#define PORTA2_SET() SET_PIN(PORTA, 2)
	#define PORTA3_SET() SET_PIN(PORTA, 3)
	#define PORTA4_SET() SET_PIN(PORTA, 4)
	#define PORTA5_SET() SET_PIN(PORTA, 5)
	#define PORTA6_SET() SET_PIN(PORTA, 6)
	#define PORTA7_SET() SET_PIN(PORTA, 7)

	#define PINA0_SET() SET_PIN(PINA, 0)
	#define PINA1_SET() SET_PIN(PINA, 1)
	#define PINA2_SET() SET_PIN(PINA, 2)
	#define PINA3_SET() SET_PIN(PINA, 3)
	#define PINA4_SET() SET_PIN(PINA, 4)
	#define PINA5_SET() SET_PIN(PINA, 5)
	#define PINA6_SET() SET_PIN(PINA, 6)
	#define PINA7_SET() SET_PIN(PINA, 7)


	#define DDRA0_CLEAR() CLEAR_PIN(DDRA, 0)
	#define DDRA1_CLEAR() CLEAR_PIN(DDRA, 1)
	#define DDRA2_CLEAR() CLEAR_PIN(DDRA, 2)
	#define DDRA3_CLEAR() CLEAR_PIN(DDRA, 3)
	#define DDRA4_CLEAR() CLEAR_PIN(DDRA, 4)
	#define DDRA5_CLEAR() CLEAR_PIN(DDRA, 5)
	#define DDRA6_CLEAR() CLEAR_PIN(DDRA, 6)
	#define DDRA7_CLEAR() CLEAR_PIN(DDRA, 7)

	#define PORTA0_CLEAR() CLEAR_PIN(PORTA, 0)
	#define PORTA1_CLEAR() CLEAR_PIN(PORTA, 1)
	#define PORTA2_CLEAR() CLEAR_PIN(PORTA, 2)
	#define PORTA3_CLEAR() CLEAR_PIN(PORTA, 3)
	#define PORTA4_CLEAR() CLEAR_PIN(PORTA, 4)
	#define PORTA5_CLEAR() CLEAR_PIN(PORTA, 5)
	#define PORTA6_CLEAR() CLEAR_PIN(PORTA, 6)
	#define PORTA7_CLEAR() CLEAR_PIN(PORTA, 7)

	#define PINA0_CLEAR() CLEAR_PIN(PINA, 0)
	#define PINA1_CLEAR() CLEAR_PIN(PINA, 1)
	#define PINA2_CLEAR() CLEAR_PIN(PINA, 2)
	#define PINA3_CLEAR() CLEAR_PIN(PINA, 3)
	#define PINA4_CLEAR() CLEAR_PIN(PINA, 4)
	#define PINA5_CLEAR() CLEAR_PIN(PINA, 5)
	#define PINA6_CLEAR() CLEAR_PIN(PINA, 6)
	#define PINA7_CLEAR() CLEAR_PIN(PINA, 7)


	#define DDRA0_TOGGLE() TOGGLE_PIN(DDRA, 0)
	#define DDRA1_TOGGLE() TOGGLE_PIN(DDRA, 1)
	#define DDRA2_TOGGLE() TOGGLE_PIN(DDRA, 2)
	#define DDRA3_TOGGLE() TOGGLE_PIN(DDRA, 3)
	#define DDRA4_TOGGLE() TOGGLE_PIN(DDRA, 4)
	#define DDRA5_TOGGLE() TOGGLE_PIN(DDRA, 5)
	#define DDRA6_TOGGLE() TOGGLE_PIN(DDRA, 6)
	#define DDRA7_TOGGLE() TOGGLE_PIN(DDRA, 7)

	#define PORTA0_TOGGLE() TOGGLE_PIN(PORTA, 0)
	#define PORTA1_TOGGLE() TOGGLE_PIN(PORTA, 1)
	#define PORTA2_TOGGLE() TOGGLE_PIN(PORTA, 2)
	#define PORTA3_TOGGLE() TOGGLE_PIN(PORTA, 3)
	#define PORTA4_TOGGLE() TOGGLE_PIN(PORTA, 4)
	#define PORTA5_TOGGLE() TOGGLE_PIN(PORTA, 5)
	#define PORTA6_TOGGLE() TOGGLE_PIN(PORTA, 6)
	#define PORTA7_TOGGLE() TOGGLE_PIN(PORTA, 7)

	#define PINA0_TOGGLE() TOGGLE_PIN(PINA, 0)
	#define PINA1_TOGGLE() TOGGLE_PIN(PINA, 1)
	#define PINA2_TOGGLE() TOGGLE_PIN(PINA, 2)
	#define PINA3_TOGGLE() TOGGLE_PIN(PINA, 3)
	#define PINA4_TOGGLE() TOGGLE_PIN(PINA, 4)
	#define PINA5_TOGGLE() TOGGLE_PIN(PINA, 5)
	#define PINA6_TOGGLE() TOGGLE_PIN(PINA, 6)
	#define PINA7_TOGGLE() TOGGLE_PIN(PINA, 7)


	#define PA0_CHECK() PIN_IN_CHECK(A, 0)
	#define PA1_CHECK() PIN_IN_CHECK(A, 1)
	#define PA2_CHECK() PIN_IN_CHECK(A, 2)
	#define PA3_CHECK() PIN_IN_CHECK(A, 3)
	#define PA4_CHECK() PIN_IN_CHECK(A, 4)
	#define PA5_CHECK() PIN_IN_CHECK(A, 5)
	#define PA6_CHECK() PIN_IN_CHECK(A, 6)
	#define PA7_CHECK() PIN_IN_CHECK(A, 7)

	#define PA0_IS_SET() PIN_IN_CHECK_SET(A, 0)
	#define PA1_IS_SET() PIN_IN_CHECK_SET(A, 1)
	#define PA2_IS_SET() PIN_IN_CHECK_SET(A, 2)
	#define PA3_IS_SET() PIN_IN_CHECK_SET(A, 3)
	#define PA4_IS_SET() PIN_IN_CHECK_SET(A, 4)
	#define PA5_IS_SET() PIN_IN_CHECK_SET(A, 5)
	#define PA6_IS_SET() PIN_IN_CHECK_SET(A, 6)
	#define PA7_IS_SET() PIN_IN_CHECK_SET(A, 7)

	#define PA0_IS_UNSET() PIN_IN_CHECK_UNSET(A, 0)
	#define PA1_IS_UNSET() PIN_IN_CHECK_UNSET(A, 1)
	#define PA2_IS_UNSET() PIN_IN_CHECK_UNSET(A, 2)
	#define PA3_IS_UNSET() PIN_IN_CHECK_UNSET(A, 3)
	#define PA4_IS_UNSET() PIN_IN_CHECK_UNSET(A, 4)
	#define PA5_IS_UNSET() PIN_IN_CHECK_UNSET(A, 5)
	#define PA6_IS_UNSET() PIN_IN_CHECK_UNSET(A, 6)
	#define PA7_IS_UNSET() PIN_IN_CHECK_UNSET(A, 7)


	#define PA0_SET() PIN_OUT_SET(A, 0)
	#define PA1_SET() PIN_OUT_SET(A, 1)
	#define PA2_SET() PIN_OUT_SET(A, 2)
	#define PA3_SET() PIN_OUT_SET(A, 3)
	#define PA4_SET() PIN_OUT_SET(A, 4)
	#define PA5_SET() PIN_OUT_SET(A, 5)
	#define PA6_SET() PIN_OUT_SET(A, 6)
	#define PA7_SET() PIN_OUT_SET(A, 7)

	#define PA0_CLEAR() PIN_OUT_CLEAR(A, 0)
	#define PA1_CLEAR() PIN_OUT_CLEAR(A, 1)
	#define PA2_CLEAR() PIN_OUT_CLEAR(A, 2)
	#define PA3_CLEAR() PIN_OUT_CLEAR(A, 3)
	#define PA4_CLEAR() PIN_OUT_CLEAR(A, 4)
	#define PA5_CLEAR() PIN_OUT_CLEAR(A, 5)
	#define PA6_CLEAR() PIN_OUT_CLEAR(A, 6)
	#define PA7_CLEAR() PIN_OUT_CLEAR(A, 7)

	#define PA0_TOGGLE() PIN_OUT_TOGGLE(A, 0)
	#define PA1_TOGGLE() PIN_OUT_TOGGLE(A, 1)
	#define PA2_TOGGLE() PIN_OUT_TOGGLE(A, 2)
	#define PA3_TOGGLE() PIN_OUT_TOGGLE(A, 3)
	#define PA4_TOGGLE() PIN_OUT_TOGGLE(A, 4)
	#define PA5_TOGGLE() PIN_OUT_TOGGLE(A, 5)
	#define PA6_TOGGLE() PIN_OUT_TOGGLE(A, 6)
	#define PA7_TOGGLE() PIN_OUT_TOGGLE(A, 7)

	#define PA0_RESET() PIN_RESET(A, 0)
	#define PA1_RESET() PIN_RESET(A, 1)
	#define PA2_RESET() PIN_RESET(A, 2)
	#define PA3_RESET() PIN_RESET(A, 3)
	#define PA4_RESET() PIN_RESET(A, 4)
	#define PA5_RESET() PIN_RESET(A, 5)
	#define PA6_RESET() PIN_RESET(A, 6)
	#define PA7_RESET() PIN_RESET(A, 7)

	#define PA0_GET_MODE() PIN_GET_MODE(A, 0)
	#define PA1_GET_MODE() PIN_GET_MODE(A, 1)
	#define PA2_GET_MODE() PIN_GET_MODE(A, 2)
	#define PA3_GET_MODE() PIN_GET_MODE(A, 3)
	#define PA4_GET_MODE() PIN_GET_MODE(A, 4)
	#define PA5_GET_MODE() PIN_GET_MODE(A, 5)
	#define PA6_GET_MODE() PIN_GET_MODE(A, 6)
	#define PA7_GET_MODE() PIN_GET_MODE(A, 7)

	#define PA0_SET_MODE(mode) PIN_SET_MODE(A, 0, mode)
	#define PA1_SET_MODE(mode) PIN_SET_MODE(A, 1, mode)
	#define PA2_SET_MODE(mode) PIN_SET_MODE(A, 2, mode)
	#define PA3_SET_MODE(mode) PIN_SET_MODE(A, 3, mode)
	#define PA4_SET_MODE(mode) PIN_SET_MODE(A, 4, mode)
	#define PA5_SET_MODE(mode) PIN_SET_MODE(A, 5, mode)
	#define PA6_SET_MODE(mode) PIN_SET_MODE(A, 6, mode)
	#define PA7_SET_MODE(mode) PIN_SET_MODE(A, 7, mode)

#endif


#ifdef PORTB

	#define DDRB_0 PIN_REF(DDRB, 0)
	#define DDRB_1 PIN_REF(DDRB, 1)
	#define DDRB_2 PIN_REF(DDRB, 2)
	#define DDRB_3 PIN_REF(DDRB, 3)
	#define DDRB_4 PIN_REF(DDRB, 4)
	#define DDRB_5 PIN_REF(DDRB, 5)
	#define DDRB_6 PIN_REF(DDRB, 6)
	#define DDRB_7 PIN_REF(DDRB, 7)

	#define PORTB_0 PIN_REF(PORTB, 0)
	#define PORTB_1 PIN_REF(PORTB, 1)
	#define PORTB_2 PIN_REF(PORTB, 2)
	#define PORTB_3 PIN_REF(PORTB, 3)
	#define PORTB_4 PIN_REF(PORTB, 4)
	#define PORTB_5 PIN_REF(PORTB, 5)
	#define PORTB_6 PIN_REF(PORTB, 6)
	#define PORTB_7 PIN_REF(PORTB, 7)

	#define PINB_0 PIN_REF(PINB, 0)
	#define PINB_1 PIN_REF(PINB, 1)
	#define PINB_2 PIN_REF(PINB, 2)
	#define PINB_3 PIN_REF(PINB, 3)
	#define PINB_4 PIN_REF(PINB, 4)
	#define PINB_5 PIN_REF(PINB, 5)
	#define PINB_6 PIN_REF(PINB, 6)
	#define PINB_7 PIN_REF(PINB, 7)


	#define DDRB0_PAIR() PORT_PIN_PAIR(DDRB, 0)
	#define DDRB1_PAIR() PORT_PIN_PAIR(DDRB, 1)
	#define DDRB2_PAIR() PORT_PIN_PAIR(DDRB, 2)
	#define DDRB3_PAIR() PORT_PIN_PAIR(DDRB, 3)
	#define DDRB4_PAIR() PORT_PIN_PAIR(DDRB, 4)
	#define DDRB5_PAIR() PORT_PIN_PAIR(DDRB, 5)
	#define DDRB6_PAIR() PORT_PIN_PAIR(DDRB, 6)
	#define DDRB7_PAIR() PORT_PIN_PAIR(DDRB, 7)

	#define PORTB0_PAIR() PORT_PIN_PAIR(PORTB, 0)
	#define PORTB1_PAIR() PORT_PIN_PAIR(PORTB, 1)
	#define PORTB2_PAIR() PORT_PIN_PAIR(PORTB, 2)
	#define PORTB3_PAIR() PORT_PIN_PAIR(PORTB, 3)
	#define PORTB4_PAIR() PORT_PIN_PAIR(PORTB, 4)
	#define PORTB5_PAIR() PORT_PIN_PAIR(PORTB, 5)
	#define PORTB6_PAIR() PORT_PIN_PAIR(PORTB, 6)
	#define PORTB7_PAIR() PORT_PIN_PAIR(PORTB, 7)

	#define PINB0_PAIR() PORT_PIN_PAIR(PINB, 0)
	#define PINB1_PAIR() PORT_PIN_PAIR(PINB, 1)
	#define PINB2_PAIR() PORT_PIN_PAIR(PINB, 2)
	#define PINB3_PAIR() PORT_PIN_PAIR(PINB, 3)
	#define PINB4_PAIR() PORT_PIN_PAIR(PINB, 4)
	#define PINB5_PAIR() PORT_PIN_PAIR(PINB, 5)
	#define PINB6_PAIR() PORT_PIN_PAIR(PINB, 6)
	#define PINB7_PAIR() PORT_PIN_PAIR(PINB, 7)


	#define DDRB0_CHECK() CHECK_PIN(DDRB, 0)
	#define DDRB1_CHECK() CHECK_PIN(DDRB, 1)
	#define DDRB2_CHECK() CHECK_PIN(DDRB, 2)
	#define DDRB3_CHECK() CHECK_PIN(DDRB, 3)
	#define DDRB4_CHECK() CHECK_PIN(DDRB, 4)
	#define DDRB5_CHECK() CHECK_PIN(DDRB, 5)
	#define DDRB6_CHECK() CHECK_PIN(DDRB, 6)
	#define DDRB7_CHECK() CHECK_PIN(DDRB, 7)

	#define PORTB0_CHECK() CHECK_PIN(PORTB, 0)
	#define PORTB1_CHECK() CHECK_PIN(PORTB, 1)
	#define PORTB2_CHECK() CHECK_PIN(PORTB, 2)
	#define PORTB3_CHECK() CHECK_PIN(PORTB, 3)
	#define PORTB4_CHECK() CHECK_PIN(PORTB, 4)
	#define PORTB5_CHECK() CHECK_PIN(PORTB, 5)
	#define PORTB6_CHECK() CHECK_PIN(PORTB, 6)
	#define PORTB7_CHECK() CHECK_PIN(PORTB, 7)

	#define PINB0_CHECK() CHECK_PIN(PINB, 0)
	#define PINB1_CHECK() CHECK_PIN(PINB, 1)
	#define PINB2_CHECK() CHECK_PIN(PINB, 2)
	#define PINB3_CHECK() CHECK_PIN(PINB, 3)
	#define PINB4_CHECK() CHECK_PIN(PINB, 4)
	#define PINB5_CHECK() CHECK_PIN(PINB, 5)
	#define PINB6_CHECK() CHECK_PIN(PINB, 6)
	#define PINB7_CHECK() CHECK_PIN(PINB, 7)


	#define DDRB0_IS_SET() CHECK_SET_PIN(DDRB, 0)
	#define DDRB1_IS_SET() CHECK_SET_PIN(DDRB, 1)
	#define DDRB2_IS_SET() CHECK_SET_PIN(DDRB, 2)
	#define DDRB3_IS_SET() CHECK_SET_PIN(DDRB, 3)
	#define DDRB4_IS_SET() CHECK_SET_PIN(DDRB, 4)
	#define DDRB5_IS_SET() CHECK_SET_PIN(DDRB, 5)
	#define DDRB6_IS_SET() CHECK_SET_PIN(DDRB, 6)
	#define DDRB7_IS_SET() CHECK_SET_PIN(DDRB, 7)

	#define PORTB0_IS_SET() CHECK_SET_PIN(PORTB, 0)
	#define PORTB1_IS_SET() CHECK_SET_PIN(PORTB, 1)
	#define PORTB2_IS_SET() CHECK_SET_PIN(PORTB, 2)
	#define PORTB3_IS_SET() CHECK_SET_PIN(PORTB, 3)
	#define PORTB4_IS_SET() CHECK_SET_PIN(PORTB, 4)
	#define PORTB5_IS_SET() CHECK_SET_PIN(PORTB, 5)
	#define PORTB6_IS_SET() CHECK_SET_PIN(PORTB, 6)
	#define PORTB7_IS_SET() CHECK_SET_PIN(PORTB, 7)

	#define PINB0_IS_SET() CHECK_SET_PIN(PINB, 0)
	#define PINB1_IS_SET() CHECK_SET_PIN(PINB, 1)
	#define PINB2_IS_SET() CHECK_SET_PIN(PINB, 2)
	#define PINB3_IS_SET() CHECK_SET_PIN(PINB, 3)
	#define PINB4_IS_SET() CHECK_SET_PIN(PINB, 4)
	#define PINB5_IS_SET() CHECK_SET_PIN(PINB, 5)
	#define PINB6_IS_SET() CHECK_SET_PIN(PINB, 6)
	#define PINB7_IS_SET() CHECK_SET_PIN(PINB, 7)


	#define DDRB0_IS_UNSET() CHECK_UNSET_PIN(DDRB, 0)
	#define DDRB1_IS_UNSET() CHECK_UNSET_PIN(DDRB, 1)
	#define DDRB2_IS_UNSET() CHECK_UNSET_PIN(DDRB, 2)
	#define DDRB3_IS_UNSET() CHECK_UNSET_PIN(DDRB, 3)
	#define DDRB4_IS_UNSET() CHECK_UNSET_PIN(DDRB, 4)
	#define DDRB5_IS_UNSET() CHECK_UNSET_PIN(DDRB, 5)
	#define DDRB6_IS_UNSET() CHECK_UNSET_PIN(DDRB, 6)
	#define DDRB7_IS_UNSET() CHECK_UNSET_PIN(DDRB, 7)

	#define PORTB0_IS_UNSET() CHECK_UNSET_PIN(PORTB, 0)
	#define PORTB1_IS_UNSET() CHECK_UNSET_PIN(PORTB, 1)
	#define PORTB2_IS_UNSET() CHECK_UNSET_PIN(PORTB, 2)
	#define PORTB3_IS_UNSET() CHECK_UNSET_PIN(PORTB, 3)
	#define PORTB4_IS_UNSET() CHECK_UNSET_PIN(PORTB, 4)
	#define PORTB5_IS_UNSET() CHECK_UNSET_PIN(PORTB, 5)
	#define PORTB6_IS_UNSET() CHECK_UNSET_PIN(PORTB, 6)
	#define PORTB7_IS_UNSET() CHECK_UNSET_PIN(PORTB, 7)

	#define PINB0_IS_UNSET() CHECK_UNSET_PIN(PINB, 0)
	#define PINB1_IS_UNSET() CHECK_UNSET_PIN(PINB, 1)
	#define PINB2_IS_UNSET() CHECK_UNSET_PIN(PINB, 2)
	#define PINB3_IS_UNSET() CHECK_UNSET_PIN(PINB, 3)
	#define PINB4_IS_UNSET() CHECK_UNSET_PIN(PINB, 4)
	#define PINB5_IS_UNSET() CHECK_UNSET_PIN(PINB, 5)
	#define PINB6_IS_UNSET() CHECK_UNSET_PIN(PINB, 6)
	#define PINB7_IS_UNSET() CHECK_UNSET_PIN(PINB, 7)


	#define DDRB0_SET() SET_PIN(DDRB, 0)
	#define DDRB1_SET() SET_PIN(DDRB, 1)
	#define DDRB2_SET() SET_PIN(DDRB, 2)
	#define DDRB3_SET() SET_PIN(DDRB, 3)
	#define DDRB4_SET() SET_PIN(DDRB, 4)
	#define DDRB5_SET() SET_PIN(DDRB, 5)
	#define DDRB6_SET() SET_PIN(DDRB, 6)
	#define DDRB7_SET() SET_PIN(DDRB, 7)

	#define PORTB0_SET() SET_PIN(PORTB, 0)
	#define PORTB1_SET() SET_PIN(PORTB, 1)
	#define PORTB2_SET() SET_PIN(PORTB, 2)
	#define PORTB3_SET() SET_PIN(PORTB, 3)
	#define PORTB4_SET() SET_PIN(PORTB, 4)
	#define PORTB5_SET() SET_PIN(PORTB, 5)
	#define PORTB6_SET() SET_PIN(PORTB, 6)
	#define PORTB7_SET() SET_PIN(PORTB, 7)

	#define PINB0_SET() SET_PIN(PINB, 0)
	#define PINB1_SET() SET_PIN(PINB, 1)
	#define PINB2_SET() SET_PIN(PINB, 2)
	#define PINB3_SET() SET_PIN(PINB, 3)
	#define PINB4_SET() SET_PIN(PINB, 4)
	#define PINB5_SET() SET_PIN(PINB, 5)
	#define PINB6_SET() SET_PIN(PINB, 6)
	#define PINB7_SET() SET_PIN(PINB, 7)


	#define DDRB0_CLEAR() CLEAR_PIN(DDRB, 0)
	#define DDRB1_CLEAR() CLEAR_PIN(DDRB, 1)
	#define DDRB2_CLEAR() CLEAR_PIN(DDRB, 2)
	#define DDRB3_CLEAR() CLEAR_PIN(DDRB, 3)
	#define DDRB4_CLEAR() CLEAR_PIN(DDRB, 4)
	#define DDRB5_CLEAR() CLEAR_PIN(DDRB, 5)
	#define DDRB6_CLEAR() CLEAR_PIN(DDRB, 6)
	#define DDRB7_CLEAR() CLEAR_PIN(DDRB, 7)

	#define PORTB0_CLEAR() CLEAR_PIN(PORTB, 0)
	#define PORTB1_CLEAR() CLEAR_PIN(PORTB, 1)
	#define PORTB2_CLEAR() CLEAR_PIN(PORTB, 2)
	#define PORTB3_CLEAR() CLEAR_PIN(PORTB, 3)
	#define PORTB4_CLEAR() CLEAR_PIN(PORTB, 4)
	#define PORTB5_CLEAR() CLEAR_PIN(PORTB, 5)
	#define PORTB6_CLEAR() CLEAR_PIN(PORTB, 6)
	#define PORTB7_CLEAR() CLEAR_PIN(PORTB, 7)

	#define PINB0_CLEAR() CLEAR_PIN(PINB, 0)
	#define PINB1_CLEAR() CLEAR_PIN(PINB, 1)
	#define PINB2_CLEAR() CLEAR_PIN(PINB, 2)
	#define PINB3_CLEAR() CLEAR_PIN(PINB, 3)
	#define PINB4_CLEAR() CLEAR_PIN(PINB, 4)
	#define PINB5_CLEAR() CLEAR_PIN(PINB, 5)
	#define PINB6_CLEAR() CLEAR_PIN(PINB, 6)
	#define PINB7_CLEAR() CLEAR_PIN(PINB, 7)


	#define DDRB0_TOGGLE() TOGGLE_PIN(DDRB, 0)
	#define DDRB1_TOGGLE() TOGGLE_PIN(DDRB, 1)
	#define DDRB2_TOGGLE() TOGGLE_PIN(DDRB, 2)
	#define DDRB3_TOGGLE() TOGGLE_PIN(DDRB, 3)
	#define DDRB4_TOGGLE() TOGGLE_PIN(DDRB, 4)
	#define DDRB5_TOGGLE() TOGGLE_PIN(DDRB, 5)
	#define DDRB6_TOGGLE() TOGGLE_PIN(DDRB, 6)
	#define DDRB7_TOGGLE() TOGGLE_PIN(DDRB, 7)

	#define PORTB0_TOGGLE() TOGGLE_PIN(PORTB, 0)
	#define PORTB1_TOGGLE() TOGGLE_PIN(PORTB, 1)
	#define PORTB2_TOGGLE() TOGGLE_PIN(PORTB, 2)
	#define PORTB3_TOGGLE() TOGGLE_PIN(PORTB, 3)
	#define PORTB4_TOGGLE() TOGGLE_PIN(PORTB, 4)
	#define PORTB5_TOGGLE() TOGGLE_PIN(PORTB, 5)
	#define PORTB6_TOGGLE() TOGGLE_PIN(PORTB, 6)
	#define PORTB7_TOGGLE() TOGGLE_PIN(PORTB, 7)

	#define PINB0_TOGGLE() TOGGLE_PIN(PINB, 0)
	#define PINB1_TOGGLE() TOGGLE_PIN(PINB, 1)
	#define PINB2_TOGGLE() TOGGLE_PIN(PINB, 2)
	#define PINB3_TOGGLE() TOGGLE_PIN(PINB, 3)
	#define PINB4_TOGGLE() TOGGLE_PIN(PINB, 4)
	#define PINB5_TOGGLE() TOGGLE_PIN(PINB, 5)
	#define PINB6_TOGGLE() TOGGLE_PIN(PINB, 6)
	#define PINB7_TOGGLE() TOGGLE_PIN(PINB, 7)


	#define PB0_CHECK() PIN_IN_CHECK(B, 0)
	#define PB1_CHECK() PIN_IN_CHECK(B, 1)
	#define PB2_CHECK() PIN_IN_CHECK(B, 2)
	#define PB3_CHECK() PIN_IN_CHECK(B, 3)
	#define PB4_CHECK() PIN_IN_CHECK(B, 4)
	#define PB5_CHECK() PIN_IN_CHECK(B, 5)
	#define PB6_CHECK() PIN_IN_CHECK(B, 6)
	#define PB7_CHECK() PIN_IN_CHECK(B, 7)

	#define PB0_IS_SET() PIN_IN_CHECK_SET(B, 0)
	#define PB1_IS_SET() PIN_IN_CHECK_SET(B, 1)
	#define PB2_IS_SET() PIN_IN_CHECK_SET(B, 2)
	#define PB3_IS_SET() PIN_IN_CHECK_SET(B, 3)
	#define PB4_IS_SET() PIN_IN_CHECK_SET(B, 4)
	#define PB5_IS_SET() PIN_IN_CHECK_SET(B, 5)
	#define PB6_IS_SET() PIN_IN_CHECK_SET(B, 6)
	#define PB7_IS_SET() PIN_IN_CHECK_SET(B, 7)

	#define PB0_IS_UNSET() PIN_IN_CHECK_UNSET(B, 0)
	#define PB1_IS_UNSET() PIN_IN_CHECK_UNSET(B, 1)
	#define PB2_IS_UNSET() PIN_IN_CHECK_UNSET(B, 2)
	#define PB3_IS_UNSET() PIN_IN_CHECK_UNSET(B, 3)
	#define PB4_IS_UNSET() PIN_IN_CHECK_UNSET(B, 4)
	#define PB5_IS_UNSET() PIN_IN_CHECK_UNSET(B, 5)
	#define PB6_IS_UNSET() PIN_IN_CHECK_UNSET(B, 6)
	#define PB7_IS_UNSET() PIN_IN_CHECK_UNSET(B, 7)


	#define PB0_SET() PIN_OUT_SET(B, 0)
	#define PB1_SET() PIN_OUT_SET(B, 1)
	#define PB2_SET() PIN_OUT_SET(B, 2)
	#define PB3_SET() PIN_OUT_SET(B, 3)
	#define PB4_SET() PIN_OUT_SET(B, 4)
	#define PB5_SET() PIN_OUT_SET(B, 5)
	#define PB6_SET() PIN_OUT_SET(B, 6)
	#define PB7_SET() PIN_OUT_SET(B, 7)

	#define PB0_CLEAR() PIN_OUT_CLEAR(B, 0)
	#define PB1_CLEAR() PIN_OUT_CLEAR(B, 1)
	#define PB2_CLEAR() PIN_OUT_CLEAR(B, 2)
	#define PB3_CLEAR() PIN_OUT_CLEAR(B, 3)
	#define PB4_CLEAR() PIN_OUT_CLEAR(B, 4)
	#define PB5_CLEAR() PIN_OUT_CLEAR(B, 5)
	#define PB6_CLEAR() PIN_OUT_CLEAR(B, 6)
	#define PB7_CLEAR() PIN_OUT_CLEAR(B, 7)

	#define PB0_TOGGLE() PIN_OUT_TOGGLE(B, 0)
	#define PB1_TOGGLE() PIN_OUT_TOGGLE(B, 1)
	#define PB2_TOGGLE() PIN_OUT_TOGGLE(B, 2)
	#define PB3_TOGGLE() PIN_OUT_TOGGLE(B, 3)
	#define PB4_TOGGLE() PIN_OUT_TOGGLE(B, 4)
	#define PB5_TOGGLE() PIN_OUT_TOGGLE(B, 5)
	#define PB6_TOGGLE() PIN_OUT_TOGGLE(B, 6)
	#define PB7_TOGGLE() PIN_OUT_TOGGLE(B, 7)

	#define PB0_RESET() PIN_RESET(B, 0)
	#define PB1_RESET() PIN_RESET(B, 1)
	#define PB2_RESET() PIN_RESET(B, 2)
	#define PB3_RESET() PIN_RESET(B, 3)
	#define PB4_RESET() PIN_RESET(B, 4)
	#define PB5_RESET() PIN_RESET(B, 5)
	#define PB6_RESET() PIN_RESET(B, 6)
	#define PB7_RESET() PIN_RESET(B, 7)

	#define PB0_GET_MODE() PIN_GET_MODE(B, 0)
	#define PB1_GET_MODE() PIN_GET_MODE(B, 1)
	#define PB2_GET_MODE() PIN_GET_MODE(B, 2)
	#define PB3_GET_MODE() PIN_GET_MODE(B, 3)
	#define PB4_GET_MODE() PIN_GET_MODE(B, 4)
	#define PB5_GET_MODE() PIN_GET_MODE(B, 5)
	#define PB6_GET_MODE() PIN_GET_MODE(B, 6)
	#define PB7_GET_MODE() PIN_GET_MODE(B, 7)

	#define PB0_SET_MODE(mode) PIN_SET_MODE(B, 0, mode)
	#define PB1_SET_MODE(mode) PIN_SET_MODE(B, 1, mode)
	#define PB2_SET_MODE(mode) PIN_SET_MODE(B, 2, mode)
	#define PB3_SET_MODE(mode) PIN_SET_MODE(B, 3, mode)
	#define PB4_SET_MODE(mode) PIN_SET_MODE(B, 4, mode)
	#define PB5_SET_MODE(mode) PIN_SET_MODE(B, 5, mode)
	#define PB6_SET_MODE(mode) PIN_SET_MODE(B, 6, mode)
	#define PB7_SET_MODE(mode) PIN_SET_MODE(B, 7, mode)

#endif


#ifdef PORTC

	#define DDRC_0 PIN_REF(DDRC, 0)
	#define DDRC_1 PIN_REF(DDRC, 1)
	#define DDRC_2 PIN_REF(DDRC, 2)
	#define DDRC_3 PIN_REF(DDRC, 3)
	#define DDRC_4 PIN_REF(DDRC, 4)
	#define DDRC_5 PIN_REF(DDRC, 5)
	#define DDRC_6 PIN_REF(DDRC, 6)
	#define DDRC_7 PIN_REF(DDRC, 7)

	#define PORTC_0 PIN_REF(PORTC, 0)
	#define PORTC_1 PIN_REF(PORTC, 1)
	#define PORTC_2 PIN_REF(PORTC, 2)
	#define PORTC_3 PIN_REF(PORTC, 3)
	#define PORTC_4 PIN_REF(PORTC, 4)
	#define PORTC_5 PIN_REF(PORTC, 5)
	#define PORTC_6 PIN_REF(PORTC, 6)
	#define PORTC_7 PIN_REF(PORTC, 7)

	#define PINC_0 PIN_REF(PINC, 0)
	#define PINC_1 PIN_REF(PINC, 1)
	#define PINC_2 PIN_REF(PINC, 2)
	#define PINC_3 PIN_REF(PINC, 3)
	#define PINC_4 PIN_REF(PINC, 4)
	#define PINC_5 PIN_REF(PINC, 5)
	#define PINC_6 PIN_REF(PINC, 6)
	#define PINC_7 PIN_REF(PINC, 7)


	#define DDRC0_PAIR() PORT_PIN_PAIR(DDRC, 0)
	#define DDRC1_PAIR() PORT_PIN_PAIR(DDRC, 1)
	#define DDRC2_PAIR() PORT_PIN_PAIR(DDRC, 2)
	#define DDRC3_PAIR() PORT_PIN_PAIR(DDRC, 3)
	#define DDRC4_PAIR() PORT_PIN_PAIR(DDRC, 4)
	#define DDRC5_PAIR() PORT_PIN_PAIR(DDRC, 5)
	#define DDRC6_PAIR() PORT_PIN_PAIR(DDRC, 6)
	#define DDRC7_PAIR() PORT_PIN_PAIR(DDRC, 7)

	#define PORTC0_PAIR() PORT_PIN_PAIR(PORTC, 0)
	#define PORTC1_PAIR() PORT_PIN_PAIR(PORTC, 1)
	#define PORTC2_PAIR() PORT_PIN_PAIR(PORTC, 2)
	#define PORTC3_PAIR() PORT_PIN_PAIR(PORTC, 3)
	#define PORTC4_PAIR() PORT_PIN_PAIR(PORTC, 4)
	#define PORTC5_PAIR() PORT_PIN_PAIR(PORTC, 5)
	#define PORTC6_PAIR() PORT_PIN_PAIR(PORTC, 6)
	#define PORTC7_PAIR() PORT_PIN_PAIR(PORTC, 7)

	#define PINC0_PAIR() PORT_PIN_PAIR(PINC, 0)
	#define PINC1_PAIR() PORT_PIN_PAIR(PINC, 1)
	#define PINC2_PAIR() PORT_PIN_PAIR(PINC, 2)
	#define PINC3_PAIR() PORT_PIN_PAIR(PINC, 3)
	#define PINC4_PAIR() PORT_PIN_PAIR(PINC, 4)
	#define PINC5_PAIR() PORT_PIN_PAIR(PINC, 5)
	#define PINC6_PAIR() PORT_PIN_PAIR(PINC, 6)
	#define PINC7_PAIR() PORT_PIN_PAIR(PINC, 7)


	#define DDRC0_CHECK() CHECK_PIN(DDRC, 0)
	#define DDRC1_CHECK() CHECK_PIN(DDRC, 1)
	#define DDRC2_CHECK() CHECK_PIN(DDRC, 2)
	#define DDRC3_CHECK() CHECK_PIN(DDRC, 3)
	#define DDRC4_CHECK() CHECK_PIN(DDRC, 4)
	#define DDRC5_CHECK() CHECK_PIN(DDRC, 5)
	#define DDRC6_CHECK() CHECK_PIN(DDRC, 6)
	#define DDRC7_CHECK() CHECK_PIN(DDRC, 7)

	#define PORTC0_CHECK() CHECK_PIN(PORTC, 0)
	#define PORTC1_CHECK() CHECK_PIN(PORTC, 1)
	#define PORTC2_CHECK() CHECK_PIN(PORTC, 2)
	#define PORTC3_CHECK() CHECK_PIN(PORTC, 3)
	#define PORTC4_CHECK() CHECK_PIN(PORTC, 4)
	#define PORTC5_CHECK() CHECK_PIN(PORTC, 5)
	#define PORTC6_CHECK() CHECK_PIN(PORTC, 6)
	#define PORTC7_CHECK() CHECK_PIN(PORTC, 7)

	#define PINC0_CHECK() CHECK_PIN(PINC, 0)
	#define PINC1_CHECK() CHECK_PIN(PINC, 1)
	#define PINC2_CHECK() CHECK_PIN(PINC, 2)
	#define PINC3_CHECK() CHECK_PIN(PINC, 3)
	#define PINC4_CHECK() CHECK_PIN(PINC, 4)
	#define PINC5_CHECK() CHECK_PIN(PINC, 5)
	#define PINC6_CHECK() CHECK_PIN(PINC, 6)
	#define PINC7_CHECK() CHECK_PIN(PINC, 7)


	#define DDRC0_IS_SET() CHECK_SET_PIN(DDRC, 0)
	#define DDRC1_IS_SET() CHECK_SET_PIN(DDRC, 1)
	#define DDRC2_IS_SET() CHECK_SET_PIN(DDRC, 2)
	#define DDRC3_IS_SET() CHECK_SET_PIN(DDRC, 3)
	#define DDRC4_IS_SET() CHECK_SET_PIN(DDRC, 4)
	#define DDRC5_IS_SET() CHECK_SET_PIN(DDRC, 5)
	#define DDRC6_IS_SET() CHECK_SET_PIN(DDRC, 6)
	#define DDRC7_IS_SET() CHECK_SET_PIN(DDRC, 7)

	#define PORTC0_IS_SET() CHECK_SET_PIN(PORTC, 0)
	#define PORTC1_IS_SET() CHECK_SET_PIN(PORTC, 1)
	#define PORTC2_IS_SET() CHECK_SET_PIN(PORTC, 2)
	#define PORTC3_IS_SET() CHECK_SET_PIN(PORTC, 3)
	#define PORTC4_IS_SET() CHECK_SET_PIN(PORTC, 4)
	#define PORTC5_IS_SET() CHECK_SET_PIN(PORTC, 5)
	#define PORTC6_IS_SET() CHECK_SET_PIN(PORTC, 6)
	#define PORTC7_IS_SET() CHECK_SET_PIN(PORTC, 7)

	#define PINC0_IS_SET() CHECK_SET_PIN(PINC, 0)
	#define PINC1_IS_SET() CHECK_SET_PIN(PINC, 1)
	#define PINC2_IS_SET() CHECK_SET_PIN(PINC, 2)
	#define PINC3_IS_SET() CHECK_SET_PIN(PINC, 3)
	#define PINC4_IS_SET() CHECK_SET_PIN(PINC, 4)
	#define PINC5_IS_SET() CHECK_SET_PIN(PINC, 5)
	#define PINC6_IS_SET() CHECK_SET_PIN(PINC, 6)
	#define PINC7_IS_SET() CHECK_SET_PIN(PINC, 7)


	#define DDRC0_IS_UNSET() CHECK_UNSET_PIN(DDRC, 0)
	#define DDRC1_IS_UNSET() CHECK_UNSET_PIN(DDRC, 1)
	#define DDRC2_IS_UNSET() CHECK_UNSET_PIN(DDRC, 2)
	#define DDRC3_IS_UNSET() CHECK_UNSET_PIN(DDRC, 3)
	#define DDRC4_IS_UNSET() CHECK_UNSET_PIN(DDRC, 4)
	#define DDRC5_IS_UNSET() CHECK_UNSET_PIN(DDRC, 5)
	#define DDRC6_IS_UNSET() CHECK_UNSET_PIN(DDRC, 6)
	#define DDRC7_IS_UNSET() CHECK_UNSET_PIN(DDRC, 7)

	#define PORTC0_IS_UNSET() CHECK_UNSET_PIN(PORTC, 0)
	#define PORTC1_IS_UNSET() CHECK_UNSET_PIN(PORTC, 1)
	#define PORTC2_IS_UNSET() CHECK_UNSET_PIN(PORTC, 2)
	#define PORTC3_IS_UNSET() CHECK_UNSET_PIN(PORTC, 3)
	#define PORTC4_IS_UNSET() CHECK_UNSET_PIN(PORTC, 4)
	#define PORTC5_IS_UNSET() CHECK_UNSET_PIN(PORTC, 5)
	#define PORTC6_IS_UNSET() CHECK_UNSET_PIN(PORTC, 6)
	#define PORTC7_IS_UNSET() CHECK_UNSET_PIN(PORTC, 7)

	#define PINC0_IS_UNSET() CHECK_UNSET_PIN(PINC, 0)
	#define PINC1_IS_UNSET() CHECK_UNSET_PIN(PINC, 1)
	#define PINC2_IS_UNSET() CHECK_UNSET_PIN(PINC, 2)
	#define PINC3_IS_UNSET() CHECK_UNSET_PIN(PINC, 3)
	#define PINC4_IS_UNSET() CHECK_UNSET_PIN(PINC, 4)
	#define PINC5_IS_UNSET() CHECK_UNSET_PIN(PINC, 5)
	#define PINC6_IS_UNSET() CHECK_UNSET_PIN(PINC, 6)
	#define PINC7_IS_UNSET() CHECK_UNSET_PIN(PINC, 7)


	#define DDRC0_SET() SET_PIN(DDRC, 0)
	#define DDRC1_SET() SET_PIN(DDRC, 1)
	#define DDRC2_SET() SET_PIN(DDRC, 2)
	#define DDRC3_SET() SET_PIN(DDRC, 3)
	#define DDRC4_SET() SET_PIN(DDRC, 4)
	#define DDRC5_SET() SET_PIN(DDRC, 5)
	#define DDRC6_SET() SET_PIN(DDRC, 6)
	#define DDRC7_SET() SET_PIN(DDRC, 7)

	#define PORTC0_SET() SET_PIN(PORTC, 0)
	#define PORTC1_SET() SET_PIN(PORTC, 1)
	#define PORTC2_SET() SET_PIN(PORTC, 2)
	#define PORTC3_SET() SET_PIN(PORTC, 3)
	#define PORTC4_SET() SET_PIN(PORTC, 4)
	#define PORTC5_SET() SET_PIN(PORTC, 5)
	#define PORTC6_SET() SET_PIN(PORTC, 6)
	#define PORTC7_SET() SET_PIN(PORTC, 7)

	#define PINC0_SET() SET_PIN(PINC, 0)
	#define PINC1_SET() SET_PIN(PINC, 1)
	#define PINC2_SET() SET_PIN(PINC, 2)
	#define PINC3_SET() SET_PIN(PINC, 3)
	#define PINC4_SET() SET_PIN(PINC, 4)
	#define PINC5_SET() SET_PIN(PINC, 5)
	#define PINC6_SET() SET_PIN(PINC, 6)
	#define PINC7_SET() SET_PIN(PINC, 7)


	#define DDRC0_CLEAR() CLEAR_PIN(DDRC, 0)
	#define DDRC1_CLEAR() CLEAR_PIN(DDRC, 1)
	#define DDRC2_CLEAR() CLEAR_PIN(DDRC, 2)
	#define DDRC3_CLEAR() CLEAR_PIN(DDRC, 3)
	#define DDRC4_CLEAR() CLEAR_PIN(DDRC, 4)
	#define DDRC5_CLEAR() CLEAR_PIN(DDRC, 5)
	#define DDRC6_CLEAR() CLEAR_PIN(DDRC, 6)
	#define DDRC7_CLEAR() CLEAR_PIN(DDRC, 7)

	#define PORTC0_CLEAR() CLEAR_PIN(PORTC, 0)
	#define PORTC1_CLEAR() CLEAR_PIN(PORTC, 1)
	#define PORTC2_CLEAR() CLEAR_PIN(PORTC, 2)
	#define PORTC3_CLEAR() CLEAR_PIN(PORTC, 3)
	#define PORTC4_CLEAR() CLEAR_PIN(PORTC, 4)
	#define PORTC5_CLEAR() CLEAR_PIN(PORTC, 5)
	#define PORTC6_CLEAR() CLEAR_PIN(PORTC, 6)
	#define PORTC7_CLEAR() CLEAR_PIN(PORTC, 7)

	#define PINC0_CLEAR() CLEAR_PIN(PINC, 0)
	#define PINC1_CLEAR() CLEAR_PIN(PINC, 1)
	#define PINC2_CLEAR() CLEAR_PIN(PINC, 2)
	#define PINC3_CLEAR() CLEAR_PIN(PINC, 3)
	#define PINC4_CLEAR() CLEAR_PIN(PINC, 4)
	#define PINC5_CLEAR() CLEAR_PIN(PINC, 5)
	#define PINC6_CLEAR() CLEAR_PIN(PINC, 6)
	#define PINC7_CLEAR() CLEAR_PIN(PINC, 7)


	#define DDRC0_TOGGLE() TOGGLE_PIN(DDRC, 0)
	#define DDRC1_TOGGLE() TOGGLE_PIN(DDRC, 1)
	#define DDRC2_TOGGLE() TOGGLE_PIN(DDRC, 2)
	#define DDRC3_TOGGLE() TOGGLE_PIN(DDRC, 3)
	#define DDRC4_TOGGLE() TOGGLE_PIN(DDRC, 4)
	#define DDRC5_TOGGLE() TOGGLE_PIN(DDRC, 5)
	#define DDRC6_TOGGLE() TOGGLE_PIN(DDRC, 6)
	#define DDRC7_TOGGLE() TOGGLE_PIN(DDRC, 7)

	#define PORTC0_TOGGLE() TOGGLE_PIN(PORTC, 0)
	#define PORTC1_TOGGLE() TOGGLE_PIN(PORTC, 1)
	#define PORTC2_TOGGLE() TOGGLE_PIN(PORTC, 2)
	#define PORTC3_TOGGLE() TOGGLE_PIN(PORTC, 3)
	#define PORTC4_TOGGLE() TOGGLE_PIN(PORTC, 4)
	#define PORTC5_TOGGLE() TOGGLE_PIN(PORTC, 5)
	#define PORTC6_TOGGLE() TOGGLE_PIN(PORTC, 6)
	#define PORTC7_TOGGLE() TOGGLE_PIN(PORTC, 7)

	#define PINC0_TOGGLE() TOGGLE_PIN(PINC, 0)
	#define PINC1_TOGGLE() TOGGLE_PIN(PINC, 1)
	#define PINC2_TOGGLE() TOGGLE_PIN(PINC, 2)
	#define PINC3_TOGGLE() TOGGLE_PIN(PINC, 3)
	#define PINC4_TOGGLE() TOGGLE_PIN(PINC, 4)
	#define PINC5_TOGGLE() TOGGLE_PIN(PINC, 5)
	#define PINC6_TOGGLE() TOGGLE_PIN(PINC, 6)
	#define PINC7_TOGGLE() TOGGLE_PIN(PINC, 7)


	#define PC0_CHECK() PIN_IN_CHECK(C, 0)
	#define PC1_CHECK() PIN_IN_CHECK(C, 1)
	#define PC2_CHECK() PIN_IN_CHECK(C, 2)
	#define PC3_CHECK() PIN_IN_CHECK(C, 3)
	#define PC4_CHECK() PIN_IN_CHECK(C, 4)
	#define PC5_CHECK() PIN_IN_CHECK(C, 5)
	#define PC6_CHECK() PIN_IN_CHECK(C, 6)
	#define PC7_CHECK() PIN_IN_CHECK(C, 7)

	#define PC0_IS_SET() PIN_IN_CHECK_SET(C, 0)
	#define PC1_IS_SET() PIN_IN_CHECK_SET(C, 1)
	#define PC2_IS_SET() PIN_IN_CHECK_SET(C, 2)
	#define PC3_IS_SET() PIN_IN_CHECK_SET(C, 3)
	#define PC4_IS_SET() PIN_IN_CHECK_SET(C, 4)
	#define PC5_IS_SET() PIN_IN_CHECK_SET(C, 5)
	#define PC6_IS_SET() PIN_IN_CHECK_SET(C, 6)
	#define PC7_IS_SET() PIN_IN_CHECK_SET(C, 7)

	#define PC0_IS_UNSET() PIN_IN_CHECK_UNSET(C, 0)
	#define PC1_IS_UNSET() PIN_IN_CHECK_UNSET(C, 1)
	#define PC2_IS_UNSET() PIN_IN_CHECK_UNSET(C, 2)
	#define PC3_IS_UNSET() PIN_IN_CHECK_UNSET(C, 3)
	#define PC4_IS_UNSET() PIN_IN_CHECK_UNSET(C, 4)
	#define PC5_IS_UNSET() PIN_IN_CHECK_UNSET(C, 5)
	#define PC6_IS_UNSET() PIN_IN_CHECK_UNSET(C, 6)
	#define PC7_IS_UNSET() PIN_IN_CHECK_UNSET(C, 7)


	#define PC0_SET() PIN_OUT_SET(C, 0)
	#define PC1_SET() PIN_OUT_SET(C, 1)
	#define PC2_SET() PIN_OUT_SET(C, 2)
	#define PC3_SET() PIN_OUT_SET(C, 3)
	#define PC4_SET() PIN_OUT_SET(C, 4)
	#define PC5_SET() PIN_OUT_SET(C, 5)
	#define PC6_SET() PIN_OUT_SET(C, 6)
	#define PC7_SET() PIN_OUT_SET(C, 7)

	#define PC0_CLEAR() PIN_OUT_CLEAR(C, 0)
	#define PC1_CLEAR() PIN_OUT_CLEAR(C, 1)
	#define PC2_CLEAR() PIN_OUT_CLEAR(C, 2)
	#define PC3_CLEAR() PIN_OUT_CLEAR(C, 3)
	#define PC4_CLEAR() PIN_OUT_CLEAR(C, 4)
	#define PC5_CLEAR() PIN_OUT_CLEAR(C, 5)
	#define PC6_CLEAR() PIN_OUT_CLEAR(C, 6)
	#define PC7_CLEAR() PIN_OUT_CLEAR(C, 7)

	#define PC0_TOGGLE() PIN_OUT_TOGGLE(C, 0)
	#define PC1_TOGGLE() PIN_OUT_TOGGLE(C, 1)
	#define PC2_TOGGLE() PIN_OUT_TOGGLE(C, 2)
	#define PC3_TOGGLE() PIN_OUT_TOGGLE(C, 3)
	#define PC4_TOGGLE() PIN_OUT_TOGGLE(C, 4)
	#define PC5_TOGGLE() PIN_OUT_TOGGLE(C, 5)
	#define PC6_TOGGLE() PIN_OUT_TOGGLE(C, 6)
	#define PC7_TOGGLE() PIN_OUT_TOGGLE(C, 7)

	#define PC0_RESET() PIN_RESET(C, 0)
	#define PC1_RESET() PIN_RESET(C, 1)
	#define PC2_RESET() PIN_RESET(C, 2)
	#define PC3_RESET() PIN_RESET(C, 3)
	#define PC4_RESET() PIN_RESET(C, 4)
	#define PC5_RESET() PIN_RESET(C, 5)
	#define PC6_RESET() PIN_RESET(C, 6)
	#define PC7_RESET() PIN_RESET(C, 7)

	#define PC0_GET_MODE() PIN_GET_MODE(C, 0)
	#define PC1_GET_MODE() PIN_GET_MODE(C, 1)
	#define PC2_GET_MODE() PIN_GET_MODE(C, 2)
	#define PC3_GET_MODE() PIN_GET_MODE(C, 3)
	#define PC4_GET_MODE() PIN_GET_MODE(C, 4)
	#define PC5_GET_MODE() PIN_GET_MODE(C, 5)
	#define PC6_GET_MODE() PIN_GET_MODE(C, 6)
	#define PC7_GET_MODE() PIN_GET_MODE(C, 7)

	#define PC0_SET_MODE(mode) PIN_SET_MODE(C, 0, mode)
	#define PC1_SET_MODE(mode) PIN_SET_MODE(C, 1, mode)
	#define PC2_SET_MODE(mode) PIN_SET_MODE(C, 2, mode)
	#define PC3_SET_MODE(mode) PIN_SET_MODE(C, 3, mode)
	#define PC4_SET_MODE(mode) PIN_SET_MODE(C, 4, mode)
	#define PC5_SET_MODE(mode) PIN_SET_MODE(C, 5, mode)
	#define PC6_SET_MODE(mode) PIN_SET_MODE(C, 6, mode)
	#define PC7_SET_MODE(mode) PIN_SET_MODE(C, 7, mode)

#endif


#ifdef PORTD

	#define DDRD_0 PIN_REF(DDRD, 0)
	#define DDRD_1 PIN_REF(DDRD, 1)
	#define DDRD_2 PIN_REF(DDRD, 2)
	#define DDRD_3 PIN_REF(DDRD, 3)
	#define DDRD_4 PIN_REF(DDRD, 4)
	#define DDRD_5 PIN_REF(DDRD, 5)
	#define DDRD_6 PIN_REF(DDRD, 6)
	#define DDRD_7 PIN_REF(DDRD, 7)

	#define PORTD_0 PIN_REF(PORTD, 0)
	#define PORTD_1 PIN_REF(PORTD, 1)
	#define PORTD_2 PIN_REF(PORTD, 2)
	#define PORTD_3 PIN_REF(PORTD, 3)
	#define PORTD_4 PIN_REF(PORTD, 4)
	#define PORTD_5 PIN_REF(PORTD, 5)
	#define PORTD_6 PIN_REF(PORTD, 6)
	#define PORTD_7 PIN_REF(PORTD, 7)

	#define PIND_0 PIN_REF(PIND, 0)
	#define PIND_1 PIN_REF(PIND, 1)
	#define PIND_2 PIN_REF(PIND, 2)
	#define PIND_3 PIN_REF(PIND, 3)
	#define PIND_4 PIN_REF(PIND, 4)
	#define PIND_5 PIN_REF(PIND, 5)
	#define PIND_6 PIN_REF(PIND, 6)
	#define PIND_7 PIN_REF(PIND, 7)


	#define DDRD0_PAIR() PORT_PIN_PAIR(DDRD, 0)
	#define DDRD1_PAIR() PORT_PIN_PAIR(DDRD, 1)
	#define DDRD2_PAIR() PORT_PIN_PAIR(DDRD, 2)
	#define DDRD3_PAIR() PORT_PIN_PAIR(DDRD, 3)
	#define DDRD4_PAIR() PORT_PIN_PAIR(DDRD, 4)
	#define DDRD5_PAIR() PORT_PIN_PAIR(DDRD, 5)
	#define DDRD6_PAIR() PORT_PIN_PAIR(DDRD, 6)
	#define DDRD7_PAIR() PORT_PIN_PAIR(DDRD, 7)

	#define PORTD0_PAIR() PORT_PIN_PAIR(PORTD, 0)
	#define PORTD1_PAIR() PORT_PIN_PAIR(PORTD, 1)
	#define PORTD2_PAIR() PORT_PIN_PAIR(PORTD, 2)
	#define PORTD3_PAIR() PORT_PIN_PAIR(PORTD, 3)
	#define PORTD4_PAIR() PORT_PIN_PAIR(PORTD, 4)
	#define PORTD5_PAIR() PORT_PIN_PAIR(PORTD, 5)
	#define PORTD6_PAIR() PORT_PIN_PAIR(PORTD, 6)
	#define PORTD7_PAIR() PORT_PIN_PAIR(PORTD, 7)

	#define PIND0_PAIR() PORT_PIN_PAIR(PIND, 0)
	#define PIND1_PAIR() PORT_PIN_PAIR(PIND, 1)
	#define PIND2_PAIR() PORT_PIN_PAIR(PIND, 2)
	#define PIND3_PAIR() PORT_PIN_PAIR(PIND, 3)
	#define PIND4_PAIR() PORT_PIN_PAIR(PIND, 4)
	#define PIND5_PAIR() PORT_PIN_PAIR(PIND, 5)
	#define PIND6_PAIR() PORT_PIN_PAIR(PIND, 6)
	#define PIND7_PAIR() PORT_PIN_PAIR(PIND, 7)


	#define DDRD0_CHECK() CHECK_PIN(DDRD, 0)
	#define DDRD1_CHECK() CHECK_PIN(DDRD, 1)
	#define DDRD2_CHECK() CHECK_PIN(DDRD, 2)
	#define DDRD3_CHECK() CHECK_PIN(DDRD, 3)
	#define DDRD4_CHECK() CHECK_PIN(DDRD, 4)
	#define DDRD5_CHECK() CHECK_PIN(DDRD, 5)
	#define DDRD6_CHECK() CHECK_PIN(DDRD, 6)
	#define DDRD7_CHECK() CHECK_PIN(DDRD, 7)

	#define PORTD0_CHECK() CHECK_PIN(PORTD, 0)
	#define PORTD1_CHECK() CHECK_PIN(PORTD, 1)
	#define PORTD2_CHECK() CHECK_PIN(PORTD, 2)
	#define PORTD3_CHECK() CHECK_PIN(PORTD, 3)
	#define PORTD4_CHECK() CHECK_PIN(PORTD, 4)
	#define PORTD5_CHECK() CHECK_PIN(PORTD, 5)
	#define PORTD6_CHECK() CHECK_PIN(PORTD, 6)
	#define PORTD7_CHECK() CHECK_PIN(PORTD, 7)

	#define PIND0_CHECK() CHECK_PIN(PIND, 0)
	#define PIND1_CHECK() CHECK_PIN(PIND, 1)
	#define PIND2_CHECK() CHECK_PIN(PIND, 2)
	#define PIND3_CHECK() CHECK_PIN(PIND, 3)
	#define PIND4_CHECK() CHECK_PIN(PIND, 4)
	#define PIND5_CHECK() CHECK_PIN(PIND, 5)
	#define PIND6_CHECK() CHECK_PIN(PIND, 6)
	#define PIND7_CHECK() CHECK_PIN(PIND, 7)


	#define DDRD0_IS_SET() CHECK_SET_PIN(DDRD, 0)
	#define DDRD1_IS_SET() CHECK_SET_PIN(DDRD, 1)
	#define DDRD2_IS_SET() CHECK_SET_PIN(DDRD, 2)
	#define DDRD3_IS_SET() CHECK_SET_PIN(DDRD, 3)
	#define DDRD4_IS_SET() CHECK_SET_PIN(DDRD, 4)
	#define DDRD5_IS_SET() CHECK_SET_PIN(DDRD, 5)
	#define DDRD6_IS_SET() CHECK_SET_PIN(DDRD, 6)
	#define DDRD7_IS_SET() CHECK_SET_PIN(DDRD, 7)

	#define PORTD0_IS_SET() CHECK_SET_PIN(PORTD, 0)
	#define PORTD1_IS_SET() CHECK_SET_PIN(PORTD, 1)
	#define PORTD2_IS_SET() CHECK_SET_PIN(PORTD, 2)
	#define PORTD3_IS_SET() CHECK_SET_PIN(PORTD, 3)
	#define PORTD4_IS_SET() CHECK_SET_PIN(PORTD, 4)
	#define PORTD5_IS_SET() CHECK_SET_PIN(PORTD, 5)
	#define PORTD6_IS_SET() CHECK_SET_PIN(PORTD, 6)
	#define PORTD7_IS_SET() CHECK_SET_PIN(PORTD, 7)

	#define PIND0_IS_SET() CHECK_SET_PIN(PIND, 0)
	#define PIND1_IS_SET() CHECK_SET_PIN(PIND, 1)
	#define PIND2_IS_SET() CHECK_SET_PIN(PIND, 2)
	#define PIND3_IS_SET() CHECK_SET_PIN(PIND, 3)
	#define PIND4_IS_SET() CHECK_SET_PIN(PIND, 4)
	#define PIND5_IS_SET() CHECK_SET_PIN(PIND, 5)
	#define PIND6_IS_SET() CHECK_SET_PIN(PIND, 6)
	#define PIND7_IS_SET() CHECK_SET_PIN(PIND, 7)


	#define DDRD0_IS_UNSET() CHECK_UNSET_PIN(DDRD, 0)
	#define DDRD1_IS_UNSET() CHECK_UNSET_PIN(DDRD, 1)
	#define DDRD2_IS_UNSET() CHECK_UNSET_PIN(DDRD, 2)
	#define DDRD3_IS_UNSET() CHECK_UNSET_PIN(DDRD, 3)
	#define DDRD4_IS_UNSET() CHECK_UNSET_PIN(DDRD, 4)
	#define DDRD5_IS_UNSET() CHECK_UNSET_PIN(DDRD, 5)
	#define DDRD6_IS_UNSET() CHECK_UNSET_PIN(DDRD, 6)
	#define DDRD7_IS_UNSET() CHECK_UNSET_PIN(DDRD, 7)

	#define PORTD0_IS_UNSET() CHECK_UNSET_PIN(PORTD, 0)
	#define PORTD1_IS_UNSET() CHECK_UNSET_PIN(PORTD, 1)
	#define PORTD2_IS_UNSET() CHECK_UNSET_PIN(PORTD, 2)
	#define PORTD3_IS_UNSET() CHECK_UNSET_PIN(PORTD, 3)
	#define PORTD4_IS_UNSET() CHECK_UNSET_PIN(PORTD, 4)
	#define PORTD5_IS_UNSET() CHECK_UNSET_PIN(PORTD, 5)
	#define PORTD6_IS_UNSET() CHECK_UNSET_PIN(PORTD, 6)
	#define PORTD7_IS_UNSET() CHECK_UNSET_PIN(PORTD, 7)

	#define PIND0_IS_UNSET() CHECK_UNSET_PIN(PIND, 0)
	#define PIND1_IS_UNSET() CHECK_UNSET_PIN(PIND, 1)
	#define PIND2_IS_UNSET() CHECK_UNSET_PIN(PIND, 2)
	#define PIND3_IS_UNSET() CHECK_UNSET_PIN(PIND, 3)
	#define PIND4_IS_UNSET() CHECK_UNSET_PIN(PIND, 4)
	#define PIND5_IS_UNSET() CHECK_UNSET_PIN(PIND, 5)
	#define PIND6_IS_UNSET() CHECK_UNSET_PIN(PIND, 6)
	#define PIND7_IS_UNSET() CHECK_UNSET_PIN(PIND, 7)


	#define DDRD0_SET() SET_PIN(DDRD, 0)
	#define DDRD1_SET() SET_PIN(DDRD, 1)
	#define DDRD2_SET() SET_PIN(DDRD, 2)
	#define DDRD3_SET() SET_PIN(DDRD, 3)
	#define DDRD4_SET() SET_PIN(DDRD, 4)
	#define DDRD5_SET() SET_PIN(DDRD, 5)
	#define DDRD6_SET() SET_PIN(DDRD, 6)
	#define DDRD7_SET() SET_PIN(DDRD, 7)

	#define PORTD0_SET() SET_PIN(PORTD, 0)
	#define PORTD1_SET() SET_PIN(PORTD, 1)
	#define PORTD2_SET() SET_PIN(PORTD, 2)
	#define PORTD3_SET() SET_PIN(PORTD, 3)
	#define PORTD4_SET() SET_PIN(PORTD, 4)
	#define PORTD5_SET() SET_PIN(PORTD, 5)
	#define PORTD6_SET() SET_PIN(PORTD, 6)
	#define PORTD7_SET() SET_PIN(PORTD, 7)

	#define PIND0_SET() SET_PIN(PIND, 0)
	#define PIND1_SET() SET_PIN(PIND, 1)
	#define PIND2_SET() SET_PIN(PIND, 2)
	#define PIND3_SET() SET_PIN(PIND, 3)
	#define PIND4_SET() SET_PIN(PIND, 4)
	#define PIND5_SET() SET_PIN(PIND, 5)
	#define PIND6_SET() SET_PIN(PIND, 6)
	#define PIND7_SET() SET_PIN(PIND, 7)


	#define DDRD0_CLEAR() CLEAR_PIN(DDRD, 0)
	#define DDRD1_CLEAR() CLEAR_PIN(DDRD, 1)
	#define DDRD2_CLEAR() CLEAR_PIN(DDRD, 2)
	#define DDRD3_CLEAR() CLEAR_PIN(DDRD, 3)
	#define DDRD4_CLEAR() CLEAR_PIN(DDRD, 4)
	#define DDRD5_CLEAR() CLEAR_PIN(DDRD, 5)
	#define DDRD6_CLEAR() CLEAR_PIN(DDRD, 6)
	#define DDRD7_CLEAR() CLEAR_PIN(DDRD, 7)

	#define PORTD0_CLEAR() CLEAR_PIN(PORTD, 0)
	#define PORTD1_CLEAR() CLEAR_PIN(PORTD, 1)
	#define PORTD2_CLEAR() CLEAR_PIN(PORTD, 2)
	#define PORTD3_CLEAR() CLEAR_PIN(PORTD, 3)
	#define PORTD4_CLEAR() CLEAR_PIN(PORTD, 4)
	#define PORTD5_CLEAR() CLEAR_PIN(PORTD, 5)
	#define PORTD6_CLEAR() CLEAR_PIN(PORTD, 6)
	#define PORTD7_CLEAR() CLEAR_PIN(PORTD, 7)

	#define PIND0_CLEAR() CLEAR_PIN(PIND, 0)
	#define PIND1_CLEAR() CLEAR_PIN(PIND, 1)
	#define PIND2_CLEAR() CLEAR_PIN(PIND, 2)
	#define PIND3_CLEAR() CLEAR_PIN(PIND, 3)
	#define PIND4_CLEAR() CLEAR_PIN(PIND, 4)
	#define PIND5_CLEAR() CLEAR_PIN(PIND, 5)
	#define PIND6_CLEAR() CLEAR_PIN(PIND, 6)
	#define PIND7_CLEAR() CLEAR_PIN(PIND, 7)


	#define DDRD0_TOGGLE() TOGGLE_PIN(DDRD, 0)
	#define DDRD1_TOGGLE() TOGGLE_PIN(DDRD, 1)
	#define DDRD2_TOGGLE() TOGGLE_PIN(DDRD, 2)
	#define DDRD3_TOGGLE() TOGGLE_PIN(DDRD, 3)
	#define DDRD4_TOGGLE() TOGGLE_PIN(DDRD, 4)
	#define DDRD5_TOGGLE() TOGGLE_PIN(DDRD, 5)
	#define DDRD6_TOGGLE() TOGGLE_PIN(DDRD, 6)
	#define DDRD7_TOGGLE() TOGGLE_PIN(DDRD, 7)

	#define PORTD0_TOGGLE() TOGGLE_PIN(PORTD, 0)
	#define PORTD1_TOGGLE() TOGGLE_PIN(PORTD, 1)
	#define PORTD2_TOGGLE() TOGGLE_PIN(PORTD, 2)
	#define PORTD3_TOGGLE() TOGGLE_PIN(PORTD, 3)
	#define PORTD4_TOGGLE() TOGGLE_PIN(PORTD, 4)
	#define PORTD5_TOGGLE() TOGGLE_PIN(PORTD, 5)
	#define PORTD6_TOGGLE() TOGGLE_PIN(PORTD, 6)
	#define PORTD7_TOGGLE() TOGGLE_PIN(PORTD, 7)

	#define PIND0_TOGGLE() TOGGLE_PIN(PIND, 0)
	#define PIND1_TOGGLE() TOGGLE_PIN(PIND, 1)
	#define PIND2_TOGGLE() TOGGLE_PIN(PIND, 2)
	#define PIND3_TOGGLE() TOGGLE_PIN(PIND, 3)
	#define PIND4_TOGGLE() TOGGLE_PIN(PIND, 4)
	#define PIND5_TOGGLE() TOGGLE_PIN(PIND, 5)
	#define PIND6_TOGGLE() TOGGLE_PIN(PIND, 6)
	#define PIND7_TOGGLE() TOGGLE_PIN(PIND, 7)


	#define PD0_CHECK() PIN_IN_CHECK(D, 0)
	#define PD1_CHECK() PIN_IN_CHECK(D, 1)
	#define PD2_CHECK() PIN_IN_CHECK(D, 2)
	#define PD3_CHECK() PIN_IN_CHECK(D, 3)
	#define PD4_CHECK() PIN_IN_CHECK(D, 4)
	#define PD5_CHECK() PIN_IN_CHECK(D, 5)
	#define PD6_CHECK() PIN_IN_CHECK(D, 6)
	#define PD7_CHECK() PIN_IN_CHECK(D, 7)

	#define PD0_IS_SET() PIN_IN_CHECK_SET(D, 0)
	#define PD1_IS_SET() PIN_IN_CHECK_SET(D, 1)
	#define PD2_IS_SET() PIN_IN_CHECK_SET(D, 2)
	#define PD3_IS_SET() PIN_IN_CHECK_SET(D, 3)
	#define PD4_IS_SET() PIN_IN_CHECK_SET(D, 4)
	#define PD5_IS_SET() PIN_IN_CHECK_SET(D, 5)
	#define PD6_IS_SET() PIN_IN_CHECK_SET(D, 6)
	#define PD7_IS_SET() PIN_IN_CHECK_SET(D, 7)

	#define PD0_IS_UNSET() PIN_IN_CHECK_UNSET(D, 0)
	#define PD1_IS_UNSET() PIN_IN_CHECK_UNSET(D, 1)
	#define PD2_IS_UNSET() PIN_IN_CHECK_UNSET(D, 2)
	#define PD3_IS_UNSET() PIN_IN_CHECK_UNSET(D, 3)
	#define PD4_IS_UNSET() PIN_IN_CHECK_UNSET(D, 4)
	#define PD5_IS_UNSET() PIN_IN_CHECK_UNSET(D, 5)
	#define PD6_IS_UNSET() PIN_IN_CHECK_UNSET(D, 6)
	#define PD7_IS_UNSET() PIN_IN_CHECK_UNSET(D, 7)


	#define PD0_SET() PIN_OUT_SET(D, 0)
	#define PD1_SET() PIN_OUT_SET(D, 1)
	#define PD2_SET() PIN_OUT_SET(D, 2)
	#define PD3_SET() PIN_OUT_SET(D, 3)
	#define PD4_SET() PIN_OUT_SET(D, 4)
	#define PD5_SET() PIN_OUT_SET(D, 5)
	#define PD6_SET() PIN_OUT_SET(D, 6)
	#define PD7_SET() PIN_OUT_SET(D, 7)

	#define PD0_CLEAR() PIN_OUT_CLEAR(D, 0)
	#define PD1_CLEAR() PIN_OUT_CLEAR(D, 1)
	#define PD2_CLEAR() PIN_OUT_CLEAR(D, 2)
	#define PD3_CLEAR() PIN_OUT_CLEAR(D, 3)
	#define PD4_CLEAR() PIN_OUT_CLEAR(D, 4)
	#define PD5_CLEAR() PIN_OUT_CLEAR(D, 5)
	#define PD6_CLEAR() PIN_OUT_CLEAR(D, 6)
	#define PD7_CLEAR() PIN_OUT_CLEAR(D, 7)

	#define PD0_TOGGLE() PIN_OUT_TOGGLE(D, 0)
	#define PD1_TOGGLE() PIN_OUT_TOGGLE(D, 1)
	#define PD2_TOGGLE() PIN_OUT_TOGGLE(D, 2)
	#define PD3_TOGGLE() PIN_OUT_TOGGLE(D, 3)
	#define PD4_TOGGLE() PIN_OUT_TOGGLE(D, 4)
	#define PD5_TOGGLE() PIN_OUT_TOGGLE(D, 5)
	#define PD6_TOGGLE() PIN_OUT_TOGGLE(D, 6)
	#define PD7_TOGGLE() PIN_OUT_TOGGLE(D, 7)

	#define PD0_RESET() PIN_RESET(D, 0)
	#define PD1_RESET() PIN_RESET(D, 1)
	#define PD2_RESET() PIN_RESET(D, 2)
	#define PD3_RESET() PIN_RESET(D, 3)
	#define PD4_RESET() PIN_RESET(D, 4)
	#define PD5_RESET() PIN_RESET(D, 5)
	#define PD6_RESET() PIN_RESET(D, 6)
	#define PD7_RESET() PIN_RESET(D, 7)

	#define PD0_GET_MODE() PIN_GET_MODE(D, 0)
	#define PD1_GET_MODE() PIN_GET_MODE(D, 1)
	#define PD2_GET_MODE() PIN_GET_MODE(D, 2)
	#define PD3_GET_MODE() PIN_GET_MODE(D, 3)
	#define PD4_GET_MODE() PIN_GET_MODE(D, 4)
	#define PD5_GET_MODE() PIN_GET_MODE(D, 5)
	#define PD6_GET_MODE() PIN_GET_MODE(D, 6)
	#define PD7_GET_MODE() PIN_GET_MODE(D, 7)

	#define PD0_SET_MODE(mode) PIN_SET_MODE(D, 0, mode)
	#define PD1_SET_MODE(mode) PIN_SET_MODE(D, 1, mode)
	#define PD2_SET_MODE(mode) PIN_SET_MODE(D, 2, mode)
	#define PD3_SET_MODE(mode) PIN_SET_MODE(D, 3, mode)
	#define PD4_SET_MODE(mode) PIN_SET_MODE(D, 4, mode)
	#define PD5_SET_MODE(mode) PIN_SET_MODE(D, 5, mode)
	#define PD6_SET_MODE(mode) PIN_SET_MODE(D, 6, mode)
	#define PD7_SET_MODE(mode) PIN_SET_MODE(D, 7, mode)

#endif


#ifdef PORTE

	#define DDRE_0 PIN_REF(DDRE, 0)
	#define DDRE_1 PIN_REF(DDRE, 1)
	#define DDRE_2 PIN_REF(DDRE, 2)
	#define DDRE_3 PIN_REF(DDRE, 3)
	#define DDRE_4 PIN_REF(DDRE, 4)
	#define DDRE_5 PIN_REF(DDRE, 5)
	#define DDRE_6 PIN_REF(DDRE, 6)
	#define DDRE_7 PIN_REF(DDRE, 7)

	#define PORTE_0 PIN_REF(PORTE, 0)
	#define PORTE_1 PIN_REF(PORTE, 1)
	#define PORTE_2 PIN_REF(PORTE, 2)
	#define PORTE_3 PIN_REF(PORTE, 3)
	#define PORTE_4 PIN_REF(PORTE, 4)
	#define PORTE_5 PIN_REF(PORTE, 5)
	#define PORTE_6 PIN_REF(PORTE, 6)
	#define PORTE_7 PIN_REF(PORTE, 7)

	#define PINE_0 PIN_REF(PINE, 0)
	#define PINE_1 PIN_REF(PINE, 1)
	#define PINE_2 PIN_REF(PINE, 2)
	#define PINE_3 PIN_REF(PINE, 3)
	#define PINE_4 PIN_REF(PINE, 4)
	#define PINE_5 PIN_REF(PINE, 5)
	#define PINE_6 PIN_REF(PINE, 6)
	#define PINE_7 PIN_REF(PINE, 7)


	#define DDRE0_PAIR() PORT_PIN_PAIR(DDRE, 0)
	#define DDRE1_PAIR() PORT_PIN_PAIR(DDRE, 1)
	#define DDRE2_PAIR() PORT_PIN_PAIR(DDRE, 2)
	#define DDRE3_PAIR() PORT_PIN_PAIR(DDRE, 3)
	#define DDRE4_PAIR() PORT_PIN_PAIR(DDRE, 4)
	#define DDRE5_PAIR() PORT_PIN_PAIR(DDRE, 5)
	#define DDRE6_PAIR() PORT_PIN_PAIR(DDRE, 6)
	#define DDRE7_PAIR() PORT_PIN_PAIR(DDRE, 7)

	#define PORTE0_PAIR() PORT_PIN_PAIR(PORTE, 0)
	#define PORTE1_PAIR() PORT_PIN_PAIR(PORTE, 1)
	#define PORTE2_PAIR() PORT_PIN_PAIR(PORTE, 2)
	#define PORTE3_PAIR() PORT_PIN_PAIR(PORTE, 3)
	#define PORTE4_PAIR() PORT_PIN_PAIR(PORTE, 4)
	#define PORTE5_PAIR() PORT_PIN_PAIR(PORTE, 5)
	#define PORTE6_PAIR() PORT_PIN_PAIR(PORTE, 6)
	#define PORTE7_PAIR() PORT_PIN_PAIR(PORTE, 7)

	#define PINE0_PAIR() PORT_PIN_PAIR(PINE, 0)
	#define PINE1_PAIR() PORT_PIN_PAIR(PINE, 1)
	#define PINE2_PAIR() PORT_PIN_PAIR(PINE, 2)
	#define PINE3_PAIR() PORT_PIN_PAIR(PINE, 3)
	#define PINE4_PAIR() PORT_PIN_PAIR(PINE, 4)
	#define PINE5_PAIR() PORT_PIN_PAIR(PINE, 5)
	#define PINE6_PAIR() PORT_PIN_PAIR(PINE, 6)
	#define PINE7_PAIR() PORT_PIN_PAIR(PINE, 7)


	#define DDRE0_CHECK() CHECK_PIN(DDRE, 0)
	#define DDRE1_CHECK() CHECK_PIN(DDRE, 1)
	#define DDRE2_CHECK() CHECK_PIN(DDRE, 2)
	#define DDRE3_CHECK() CHECK_PIN(DDRE, 3)
	#define DDRE4_CHECK() CHECK_PIN(DDRE, 4)
	#define DDRE5_CHECK() CHECK_PIN(DDRE, 5)
	#define DDRE6_CHECK() CHECK_PIN(DDRE, 6)
	#define DDRE7_CHECK() CHECK_PIN(DDRE, 7)

	#define PORTE0_CHECK() CHECK_PIN(PORTE, 0)
	#define PORTE1_CHECK() CHECK_PIN(PORTE, 1)
	#define PORTE2_CHECK() CHECK_PIN(PORTE, 2)
	#define PORTE3_CHECK() CHECK_PIN(PORTE, 3)
	#define PORTE4_CHECK() CHECK_PIN(PORTE, 4)
	#define PORTE5_CHECK() CHECK_PIN(PORTE, 5)
	#define PORTE6_CHECK() CHECK_PIN(PORTE, 6)
	#define PORTE7_CHECK() CHECK_PIN(PORTE, 7)

	#define PINE0_CHECK() CHECK_PIN(PINE, 0)
	#define PINE1_CHECK() CHECK_PIN(PINE, 1)
	#define PINE2_CHECK() CHECK_PIN(PINE, 2)
	#define PINE3_CHECK() CHECK_PIN(PINE, 3)
	#define PINE4_CHECK() CHECK_PIN(PINE, 4)
	#define PINE5_CHECK() CHECK_PIN(PINE, 5)
	#define PINE6_CHECK() CHECK_PIN(PINE, 6)
	#define PINE7_CHECK() CHECK_PIN(PINE, 7)


	#define DDRE0_IS_SET() CHECK_SET_PIN(DDRE, 0)
	#define DDRE1_IS_SET() CHECK_SET_PIN(DDRE, 1)
	#define DDRE2_IS_SET() CHECK_SET_PIN(DDRE, 2)
	#define DDRE3_IS_SET() CHECK_SET_PIN(DDRE, 3)
	#define DDRE4_IS_SET() CHECK_SET_PIN(DDRE, 4)
	#define DDRE5_IS_SET() CHECK_SET_PIN(DDRE, 5)
	#define DDRE6_IS_SET() CHECK_SET_PIN(DDRE, 6)
	#define DDRE7_IS_SET() CHECK_SET_PIN(DDRE, 7)

	#define PORTE0_IS_SET() CHECK_SET_PIN(PORTE, 0)
	#define PORTE1_IS_SET() CHECK_SET_PIN(PORTE, 1)
	#define PORTE2_IS_SET() CHECK_SET_PIN(PORTE, 2)
	#define PORTE3_IS_SET() CHECK_SET_PIN(PORTE, 3)
	#define PORTE4_IS_SET() CHECK_SET_PIN(PORTE, 4)
	#define PORTE5_IS_SET() CHECK_SET_PIN(PORTE, 5)
	#define PORTE6_IS_SET() CHECK_SET_PIN(PORTE, 6)
	#define PORTE7_IS_SET() CHECK_SET_PIN(PORTE, 7)

	#define PINE0_IS_SET() CHECK_SET_PIN(PINE, 0)
	#define PINE1_IS_SET() CHECK_SET_PIN(PINE, 1)
	#define PINE2_IS_SET() CHECK_SET_PIN(PINE, 2)
	#define PINE3_IS_SET() CHECK_SET_PIN(PINE, 3)
	#define PINE4_IS_SET() CHECK_SET_PIN(PINE, 4)
	#define PINE5_IS_SET() CHECK_SET_PIN(PINE, 5)
	#define PINE6_IS_SET() CHECK_SET_PIN(PINE, 6)
	#define PINE7_IS_SET() CHECK_SET_PIN(PINE, 7)


	#define DDRE0_IS_UNSET() CHECK_UNSET_PIN(DDRE, 0)
	#define DDRE1_IS_UNSET() CHECK_UNSET_PIN(DDRE, 1)
	#define DDRE2_IS_UNSET() CHECK_UNSET_PIN(DDRE, 2)
	#define DDRE3_IS_UNSET() CHECK_UNSET_PIN(DDRE, 3)
	#define DDRE4_IS_UNSET() CHECK_UNSET_PIN(DDRE, 4)
	#define DDRE5_IS_UNSET() CHECK_UNSET_PIN(DDRE, 5)
	#define DDRE6_IS_UNSET() CHECK_UNSET_PIN(DDRE, 6)
	#define DDRE7_IS_UNSET() CHECK_UNSET_PIN(DDRE, 7)

	#define PORTE0_IS_UNSET() CHECK_UNSET_PIN(PORTE, 0)
	#define PORTE1_IS_UNSET() CHECK_UNSET_PIN(PORTE, 1)
	#define PORTE2_IS_UNSET() CHECK_UNSET_PIN(PORTE, 2)
	#define PORTE3_IS_UNSET() CHECK_UNSET_PIN(PORTE, 3)
	#define PORTE4_IS_UNSET() CHECK_UNSET_PIN(PORTE, 4)
	#define PORTE5_IS_UNSET() CHECK_UNSET_PIN(PORTE, 5)
	#define PORTE6_IS_UNSET() CHECK_UNSET_PIN(PORTE, 6)
	#define PORTE7_IS_UNSET() CHECK_UNSET_PIN(PORTE, 7)

	#define PINE0_IS_UNSET() CHECK_UNSET_PIN(PINE, 0)
	#define PINE1_IS_UNSET() CHECK_UNSET_PIN(PINE, 1)
	#define PINE2_IS_UNSET() CHECK_UNSET_PIN(PINE, 2)
	#define PINE3_IS_UNSET() CHECK_UNSET_PIN(PINE, 3)
	#define PINE4_IS_UNSET() CHECK_UNSET_PIN(PINE, 4)
	#define PINE5_IS_UNSET() CHECK_UNSET_PIN(PINE, 5)
	#define PINE6_IS_UNSET() CHECK_UNSET_PIN(PINE, 6)
	#define PINE7_IS_UNSET() CHECK_UNSET_PIN(PINE, 7)


	#define DDRE0_SET() SET_PIN(DDRE, 0)
	#define DDRE1_SET() SET_PIN(DDRE, 1)
	#define DDRE2_SET() SET_PIN(DDRE, 2)
	#define DDRE3_SET() SET_PIN(DDRE, 3)
	#define DDRE4_SET() SET_PIN(DDRE, 4)
	#define DDRE5_SET() SET_PIN(DDRE, 5)
	#define DDRE6_SET() SET_PIN(DDRE, 6)
	#define DDRE7_SET() SET_PIN(DDRE, 7)

	#define PORTE0_SET() SET_PIN(PORTE, 0)
	#define PORTE1_SET() SET_PIN(PORTE, 1)
	#define PORTE2_SET() SET_PIN(PORTE, 2)
	#define PORTE3_SET() SET_PIN(PORTE, 3)
	#define PORTE4_SET() SET_PIN(PORTE, 4)
	#define PORTE5_SET() SET_PIN(PORTE, 5)
	#define PORTE6_SET() SET_PIN(PORTE, 6)
	#define PORTE7_SET() SET_PIN(PORTE, 7)

	#define PINE0_SET() SET_PIN(PINE, 0)
	#define PINE1_SET() SET_PIN(PINE, 1)
	#define PINE2_SET() SET_PIN(PINE, 2)
	#define PINE3_SET() SET_PIN(PINE, 3)
	#define PINE4_SET() SET_PIN(PINE, 4)
	#define PINE5_SET() SET_PIN(PINE, 5)
	#define PINE6_SET() SET_PIN(PINE, 6)
	#define PINE7_SET() SET_PIN(PINE, 7)


	#define DDRE0_CLEAR() CLEAR_PIN(DDRE, 0)
	#define DDRE1_CLEAR() CLEAR_PIN(DDRE, 1)
	#define DDRE2_CLEAR() CLEAR_PIN(DDRE, 2)
	#define DDRE3_CLEAR() CLEAR_PIN(DDRE, 3)
	#define DDRE4_CLEAR() CLEAR_PIN(DDRE, 4)
	#define DDRE5_CLEAR() CLEAR_PIN(DDRE, 5)
	#define DDRE6_CLEAR() CLEAR_PIN(DDRE, 6)
	#define DDRE7_CLEAR() CLEAR_PIN(DDRE, 7)

	#define PORTE0_CLEAR() CLEAR_PIN(PORTE, 0)
	#define PORTE1_CLEAR() CLEAR_PIN(PORTE, 1)
	#define PORTE2_CLEAR() CLEAR_PIN(PORTE, 2)
	#define PORTE3_CLEAR() CLEAR_PIN(PORTE, 3)
	#define PORTE4_CLEAR() CLEAR_PIN(PORTE, 4)
	#define PORTE5_CLEAR() CLEAR_PIN(PORTE, 5)
	#define PORTE6_CLEAR() CLEAR_PIN(PORTE, 6)
	#define PORTE7_CLEAR() CLEAR_PIN(PORTE, 7)

	#define PINE0_CLEAR() CLEAR_PIN(PINE, 0)
	#define PINE1_CLEAR() CLEAR_PIN(PINE, 1)
	#define PINE2_CLEAR() CLEAR_PIN(PINE, 2)
	#define PINE3_CLEAR() CLEAR_PIN(PINE, 3)
	#define PINE4_CLEAR() CLEAR_PIN(PINE, 4)
	#define PINE5_CLEAR() CLEAR_PIN(PINE, 5)
	#define PINE6_CLEAR() CLEAR_PIN(PINE, 6)
	#define PINE7_CLEAR() CLEAR_PIN(PINE, 7)


	#define DDRE0_TOGGLE() TOGGLE_PIN(DDRE, 0)
	#define DDRE1_TOGGLE() TOGGLE_PIN(DDRE, 1)
	#define DDRE2_TOGGLE() TOGGLE_PIN(DDRE, 2)
	#define DDRE3_TOGGLE() TOGGLE_PIN(DDRE, 3)
	#define DDRE4_TOGGLE() TOGGLE_PIN(DDRE, 4)
	#define DDRE5_TOGGLE() TOGGLE_PIN(DDRE, 5)
	#define DDRE6_TOGGLE() TOGGLE_PIN(DDRE, 6)
	#define DDRE7_TOGGLE() TOGGLE_PIN(DDRE, 7)

	#define PORTE0_TOGGLE() TOGGLE_PIN(PORTE, 0)
	#define PORTE1_TOGGLE() TOGGLE_PIN(PORTE, 1)
	#define PORTE2_TOGGLE() TOGGLE_PIN(PORTE, 2)
	#define PORTE3_TOGGLE() TOGGLE_PIN(PORTE, 3)
	#define PORTE4_TOGGLE() TOGGLE_PIN(PORTE, 4)
	#define PORTE5_TOGGLE() TOGGLE_PIN(PORTE, 5)
	#define PORTE6_TOGGLE() TOGGLE_PIN(PORTE, 6)
	#define PORTE7_TOGGLE() TOGGLE_PIN(PORTE, 7)

	#define PINE0_TOGGLE() TOGGLE_PIN(PINE, 0)
	#define PINE1_TOGGLE() TOGGLE_PIN(PINE, 1)
	#define PINE2_TOGGLE() TOGGLE_PIN(PINE, 2)
	#define PINE3_TOGGLE() TOGGLE_PIN(PINE, 3)
	#define PINE4_TOGGLE() TOGGLE_PIN(PINE, 4)
	#define PINE5_TOGGLE() TOGGLE_PIN(PINE, 5)
	#define PINE6_TOGGLE() TOGGLE_PIN(PINE, 6)
	#define PINE7_TOGGLE() TOGGLE_PIN(PINE, 7)


	#define PE0_CHECK() PIN_IN_CHECK(E, 0)
	#define PE1_CHECK() PIN_IN_CHECK(E, 1)
	#define PE2_CHECK() PIN_IN_CHECK(E, 2)
	#define PE3_CHECK() PIN_IN_CHECK(E, 3)
	#define PE4_CHECK() PIN_IN_CHECK(E, 4)
	#define PE5_CHECK() PIN_IN_CHECK(E, 5)
	#define PE6_CHECK() PIN_IN_CHECK(E, 6)
	#define PE7_CHECK() PIN_IN_CHECK(E, 7)

	#define PE0_IS_SET() PIN_IN_CHECK_SET(E, 0)
	#define PE1_IS_SET() PIN_IN_CHECK_SET(E, 1)
	#define PE2_IS_SET() PIN_IN_CHECK_SET(E, 2)
	#define PE3_IS_SET() PIN_IN_CHECK_SET(E, 3)
	#define PE4_IS_SET() PIN_IN_CHECK_SET(E, 4)
	#define PE5_IS_SET() PIN_IN_CHECK_SET(E, 5)
	#define PE6_IS_SET() PIN_IN_CHECK_SET(E, 6)
	#define PE7_IS_SET() PIN_IN_CHECK_SET(E, 7)

	#define PE0_IS_UNSET() PIN_IN_CHECK_UNSET(E, 0)
	#define PE1_IS_UNSET() PIN_IN_CHECK_UNSET(E, 1)
	#define PE2_IS_UNSET() PIN_IN_CHECK_UNSET(E, 2)
	#define PE3_IS_UNSET() PIN_IN_CHECK_UNSET(E, 3)
	#define PE4_IS_UNSET() PIN_IN_CHECK_UNSET(E, 4)
	#define PE5_IS_UNSET() PIN_IN_CHECK_UNSET(E, 5)
	#define PE6_IS_UNSET() PIN_IN_CHECK_UNSET(E, 6)
	#define PE7_IS_UNSET() PIN_IN_CHECK_UNSET(E, 7)


	#define PE0_SET() PIN_OUT_SET(E, 0)
	#define PE1_SET() PIN_OUT_SET(E, 1)
	#define PE2_SET() PIN_OUT_SET(E, 2)
	#define PE3_SET() PIN_OUT_SET(E, 3)
	#define PE4_SET() PIN_OUT_SET(E, 4)
	#define PE5_SET() PIN_OUT_SET(E, 5)
	#define PE6_SET() PIN_OUT_SET(E, 6)
	#define PE7_SET() PIN_OUT_SET(E, 7)

	#define PE0_CLEAR() PIN_OUT_CLEAR(E, 0)
	#define PE1_CLEAR() PIN_OUT_CLEAR(E, 1)
	#define PE2_CLEAR() PIN_OUT_CLEAR(E, 2)
	#define PE3_CLEAR() PIN_OUT_CLEAR(E, 3)
	#define PE4_CLEAR() PIN_OUT_CLEAR(E, 4)
	#define PE5_CLEAR() PIN_OUT_CLEAR(E, 5)
	#define PE6_CLEAR() PIN_OUT_CLEAR(E, 6)
	#define PE7_CLEAR() PIN_OUT_CLEAR(E, 7)

	#define PE0_TOGGLE() PIN_OUT_TOGGLE(E, 0)
	#define PE1_TOGGLE() PIN_OUT_TOGGLE(E, 1)
	#define PE2_TOGGLE() PIN_OUT_TOGGLE(E, 2)
	#define PE3_TOGGLE() PIN_OUT_TOGGLE(E, 3)
	#define PE4_TOGGLE() PIN_OUT_TOGGLE(E, 4)
	#define PE5_TOGGLE() PIN_OUT_TOGGLE(E, 5)
	#define PE6_TOGGLE() PIN_OUT_TOGGLE(E, 6)
	#define PE7_TOGGLE() PIN_OUT_TOGGLE(E, 7)

	#define PE0_RESET() PIN_RESET(E, 0)
	#define PE1_RESET() PIN_RESET(E, 1)
	#define PE2_RESET() PIN_RESET(E, 2)
	#define PE3_RESET() PIN_RESET(E, 3)
	#define PE4_RESET() PIN_RESET(E, 4)
	#define PE5_RESET() PIN_RESET(E, 5)
	#define PE6_RESET() PIN_RESET(E, 6)
	#define PE7_RESET() PIN_RESET(E, 7)

	#define PE0_GET_MODE() PIN_GET_MODE(E, 0)
	#define PE1_GET_MODE() PIN_GET_MODE(E, 1)
	#define PE2_GET_MODE() PIN_GET_MODE(E, 2)
	#define PE3_GET_MODE() PIN_GET_MODE(E, 3)
	#define PE4_GET_MODE() PIN_GET_MODE(E, 4)
	#define PE5_GET_MODE() PIN_GET_MODE(E, 5)
	#define PE6_GET_MODE() PIN_GET_MODE(E, 6)
	#define PE7_GET_MODE() PIN_GET_MODE(E, 7)

	#define PE0_SET_MODE(mode) PIN_SET_MODE(E, 0, mode)
	#define PE1_SET_MODE(mode) PIN_SET_MODE(E, 1, mode)
	#define PE2_SET_MODE(mode) PIN_SET_MODE(E, 2, mode)
	#define PE3_SET_MODE(mode) PIN_SET_MODE(E, 3, mode)
	#define PE4_SET_MODE(mode) PIN_SET_MODE(E, 4, mode)
	#define PE5_SET_MODE(mode) PIN_SET_MODE(E, 5, mode)
	#define PE6_SET_MODE(mode) PIN_SET_MODE(E, 6, mode)
	#define PE7_SET_MODE(mode) PIN_SET_MODE(E, 7, mode)

#endif


#ifdef PORTF

	#define DDRF_0 PIN_REF(DDRF, 0)
	#define DDRF_1 PIN_REF(DDRF, 1)
	#define DDRF_2 PIN_REF(DDRF, 2)
	#define DDRF_3 PIN_REF(DDRF, 3)
	#define DDRF_4 PIN_REF(DDRF, 4)
	#define DDRF_5 PIN_REF(DDRF, 5)
	#define DDRF_6 PIN_REF(DDRF, 6)
	#define DDRF_7 PIN_REF(DDRF, 7)

	#define PORTF_0 PIN_REF(PORTF, 0)
	#define PORTF_1 PIN_REF(PORTF, 1)
	#define PORTF_2 PIN_REF(PORTF, 2)
	#define PORTF_3 PIN_REF(PORTF, 3)
	#define PORTF_4 PIN_REF(PORTF, 4)
	#define PORTF_5 PIN_REF(PORTF, 5)
	#define PORTF_6 PIN_REF(PORTF, 6)
	#define PORTF_7 PIN_REF(PORTF, 7)

	#define PINF_0 PIN_REF(PINF, 0)
	#define PINF_1 PIN_REF(PINF, 1)
	#define PINF_2 PIN_REF(PINF, 2)
	#define PINF_3 PIN_REF(PINF, 3)
	#define PINF_4 PIN_REF(PINF, 4)
	#define PINF_5 PIN_REF(PINF, 5)
	#define PINF_6 PIN_REF(PINF, 6)
	#define PINF_7 PIN_REF(PINF, 7)


	#define DDRF0_PAIR() PORT_PIN_PAIR(DDRF, 0)
	#define DDRF1_PAIR() PORT_PIN_PAIR(DDRF, 1)
	#define DDRF2_PAIR() PORT_PIN_PAIR(DDRF, 2)
	#define DDRF3_PAIR() PORT_PIN_PAIR(DDRF, 3)
	#define DDRF4_PAIR() PORT_PIN_PAIR(DDRF, 4)
	#define DDRF5_PAIR() PORT_PIN_PAIR(DDRF, 5)
	#define DDRF6_PAIR() PORT_PIN_PAIR(DDRF, 6)
	#define DDRF7_PAIR() PORT_PIN_PAIR(DDRF, 7)

	#define PORTF0_PAIR() PORT_PIN_PAIR(PORTF, 0)
	#define PORTF1_PAIR() PORT_PIN_PAIR(PORTF, 1)
	#define PORTF2_PAIR() PORT_PIN_PAIR(PORTF, 2)
	#define PORTF3_PAIR() PORT_PIN_PAIR(PORTF, 3)
	#define PORTF4_PAIR() PORT_PIN_PAIR(PORTF, 4)
	#define PORTF5_PAIR() PORT_PIN_PAIR(PORTF, 5)
	#define PORTF6_PAIR() PORT_PIN_PAIR(PORTF, 6)
	#define PORTF7_PAIR() PORT_PIN_PAIR(PORTF, 7)

	#define PINF0_PAIR() PORT_PIN_PAIR(PINF, 0)
	#define PINF1_PAIR() PORT_PIN_PAIR(PINF, 1)
	#define PINF2_PAIR() PORT_PIN_PAIR(PINF, 2)
	#define PINF3_PAIR() PORT_PIN_PAIR(PINF, 3)
	#define PINF4_PAIR() PORT_PIN_PAIR(PINF, 4)
	#define PINF5_PAIR() PORT_PIN_PAIR(PINF, 5)
	#define PINF6_PAIR() PORT_PIN_PAIR(PINF, 6)
	#define PINF7_PAIR() PORT_PIN_PAIR(PINF, 7)


	#define DDRF0_CHECK() CHECK_PIN(DDRF, 0)
	#define DDRF1_CHECK() CHECK_PIN(DDRF, 1)
	#define DDRF2_CHECK() CHECK_PIN(DDRF, 2)
	#define DDRF3_CHECK() CHECK_PIN(DDRF, 3)
	#define DDRF4_CHECK() CHECK_PIN(DDRF, 4)
	#define DDRF5_CHECK() CHECK_PIN(DDRF, 5)
	#define DDRF6_CHECK() CHECK_PIN(DDRF, 6)
	#define DDRF7_CHECK() CHECK_PIN(DDRF, 7)

	#define PORTF0_CHECK() CHECK_PIN(PORTF, 0)
	#define PORTF1_CHECK() CHECK_PIN(PORTF, 1)
	#define PORTF2_CHECK() CHECK_PIN(PORTF, 2)
	#define PORTF3_CHECK() CHECK_PIN(PORTF, 3)
	#define PORTF4_CHECK() CHECK_PIN(PORTF, 4)
	#define PORTF5_CHECK() CHECK_PIN(PORTF, 5)
	#define PORTF6_CHECK() CHECK_PIN(PORTF, 6)
	#define PORTF7_CHECK() CHECK_PIN(PORTF, 7)

	#define PINF0_CHECK() CHECK_PIN(PINF, 0)
	#define PINF1_CHECK() CHECK_PIN(PINF, 1)
	#define PINF2_CHECK() CHECK_PIN(PINF, 2)
	#define PINF3_CHECK() CHECK_PIN(PINF, 3)
	#define PINF4_CHECK() CHECK_PIN(PINF, 4)
	#define PINF5_CHECK() CHECK_PIN(PINF, 5)
	#define PINF6_CHECK() CHECK_PIN(PINF, 6)
	#define PINF7_CHECK() CHECK_PIN(PINF, 7)


	#define DDRF0_IS_SET() CHECK_SET_PIN(DDRF, 0)
	#define DDRF1_IS_SET() CHECK_SET_PIN(DDRF, 1)
	#define DDRF2_IS_SET() CHECK_SET_PIN(DDRF, 2)
	#define DDRF3_IS_SET() CHECK_SET_PIN(DDRF, 3)
	#define DDRF4_IS_SET() CHECK_SET_PIN(DDRF, 4)
	#define DDRF5_IS_SET() CHECK_SET_PIN(DDRF, 5)
	#define DDRF6_IS_SET() CHECK_SET_PIN(DDRF, 6)
	#define DDRF7_IS_SET() CHECK_SET_PIN(DDRF, 7)

	#define PORTF0_IS_SET() CHECK_SET_PIN(PORTF, 0)
	#define PORTF1_IS_SET() CHECK_SET_PIN(PORTF, 1)
	#define PORTF2_IS_SET() CHECK_SET_PIN(PORTF, 2)
	#define PORTF3_IS_SET() CHECK_SET_PIN(PORTF, 3)
	#define PORTF4_IS_SET() CHECK_SET_PIN(PORTF, 4)
	#define PORTF5_IS_SET() CHECK_SET_PIN(PORTF, 5)
	#define PORTF6_IS_SET() CHECK_SET_PIN(PORTF, 6)
	#define PORTF7_IS_SET() CHECK_SET_PIN(PORTF, 7)

	#define PINF0_IS_SET() CHECK_SET_PIN(PINF, 0)
	#define PINF1_IS_SET() CHECK_SET_PIN(PINF, 1)
	#define PINF2_IS_SET() CHECK_SET_PIN(PINF, 2)
	#define PINF3_IS_SET() CHECK_SET_PIN(PINF, 3)
	#define PINF4_IS_SET() CHECK_SET_PIN(PINF, 4)
	#define PINF5_IS_SET() CHECK_SET_PIN(PINF, 5)
	#define PINF6_IS_SET() CHECK_SET_PIN(PINF, 6)
	#define PINF7_IS_SET() CHECK_SET_PIN(PINF, 7)


	#define DDRF0_IS_UNSET() CHECK_UNSET_PIN(DDRF, 0)
	#define DDRF1_IS_UNSET() CHECK_UNSET_PIN(DDRF, 1)
	#define DDRF2_IS_UNSET() CHECK_UNSET_PIN(DDRF, 2)
	#define DDRF3_IS_UNSET() CHECK_UNSET_PIN(DDRF, 3)
	#define DDRF4_IS_UNSET() CHECK_UNSET_PIN(DDRF, 4)
	#define DDRF5_IS_UNSET() CHECK_UNSET_PIN(DDRF, 5)
	#define DDRF6_IS_UNSET() CHECK_UNSET_PIN(DDRF, 6)
	#define DDRF7_IS_UNSET() CHECK_UNSET_PIN(DDRF, 7)

	#define PORTF0_IS_UNSET() CHECK_UNSET_PIN(PORTF, 0)
	#define PORTF1_IS_UNSET() CHECK_UNSET_PIN(PORTF, 1)
	#define PORTF2_IS_UNSET() CHECK_UNSET_PIN(PORTF, 2)
	#define PORTF3_IS_UNSET() CHECK_UNSET_PIN(PORTF, 3)
	#define PORTF4_IS_UNSET() CHECK_UNSET_PIN(PORTF, 4)
	#define PORTF5_IS_UNSET() CHECK_UNSET_PIN(PORTF, 5)
	#define PORTF6_IS_UNSET() CHECK_UNSET_PIN(PORTF, 6)
	#define PORTF7_IS_UNSET() CHECK_UNSET_PIN(PORTF, 7)

	#define PINF0_IS_UNSET() CHECK_UNSET_PIN(PINF, 0)
	#define PINF1_IS_UNSET() CHECK_UNSET_PIN(PINF, 1)
	#define PINF2_IS_UNSET() CHECK_UNSET_PIN(PINF, 2)
	#define PINF3_IS_UNSET() CHECK_UNSET_PIN(PINF, 3)
	#define PINF4_IS_UNSET() CHECK_UNSET_PIN(PINF, 4)
	#define PINF5_IS_UNSET() CHECK_UNSET_PIN(PINF, 5)
	#define PINF6_IS_UNSET() CHECK_UNSET_PIN(PINF, 6)
	#define PINF7_IS_UNSET() CHECK_UNSET_PIN(PINF, 7)


	#define DDRF0_SET() SET_PIN(DDRF, 0)
	#define DDRF1_SET() SET_PIN(DDRF, 1)
	#define DDRF2_SET() SET_PIN(DDRF, 2)
	#define DDRF3_SET() SET_PIN(DDRF, 3)
	#define DDRF4_SET() SET_PIN(DDRF, 4)
	#define DDRF5_SET() SET_PIN(DDRF, 5)
	#define DDRF6_SET() SET_PIN(DDRF, 6)
	#define DDRF7_SET() SET_PIN(DDRF, 7)

	#define PORTF0_SET() SET_PIN(PORTF, 0)
	#define PORTF1_SET() SET_PIN(PORTF, 1)
	#define PORTF2_SET() SET_PIN(PORTF, 2)
	#define PORTF3_SET() SET_PIN(PORTF, 3)
	#define PORTF4_SET() SET_PIN(PORTF, 4)
	#define PORTF5_SET() SET_PIN(PORTF, 5)
	#define PORTF6_SET() SET_PIN(PORTF, 6)
	#define PORTF7_SET() SET_PIN(PORTF, 7)

	#define PINF0_SET() SET_PIN(PINF, 0)
	#define PINF1_SET() SET_PIN(PINF, 1)
	#define PINF2_SET() SET_PIN(PINF, 2)
	#define PINF3_SET() SET_PIN(PINF, 3)
	#define PINF4_SET() SET_PIN(PINF, 4)
	#define PINF5_SET() SET_PIN(PINF, 5)
	#define PINF6_SET() SET_PIN(PINF, 6)
	#define PINF7_SET() SET_PIN(PINF, 7)


	#define DDRF0_CLEAR() CLEAR_PIN(DDRF, 0)
	#define DDRF1_CLEAR() CLEAR_PIN(DDRF, 1)
	#define DDRF2_CLEAR() CLEAR_PIN(DDRF, 2)
	#define DDRF3_CLEAR() CLEAR_PIN(DDRF, 3)
	#define DDRF4_CLEAR() CLEAR_PIN(DDRF, 4)
	#define DDRF5_CLEAR() CLEAR_PIN(DDRF, 5)
	#define DDRF6_CLEAR() CLEAR_PIN(DDRF, 6)
	#define DDRF7_CLEAR() CLEAR_PIN(DDRF, 7)

	#define PORTF0_CLEAR() CLEAR_PIN(PORTF, 0)
	#define PORTF1_CLEAR() CLEAR_PIN(PORTF, 1)
	#define PORTF2_CLEAR() CLEAR_PIN(PORTF, 2)
	#define PORTF3_CLEAR() CLEAR_PIN(PORTF, 3)
	#define PORTF4_CLEAR() CLEAR_PIN(PORTF, 4)
	#define PORTF5_CLEAR() CLEAR_PIN(PORTF, 5)
	#define PORTF6_CLEAR() CLEAR_PIN(PORTF, 6)
	#define PORTF7_CLEAR() CLEAR_PIN(PORTF, 7)

	#define PINF0_CLEAR() CLEAR_PIN(PINF, 0)
	#define PINF1_CLEAR() CLEAR_PIN(PINF, 1)
	#define PINF2_CLEAR() CLEAR_PIN(PINF, 2)
	#define PINF3_CLEAR() CLEAR_PIN(PINF, 3)
	#define PINF4_CLEAR() CLEAR_PIN(PINF, 4)
	#define PINF5_CLEAR() CLEAR_PIN(PINF, 5)
	#define PINF6_CLEAR() CLEAR_PIN(PINF, 6)
	#define PINF7_CLEAR() CLEAR_PIN(PINF, 7)


	#define DDRF0_TOGGLE() TOGGLE_PIN(DDRF, 0)
	#define DDRF1_TOGGLE() TOGGLE_PIN(DDRF, 1)
	#define DDRF2_TOGGLE() TOGGLE_PIN(DDRF, 2)
	#define DDRF3_TOGGLE() TOGGLE_PIN(DDRF, 3)
	#define DDRF4_TOGGLE() TOGGLE_PIN(DDRF, 4)
	#define DDRF5_TOGGLE() TOGGLE_PIN(DDRF, 5)
	#define DDRF6_TOGGLE() TOGGLE_PIN(DDRF, 6)
	#define DDRF7_TOGGLE() TOGGLE_PIN(DDRF, 7)

	#define PORTF0_TOGGLE() TOGGLE_PIN(PORTF, 0)
	#define PORTF1_TOGGLE() TOGGLE_PIN(PORTF, 1)
	#define PORTF2_TOGGLE() TOGGLE_PIN(PORTF, 2)
	#define PORTF3_TOGGLE() TOGGLE_PIN(PORTF, 3)
	#define PORTF4_TOGGLE() TOGGLE_PIN(PORTF, 4)
	#define PORTF5_TOGGLE() TOGGLE_PIN(PORTF, 5)
	#define PORTF6_TOGGLE() TOGGLE_PIN(PORTF, 6)
	#define PORTF7_TOGGLE() TOGGLE_PIN(PORTF, 7)

	#define PINF0_TOGGLE() TOGGLE_PIN(PINF, 0)
	#define PINF1_TOGGLE() TOGGLE_PIN(PINF, 1)
	#define PINF2_TOGGLE() TOGGLE_PIN(PINF, 2)
	#define PINF3_TOGGLE() TOGGLE_PIN(PINF, 3)
	#define PINF4_TOGGLE() TOGGLE_PIN(PINF, 4)
	#define PINF5_TOGGLE() TOGGLE_PIN(PINF, 5)
	#define PINF6_TOGGLE() TOGGLE_PIN(PINF, 6)
	#define PINF7_TOGGLE() TOGGLE_PIN(PINF, 7)


	#define PF0_CHECK() PIN_IN_CHECK(F, 0)
	#define PF1_CHECK() PIN_IN_CHECK(F, 1)
	#define PF2_CHECK() PIN_IN_CHECK(F, 2)
	#define PF3_CHECK() PIN_IN_CHECK(F, 3)
	#define PF4_CHECK() PIN_IN_CHECK(F, 4)
	#define PF5_CHECK() PIN_IN_CHECK(F, 5)
	#define PF6_CHECK() PIN_IN_CHECK(F, 6)
	#define PF7_CHECK() PIN_IN_CHECK(F, 7)

	#define PF0_IS_SET() PIN_IN_CHECK_SET(F, 0)
	#define PF1_IS_SET() PIN_IN_CHECK_SET(F, 1)
	#define PF2_IS_SET() PIN_IN_CHECK_SET(F, 2)
	#define PF3_IS_SET() PIN_IN_CHECK_SET(F, 3)
	#define PF4_IS_SET() PIN_IN_CHECK_SET(F, 4)
	#define PF5_IS_SET() PIN_IN_CHECK_SET(F, 5)
	#define PF6_IS_SET() PIN_IN_CHECK_SET(F, 6)
	#define PF7_IS_SET() PIN_IN_CHECK_SET(F, 7)

	#define PF0_IS_UNSET() PIN_IN_CHECK_UNSET(F, 0)
	#define PF1_IS_UNSET() PIN_IN_CHECK_UNSET(F, 1)
	#define PF2_IS_UNSET() PIN_IN_CHECK_UNSET(F, 2)
	#define PF3_IS_UNSET() PIN_IN_CHECK_UNSET(F, 3)
	#define PF4_IS_UNSET() PIN_IN_CHECK_UNSET(F, 4)
	#define PF5_IS_UNSET() PIN_IN_CHECK_UNSET(F, 5)
	#define PF6_IS_UNSET() PIN_IN_CHECK_UNSET(F, 6)
	#define PF7_IS_UNSET() PIN_IN_CHECK_UNSET(F, 7)


	#define PF0_SET() PIN_OUT_SET(F, 0)
	#define PF1_SET() PIN_OUT_SET(F, 1)
	#define PF2_SET() PIN_OUT_SET(F, 2)
	#define PF3_SET() PIN_OUT_SET(F, 3)
	#define PF4_SET() PIN_OUT_SET(F, 4)
	#define PF5_SET() PIN_OUT_SET(F, 5)
	#define PF6_SET() PIN_OUT_SET(F, 6)
	#define PF7_SET() PIN_OUT_SET(F, 7)

	#define PF0_CLEAR() PIN_OUT_CLEAR(F, 0)
	#define PF1_CLEAR() PIN_OUT_CLEAR(F, 1)
	#define PF2_CLEAR() PIN_OUT_CLEAR(F, 2)
	#define PF3_CLEAR() PIN_OUT_CLEAR(F, 3)
	#define PF4_CLEAR() PIN_OUT_CLEAR(F, 4)
	#define PF5_CLEAR() PIN_OUT_CLEAR(F, 5)
	#define PF6_CLEAR() PIN_OUT_CLEAR(F, 6)
	#define PF7_CLEAR() PIN_OUT_CLEAR(F, 7)

	#define PF0_TOGGLE() PIN_OUT_TOGGLE(F, 0)
	#define PF1_TOGGLE() PIN_OUT_TOGGLE(F, 1)
	#define PF2_TOGGLE() PIN_OUT_TOGGLE(F, 2)
	#define PF3_TOGGLE() PIN_OUT_TOGGLE(F, 3)
	#define PF4_TOGGLE() PIN_OUT_TOGGLE(F, 4)
	#define PF5_TOGGLE() PIN_OUT_TOGGLE(F, 5)
	#define PF6_TOGGLE() PIN_OUT_TOGGLE(F, 6)
	#define PF7_TOGGLE() PIN_OUT_TOGGLE(F, 7)

	#define PF0_RESET() PIN_RESET(F, 0)
	#define PF1_RESET() PIN_RESET(F, 1)
	#define PF2_RESET() PIN_RESET(F, 2)
	#define PF3_RESET() PIN_RESET(F, 3)
	#define PF4_RESET() PIN_RESET(F, 4)
	#define PF5_RESET() PIN_RESET(F, 5)
	#define PF6_RESET() PIN_RESET(F, 6)
	#define PF7_RESET() PIN_RESET(F, 7)

	#define PF0_GET_MODE() PIN_GET_MODE(F, 0)
	#define PF1_GET_MODE() PIN_GET_MODE(F, 1)
	#define PF2_GET_MODE() PIN_GET_MODE(F, 2)
	#define PF3_GET_MODE() PIN_GET_MODE(F, 3)
	#define PF4_GET_MODE() PIN_GET_MODE(F, 4)
	#define PF5_GET_MODE() PIN_GET_MODE(F, 5)
	#define PF6_GET_MODE() PIN_GET_MODE(F, 6)
	#define PF7_GET_MODE() PIN_GET_MODE(F, 7)

	#define PF0_SET_MODE(mode) PIN_SET_MODE(F, 0, mode)
	#define PF1_SET_MODE(mode) PIN_SET_MODE(F, 1, mode)
	#define PF2_SET_MODE(mode) PIN_SET_MODE(F, 2, mode)
	#define PF3_SET_MODE(mode) PIN_SET_MODE(F, 3, mode)
	#define PF4_SET_MODE(mode) PIN_SET_MODE(F, 4, mode)
	#define PF5_SET_MODE(mode) PIN_SET_MODE(F, 5, mode)
	#define PF6_SET_MODE(mode) PIN_SET_MODE(F, 6, mode)
	#define PF7_SET_MODE(mode) PIN_SET_MODE(F, 7, mode)

#endif


#ifdef PORTG

	#define DDRG_0 PIN_REF(DDRG, 0)
	#define DDRG_1 PIN_REF(DDRG, 1)
	#define DDRG_2 PIN_REF(DDRG, 2)
	#define DDRG_3 PIN_REF(DDRG, 3)
	#define DDRG_4 PIN_REF(DDRG, 4)
	#define DDRG_5 PIN_REF(DDRG, 5)
	#define DDRG_6 PIN_REF(DDRG, 6)
	#define DDRG_7 PIN_REF(DDRG, 7)

	#define PORTG_0 PIN_REF(PORTG, 0)
	#define PORTG_1 PIN_REF(PORTG, 1)
	#define PORTG_2 PIN_REF(PORTG, 2)
	#define PORTG_3 PIN_REF(PORTG, 3)
	#define PORTG_4 PIN_REF(PORTG, 4)
	#define PORTG_5 PIN_REF(PORTG, 5)
	#define PORTG_6 PIN_REF(PORTG, 6)
	#define PORTG_7 PIN_REF(PORTG, 7)

	#define PING_0 PIN_REF(PING, 0)
	#define PING_1 PIN_REF(PING, 1)
	#define PING_2 PIN_REF(PING, 2)
	#define PING_3 PIN_REF(PING, 3)
	#define PING_4 PIN_REF(PING, 4)
	#define PING_5 PIN_REF(PING, 5)
	#define PING_6 PIN_REF(PING, 6)
	#define PING_7 PIN_REF(PING, 7)


	#define DDRG0_PAIR() PORT_PIN_PAIR(DDRG, 0)
	#define DDRG1_PAIR() PORT_PIN_PAIR(DDRG, 1)
	#define DDRG2_PAIR() PORT_PIN_PAIR(DDRG, 2)
	#define DDRG3_PAIR() PORT_PIN_PAIR(DDRG, 3)
	#define DDRG4_PAIR() PORT_PIN_PAIR(DDRG, 4)
	#define DDRG5_PAIR() PORT_PIN_PAIR(DDRG, 5)
	#define DDRG6_PAIR() PORT_PIN_PAIR(DDRG, 6)
	#define DDRG7_PAIR() PORT_PIN_PAIR(DDRG, 7)

	#define PORTG0_PAIR() PORT_PIN_PAIR(PORTG, 0)
	#define PORTG1_PAIR() PORT_PIN_PAIR(PORTG, 1)
	#define PORTG2_PAIR() PORT_PIN_PAIR(PORTG, 2)
	#define PORTG3_PAIR() PORT_PIN_PAIR(PORTG, 3)
	#define PORTG4_PAIR() PORT_PIN_PAIR(PORTG, 4)
	#define PORTG5_PAIR() PORT_PIN_PAIR(PORTG, 5)
	#define PORTG6_PAIR() PORT_PIN_PAIR(PORTG, 6)
	#define PORTG7_PAIR() PORT_PIN_PAIR(PORTG, 7)

	#define PING0_PAIR() PORT_PIN_PAIR(PING, 0)
	#define PING1_PAIR() PORT_PIN_PAIR(PING, 1)
	#define PING2_PAIR() PORT_PIN_PAIR(PING, 2)
	#define PING3_PAIR() PORT_PIN_PAIR(PING, 3)
	#define PING4_PAIR() PORT_PIN_PAIR(PING, 4)
	#define PING5_PAIR() PORT_PIN_PAIR(PING, 5)
	#define PING6_PAIR() PORT_PIN_PAIR(PING, 6)
	#define PING7_PAIR() PORT_PIN_PAIR(PING, 7)


	#define DDRG0_CHECK() CHECK_PIN(DDRG, 0)
	#define DDRG1_CHECK() CHECK_PIN(DDRG, 1)
	#define DDRG2_CHECK() CHECK_PIN(DDRG, 2)
	#define DDRG3_CHECK() CHECK_PIN(DDRG, 3)
	#define DDRG4_CHECK() CHECK_PIN(DDRG, 4)
	#define DDRG5_CHECK() CHECK_PIN(DDRG, 5)
	#define DDRG6_CHECK() CHECK_PIN(DDRG, 6)
	#define DDRG7_CHECK() CHECK_PIN(DDRG, 7)

	#define PORTG0_CHECK() CHECK_PIN(PORTG, 0)
	#define PORTG1_CHECK() CHECK_PIN(PORTG, 1)
	#define PORTG2_CHECK() CHECK_PIN(PORTG, 2)
	#define PORTG3_CHECK() CHECK_PIN(PORTG, 3)
	#define PORTG4_CHECK() CHECK_PIN(PORTG, 4)
	#define PORTG5_CHECK() CHECK_PIN(PORTG, 5)
	#define PORTG6_CHECK() CHECK_PIN(PORTG, 6)
	#define PORTG7_CHECK() CHECK_PIN(PORTG, 7)

	#define PING0_CHECK() CHECK_PIN(PING, 0)
	#define PING1_CHECK() CHECK_PIN(PING, 1)
	#define PING2_CHECK() CHECK_PIN(PING, 2)
	#define PING3_CHECK() CHECK_PIN(PING, 3)
	#define PING4_CHECK() CHECK_PIN(PING, 4)
	#define PING5_CHECK() CHECK_PIN(PING, 5)
	#define PING6_CHECK() CHECK_PIN(PING, 6)
	#define PING7_CHECK() CHECK_PIN(PING, 7)


	#define DDRG0_IS_SET() CHECK_SET_PIN(DDRG, 0)
	#define DDRG1_IS_SET() CHECK_SET_PIN(DDRG, 1)
	#define DDRG2_IS_SET() CHECK_SET_PIN(DDRG, 2)
	#define DDRG3_IS_SET() CHECK_SET_PIN(DDRG, 3)
	#define DDRG4_IS_SET() CHECK_SET_PIN(DDRG, 4)
	#define DDRG5_IS_SET() CHECK_SET_PIN(DDRG, 5)
	#define DDRG6_IS_SET() CHECK_SET_PIN(DDRG, 6)
	#define DDRG7_IS_SET() CHECK_SET_PIN(DDRG, 7)

	#define PORTG0_IS_SET() CHECK_SET_PIN(PORTG, 0)
	#define PORTG1_IS_SET() CHECK_SET_PIN(PORTG, 1)
	#define PORTG2_IS_SET() CHECK_SET_PIN(PORTG, 2)
	#define PORTG3_IS_SET() CHECK_SET_PIN(PORTG, 3)
	#define PORTG4_IS_SET() CHECK_SET_PIN(PORTG, 4)
	#define PORTG5_IS_SET() CHECK_SET_PIN(PORTG, 5)
	#define PORTG6_IS_SET() CHECK_SET_PIN(PORTG, 6)
	#define PORTG7_IS_SET() CHECK_SET_PIN(PORTG, 7)

	#define PING0_IS_SET() CHECK_SET_PIN(PING, 0)
	#define PING1_IS_SET() CHECK_SET_PIN(PING, 1)
	#define PING2_IS_SET() CHECK_SET_PIN(PING, 2)
	#define PING3_IS_SET() CHECK_SET_PIN(PING, 3)
	#define PING4_IS_SET() CHECK_SET_PIN(PING, 4)
	#define PING5_IS_SET() CHECK_SET_PIN(PING, 5)
	#define PING6_IS_SET() CHECK_SET_PIN(PING, 6)
	#define PING7_IS_SET() CHECK_SET_PIN(PING, 7)


	#define DDRG0_IS_UNSET() CHECK_UNSET_PIN(DDRG, 0)
	#define DDRG1_IS_UNSET() CHECK_UNSET_PIN(DDRG, 1)
	#define DDRG2_IS_UNSET() CHECK_UNSET_PIN(DDRG, 2)
	#define DDRG3_IS_UNSET() CHECK_UNSET_PIN(DDRG, 3)
	#define DDRG4_IS_UNSET() CHECK_UNSET_PIN(DDRG, 4)
	#define DDRG5_IS_UNSET() CHECK_UNSET_PIN(DDRG, 5)
	#define DDRG6_IS_UNSET() CHECK_UNSET_PIN(DDRG, 6)
	#define DDRG7_IS_UNSET() CHECK_UNSET_PIN(DDRG, 7)

	#define PORTG0_IS_UNSET() CHECK_UNSET_PIN(PORTG, 0)
	#define PORTG1_IS_UNSET() CHECK_UNSET_PIN(PORTG, 1)
	#define PORTG2_IS_UNSET() CHECK_UNSET_PIN(PORTG, 2)
	#define PORTG3_IS_UNSET() CHECK_UNSET_PIN(PORTG, 3)
	#define PORTG4_IS_UNSET() CHECK_UNSET_PIN(PORTG, 4)
	#define PORTG5_IS_UNSET() CHECK_UNSET_PIN(PORTG, 5)
	#define PORTG6_IS_UNSET() CHECK_UNSET_PIN(PORTG, 6)
	#define PORTG7_IS_UNSET() CHECK_UNSET_PIN(PORTG, 7)

	#define PING0_IS_UNSET() CHECK_UNSET_PIN(PING, 0)
	#define PING1_IS_UNSET() CHECK_UNSET_PIN(PING, 1)
	#define PING2_IS_UNSET() CHECK_UNSET_PIN(PING, 2)
	#define PING3_IS_UNSET() CHECK_UNSET_PIN(PING, 3)
	#define PING4_IS_UNSET() CHECK_UNSET_PIN(PING, 4)
	#define PING5_IS_UNSET() CHECK_UNSET_PIN(PING, 5)
	#define PING6_IS_UNSET() CHECK_UNSET_PIN(PING, 6)
	#define PING7_IS_UNSET() CHECK_UNSET_PIN(PING, 7)


	#define DDRG0_SET() SET_PIN(DDRG, 0)
	#define DDRG1_SET() SET_PIN(DDRG, 1)
	#define DDRG2_SET() SET_PIN(DDRG, 2)
	#define DDRG3_SET() SET_PIN(DDRG, 3)
	#define DDRG4_SET() SET_PIN(DDRG, 4)
	#define DDRG5_SET() SET_PIN(DDRG, 5)
	#define DDRG6_SET() SET_PIN(DDRG, 6)
	#define DDRG7_SET() SET_PIN(DDRG, 7)

	#define PORTG0_SET() SET_PIN(PORTG, 0)
	#define PORTG1_SET() SET_PIN(PORTG, 1)
	#define PORTG2_SET() SET_PIN(PORTG, 2)
	#define PORTG3_SET() SET_PIN(PORTG, 3)
	#define PORTG4_SET() SET_PIN(PORTG, 4)
	#define PORTG5_SET() SET_PIN(PORTG, 5)
	#define PORTG6_SET() SET_PIN(PORTG, 6)
	#define PORTG7_SET() SET_PIN(PORTG, 7)

	#define PING0_SET() SET_PIN(PING, 0)
	#define PING1_SET() SET_PIN(PING, 1)
	#define PING2_SET() SET_PIN(PING, 2)
	#define PING3_SET() SET_PIN(PING, 3)
	#define PING4_SET() SET_PIN(PING, 4)
	#define PING5_SET() SET_PIN(PING, 5)
	#define PING6_SET() SET_PIN(PING, 6)
	#define PING7_SET() SET_PIN(PING, 7)


	#define DDRG0_CLEAR() CLEAR_PIN(DDRG, 0)
	#define DDRG1_CLEAR() CLEAR_PIN(DDRG, 1)
	#define DDRG2_CLEAR() CLEAR_PIN(DDRG, 2)
	#define DDRG3_CLEAR() CLEAR_PIN(DDRG, 3)
	#define DDRG4_CLEAR() CLEAR_PIN(DDRG, 4)
	#define DDRG5_CLEAR() CLEAR_PIN(DDRG, 5)
	#define DDRG6_CLEAR() CLEAR_PIN(DDRG, 6)
	#define DDRG7_CLEAR() CLEAR_PIN(DDRG, 7)

	#define PORTG0_CLEAR() CLEAR_PIN(PORTG, 0)
	#define PORTG1_CLEAR() CLEAR_PIN(PORTG, 1)
	#define PORTG2_CLEAR() CLEAR_PIN(PORTG, 2)
	#define PORTG3_CLEAR() CLEAR_PIN(PORTG, 3)
	#define PORTG4_CLEAR() CLEAR_PIN(PORTG, 4)
	#define PORTG5_CLEAR() CLEAR_PIN(PORTG, 5)
	#define PORTG6_CLEAR() CLEAR_PIN(PORTG, 6)
	#define PORTG7_CLEAR() CLEAR_PIN(PORTG, 7)

	#define PING0_CLEAR() CLEAR_PIN(PING, 0)
	#define PING1_CLEAR() CLEAR_PIN(PING, 1)
	#define PING2_CLEAR() CLEAR_PIN(PING, 2)
	#define PING3_CLEAR() CLEAR_PIN(PING, 3)
	#define PING4_CLEAR() CLEAR_PIN(PING, 4)
	#define PING5_CLEAR() CLEAR_PIN(PING, 5)
	#define PING6_CLEAR() CLEAR_PIN(PING, 6)
	#define PING7_CLEAR() CLEAR_PIN(PING, 7)


	#define DDRG0_TOGGLE() TOGGLE_PIN(DDRG, 0)
	#define DDRG1_TOGGLE() TOGGLE_PIN(DDRG, 1)
	#define DDRG2_TOGGLE() TOGGLE_PIN(DDRG, 2)
	#define DDRG3_TOGGLE() TOGGLE_PIN(DDRG, 3)
	#define DDRG4_TOGGLE() TOGGLE_PIN(DDRG, 4)
	#define DDRG5_TOGGLE() TOGGLE_PIN(DDRG, 5)
	#define DDRG6_TOGGLE() TOGGLE_PIN(DDRG, 6)
	#define DDRG7_TOGGLE() TOGGLE_PIN(DDRG, 7)

	#define PORTG0_TOGGLE() TOGGLE_PIN(PORTG, 0)
	#define PORTG1_TOGGLE() TOGGLE_PIN(PORTG, 1)
	#define PORTG2_TOGGLE() TOGGLE_PIN(PORTG, 2)
	#define PORTG3_TOGGLE() TOGGLE_PIN(PORTG, 3)
	#define PORTG4_TOGGLE() TOGGLE_PIN(PORTG, 4)
	#define PORTG5_TOGGLE() TOGGLE_PIN(PORTG, 5)
	#define PORTG6_TOGGLE() TOGGLE_PIN(PORTG, 6)
	#define PORTG7_TOGGLE() TOGGLE_PIN(PORTG, 7)

	#define PING0_TOGGLE() TOGGLE_PIN(PING, 0)
	#define PING1_TOGGLE() TOGGLE_PIN(PING, 1)
	#define PING2_TOGGLE() TOGGLE_PIN(PING, 2)
	#define PING3_TOGGLE() TOGGLE_PIN(PING, 3)
	#define PING4_TOGGLE() TOGGLE_PIN(PING, 4)
	#define PING5_TOGGLE() TOGGLE_PIN(PING, 5)
	#define PING6_TOGGLE() TOGGLE_PIN(PING, 6)
	#define PING7_TOGGLE() TOGGLE_PIN(PING, 7)


	#define PG0_CHECK() PIN_IN_CHECK(G, 0)
	#define PG1_CHECK() PIN_IN_CHECK(G, 1)
	#define PG2_CHECK() PIN_IN_CHECK(G, 2)
	#define PG3_CHECK() PIN_IN_CHECK(G, 3)
	#define PG4_CHECK() PIN_IN_CHECK(G, 4)
	#define PG5_CHECK() PIN_IN_CHECK(G, 5)
	#define PG6_CHECK() PIN_IN_CHECK(G, 6)
	#define PG7_CHECK() PIN_IN_CHECK(G, 7)

	#define PG0_IS_SET() PIN_IN_CHECK_SET(G, 0)
	#define PG1_IS_SET() PIN_IN_CHECK_SET(G, 1)
	#define PG2_IS_SET() PIN_IN_CHECK_SET(G, 2)
	#define PG3_IS_SET() PIN_IN_CHECK_SET(G, 3)
	#define PG4_IS_SET() PIN_IN_CHECK_SET(G, 4)
	#define PG5_IS_SET() PIN_IN_CHECK_SET(G, 5)
	#define PG6_IS_SET() PIN_IN_CHECK_SET(G, 6)
	#define PG7_IS_SET() PIN_IN_CHECK_SET(G, 7)

	#define PG0_IS_UNSET() PIN_IN_CHECK_UNSET(G, 0)
	#define PG1_IS_UNSET() PIN_IN_CHECK_UNSET(G, 1)
	#define PG2_IS_UNSET() PIN_IN_CHECK_UNSET(G, 2)
	#define PG3_IS_UNSET() PIN_IN_CHECK_UNSET(G, 3)
	#define PG4_IS_UNSET() PIN_IN_CHECK_UNSET(G, 4)
	#define PG5_IS_UNSET() PIN_IN_CHECK_UNSET(G, 5)
	#define PG6_IS_UNSET() PIN_IN_CHECK_UNSET(G, 6)
	#define PG7_IS_UNSET() PIN_IN_CHECK_UNSET(G, 7)


	#define PG0_SET() PIN_OUT_SET(G, 0)
	#define PG1_SET() PIN_OUT_SET(G, 1)
	#define PG2_SET() PIN_OUT_SET(G, 2)
	#define PG3_SET() PIN_OUT_SET(G, 3)
	#define PG4_SET() PIN_OUT_SET(G, 4)
	#define PG5_SET() PIN_OUT_SET(G, 5)
	#define PG6_SET() PIN_OUT_SET(G, 6)
	#define PG7_SET() PIN_OUT_SET(G, 7)

	#define PG0_CLEAR() PIN_OUT_CLEAR(G, 0)
	#define PG1_CLEAR() PIN_OUT_CLEAR(G, 1)
	#define PG2_CLEAR() PIN_OUT_CLEAR(G, 2)
	#define PG3_CLEAR() PIN_OUT_CLEAR(G, 3)
	#define PG4_CLEAR() PIN_OUT_CLEAR(G, 4)
	#define PG5_CLEAR() PIN_OUT_CLEAR(G, 5)
	#define PG6_CLEAR() PIN_OUT_CLEAR(G, 6)
	#define PG7_CLEAR() PIN_OUT_CLEAR(G, 7)

	#define PG0_TOGGLE() PIN_OUT_TOGGLE(G, 0)
	#define PG1_TOGGLE() PIN_OUT_TOGGLE(G, 1)
	#define PG2_TOGGLE() PIN_OUT_TOGGLE(G, 2)
	#define PG3_TOGGLE() PIN_OUT_TOGGLE(G, 3)
	#define PG4_TOGGLE() PIN_OUT_TOGGLE(G, 4)
	#define PG5_TOGGLE() PIN_OUT_TOGGLE(G, 5)
	#define PG6_TOGGLE() PIN_OUT_TOGGLE(G, 6)
	#define PG7_TOGGLE() PIN_OUT_TOGGLE(G, 7)

	#define PG0_RESET() PIN_RESET(G, 0)
	#define PG1_RESET() PIN_RESET(G, 1)
	#define PG2_RESET() PIN_RESET(G, 2)
	#define PG3_RESET() PIN_RESET(G, 3)
	#define PG4_RESET() PIN_RESET(G, 4)
	#define PG5_RESET() PIN_RESET(G, 5)
	#define PG6_RESET() PIN_RESET(G, 6)
	#define PG7_RESET() PIN_RESET(G, 7)

	#define PG0_GET_MODE() PIN_GET_MODE(G, 0)
	#define PG1_GET_MODE() PIN_GET_MODE(G, 1)
	#define PG2_GET_MODE() PIN_GET_MODE(G, 2)
	#define PG3_GET_MODE() PIN_GET_MODE(G, 3)
	#define PG4_GET_MODE() PIN_GET_MODE(G, 4)
	#define PG5_GET_MODE() PIN_GET_MODE(G, 5)
	#define PG6_GET_MODE() PIN_GET_MODE(G, 6)
	#define PG7_GET_MODE() PIN_GET_MODE(G, 7)

	#define PG0_SET_MODE(mode) PIN_SET_MODE(G, 0, mode)
	#define PG1_SET_MODE(mode) PIN_SET_MODE(G, 1, mode)
	#define PG2_SET_MODE(mode) PIN_SET_MODE(G, 2, mode)
	#define PG3_SET_MODE(mode) PIN_SET_MODE(G, 3, mode)
	#define PG4_SET_MODE(mode) PIN_SET_MODE(G, 4, mode)
	#define PG5_SET_MODE(mode) PIN_SET_MODE(G, 5, mode)
	#define PG6_SET_MODE(mode) PIN_SET_MODE(G, 6, mode)
	#define PG7_SET_MODE(mode) PIN_SET_MODE(G, 7, mode)

#endif


#ifdef PORTH

	#define DDRH_0 PIN_REF(DDRH, 0)
	#define DDRH_1 PIN_REF(DDRH, 1)
	#define DDRH_2 PIN_REF(DDRH, 2)
	#define DDRH_3 PIN_REF(DDRH, 3)
	#define DDRH_4 PIN_REF(DDRH, 4)
	#define DDRH_5 PIN_REF(DDRH, 5)
	#define DDRH_6 PIN_REF(DDRH, 6)
	#define DDRH_7 PIN_REF(DDRH, 7)

	#define PORTH_0 PIN_REF(PORTH, 0)
	#define PORTH_1 PIN_REF(PORTH, 1)
	#define PORTH_2 PIN_REF(PORTH, 2)
	#define PORTH_3 PIN_REF(PORTH, 3)
	#define PORTH_4 PIN_REF(PORTH, 4)
	#define PORTH_5 PIN_REF(PORTH, 5)
	#define PORTH_6 PIN_REF(PORTH, 6)
	#define PORTH_7 PIN_REF(PORTH, 7)

	#define PINH_0 PIN_REF(PINH, 0)
	#define PINH_1 PIN_REF(PINH, 1)
	#define PINH_2 PIN_REF(PINH, 2)
	#define PINH_3 PIN_REF(PINH, 3)
	#define PINH_4 PIN_REF(PINH, 4)
	#define PINH_5 PIN_REF(PINH, 5)
	#define PINH_6 PIN_REF(PINH, 6)
	#define PINH_7 PIN_REF(PINH, 7)


	#define DDRH0_PAIR() PORT_PIN_PAIR(DDRH, 0)
	#define DDRH1_PAIR() PORT_PIN_PAIR(DDRH, 1)
	#define DDRH2_PAIR() PORT_PIN_PAIR(DDRH, 2)
	#define DDRH3_PAIR() PORT_PIN_PAIR(DDRH, 3)
	#define DDRH4_PAIR() PORT_PIN_PAIR(DDRH, 4)
	#define DDRH5_PAIR() PORT_PIN_PAIR(DDRH, 5)
	#define DDRH6_PAIR() PORT_PIN_PAIR(DDRH, 6)
	#define DDRH7_PAIR() PORT_PIN_PAIR(DDRH, 7)

	#define PORTH0_PAIR() PORT_PIN_PAIR(PORTH, 0)
	#define PORTH1_PAIR() PORT_PIN_PAIR(PORTH, 1)
	#define PORTH2_PAIR() PORT_PIN_PAIR(PORTH, 2)
	#define PORTH3_PAIR() PORT_PIN_PAIR(PORTH, 3)
	#define PORTH4_PAIR() PORT_PIN_PAIR(PORTH, 4)
	#define PORTH5_PAIR() PORT_PIN_PAIR(PORTH, 5)
	#define PORTH6_PAIR() PORT_PIN_PAIR(PORTH, 6)
	#define PORTH7_PAIR() PORT_PIN_PAIR(PORTH, 7)

	#define PINH0_PAIR() PORT_PIN_PAIR(PINH, 0)
	#define PINH1_PAIR() PORT_PIN_PAIR(PINH, 1)
	#define PINH2_PAIR() PORT_PIN_PAIR(PINH, 2)
	#define PINH3_PAIR() PORT_PIN_PAIR(PINH, 3)
	#define PINH4_PAIR() PORT_PIN_PAIR(PINH, 4)
	#define PINH5_PAIR() PORT_PIN_PAIR(PINH, 5)
	#define PINH6_PAIR() PORT_PIN_PAIR(PINH, 6)
	#define PINH7_PAIR() PORT_PIN_PAIR(PINH, 7)


	#define DDRH0_CHECK() CHECK_PIN(DDRH, 0)
	#define DDRH1_CHECK() CHECK_PIN(DDRH, 1)
	#define DDRH2_CHECK() CHECK_PIN(DDRH, 2)
	#define DDRH3_CHECK() CHECK_PIN(DDRH, 3)
	#define DDRH4_CHECK() CHECK_PIN(DDRH, 4)
	#define DDRH5_CHECK() CHECK_PIN(DDRH, 5)
	#define DDRH6_CHECK() CHECK_PIN(DDRH, 6)
	#define DDRH7_CHECK() CHECK_PIN(DDRH, 7)

	#define PORTH0_CHECK() CHECK_PIN(PORTH, 0)
	#define PORTH1_CHECK() CHECK_PIN(PORTH, 1)
	#define PORTH2_CHECK() CHECK_PIN(PORTH, 2)
	#define PORTH3_CHECK() CHECK_PIN(PORTH, 3)
	#define PORTH4_CHECK() CHECK_PIN(PORTH, 4)
	#define PORTH5_CHECK() CHECK_PIN(PORTH, 5)
	#define PORTH6_CHECK() CHECK_PIN(PORTH, 6)
	#define PORTH7_CHECK() CHECK_PIN(PORTH, 7)

	#define PINH0_CHECK() CHECK_PIN(PINH, 0)
	#define PINH1_CHECK() CHECK_PIN(PINH, 1)
	#define PINH2_CHECK() CHECK_PIN(PINH, 2)
	#define PINH3_CHECK() CHECK_PIN(PINH, 3)
	#define PINH4_CHECK() CHECK_PIN(PINH, 4)
	#define PINH5_CHECK() CHECK_PIN(PINH, 5)
	#define PINH6_CHECK() CHECK_PIN(PINH, 6)
	#define PINH7_CHECK() CHECK_PIN(PINH, 7)


	#define DDRH0_IS_SET() CHECK_SET_PIN(DDRH, 0)
	#define DDRH1_IS_SET() CHECK_SET_PIN(DDRH, 1)
	#define DDRH2_IS_SET() CHECK_SET_PIN(DDRH, 2)
	#define DDRH3_IS_SET() CHECK_SET_PIN(DDRH, 3)
	#define DDRH4_IS_SET() CHECK_SET_PIN(DDRH, 4)
	#define DDRH5_IS_SET() CHECK_SET_PIN(DDRH, 5)
	#define DDRH6_IS_SET() CHECK_SET_PIN(DDRH, 6)
	#define DDRH7_IS_SET() CHECK_SET_PIN(DDRH, 7)

	#define PORTH0_IS_SET() CHECK_SET_PIN(PORTH, 0)
	#define PORTH1_IS_SET() CHECK_SET_PIN(PORTH, 1)
	#define PORTH2_IS_SET() CHECK_SET_PIN(PORTH, 2)
	#define PORTH3_IS_SET() CHECK_SET_PIN(PORTH, 3)
	#define PORTH4_IS_SET() CHECK_SET_PIN(PORTH, 4)
	#define PORTH5_IS_SET() CHECK_SET_PIN(PORTH, 5)
	#define PORTH6_IS_SET() CHECK_SET_PIN(PORTH, 6)
	#define PORTH7_IS_SET() CHECK_SET_PIN(PORTH, 7)

	#define PINH0_IS_SET() CHECK_SET_PIN(PINH, 0)
	#define PINH1_IS_SET() CHECK_SET_PIN(PINH, 1)
	#define PINH2_IS_SET() CHECK_SET_PIN(PINH, 2)
	#define PINH3_IS_SET() CHECK_SET_PIN(PINH, 3)
	#define PINH4_IS_SET() CHECK_SET_PIN(PINH, 4)
	#define PINH5_IS_SET() CHECK_SET_PIN(PINH, 5)
	#define PINH6_IS_SET() CHECK_SET_PIN(PINH, 6)
	#define PINH7_IS_SET() CHECK_SET_PIN(PINH, 7)


	#define DDRH0_IS_UNSET() CHECK_UNSET_PIN(DDRH, 0)
	#define DDRH1_IS_UNSET() CHECK_UNSET_PIN(DDRH, 1)
	#define DDRH2_IS_UNSET() CHECK_UNSET_PIN(DDRH, 2)
	#define DDRH3_IS_UNSET() CHECK_UNSET_PIN(DDRH, 3)
	#define DDRH4_IS_UNSET() CHECK_UNSET_PIN(DDRH, 4)
	#define DDRH5_IS_UNSET() CHECK_UNSET_PIN(DDRH, 5)
	#define DDRH6_IS_UNSET() CHECK_UNSET_PIN(DDRH, 6)
	#define DDRH7_IS_UNSET() CHECK_UNSET_PIN(DDRH, 7)

	#define PORTH0_IS_UNSET() CHECK_UNSET_PIN(PORTH, 0)
	#define PORTH1_IS_UNSET() CHECK_UNSET_PIN(PORTH, 1)
	#define PORTH2_IS_UNSET() CHECK_UNSET_PIN(PORTH, 2)
	#define PORTH3_IS_UNSET() CHECK_UNSET_PIN(PORTH, 3)
	#define PORTH4_IS_UNSET() CHECK_UNSET_PIN(PORTH, 4)
	#define PORTH5_IS_UNSET() CHECK_UNSET_PIN(PORTH, 5)
	#define PORTH6_IS_UNSET() CHECK_UNSET_PIN(PORTH, 6)
	#define PORTH7_IS_UNSET() CHECK_UNSET_PIN(PORTH, 7)

	#define PINH0_IS_UNSET() CHECK_UNSET_PIN(PINH, 0)
	#define PINH1_IS_UNSET() CHECK_UNSET_PIN(PINH, 1)
	#define PINH2_IS_UNSET() CHECK_UNSET_PIN(PINH, 2)
	#define PINH3_IS_UNSET() CHECK_UNSET_PIN(PINH, 3)
	#define PINH4_IS_UNSET() CHECK_UNSET_PIN(PINH, 4)
	#define PINH5_IS_UNSET() CHECK_UNSET_PIN(PINH, 5)
	#define PINH6_IS_UNSET() CHECK_UNSET_PIN(PINH, 6)
	#define PINH7_IS_UNSET() CHECK_UNSET_PIN(PINH, 7)


	#define DDRH0_SET() SET_PIN(DDRH, 0)
	#define DDRH1_SET() SET_PIN(DDRH, 1)
	#define DDRH2_SET() SET_PIN(DDRH, 2)
	#define DDRH3_SET() SET_PIN(DDRH, 3)
	#define DDRH4_SET() SET_PIN(DDRH, 4)
	#define DDRH5_SET() SET_PIN(DDRH, 5)
	#define DDRH6_SET() SET_PIN(DDRH, 6)
	#define DDRH7_SET() SET_PIN(DDRH, 7)

	#define PORTH0_SET() SET_PIN(PORTH, 0)
	#define PORTH1_SET() SET_PIN(PORTH, 1)
	#define PORTH2_SET() SET_PIN(PORTH, 2)
	#define PORTH3_SET() SET_PIN(PORTH, 3)
	#define PORTH4_SET() SET_PIN(PORTH, 4)
	#define PORTH5_SET() SET_PIN(PORTH, 5)
	#define PORTH6_SET() SET_PIN(PORTH, 6)
	#define PORTH7_SET() SET_PIN(PORTH, 7)

	#define PINH0_SET() SET_PIN(PINH, 0)
	#define PINH1_SET() SET_PIN(PINH, 1)
	#define PINH2_SET() SET_PIN(PINH, 2)
	#define PINH3_SET() SET_PIN(PINH, 3)
	#define PINH4_SET() SET_PIN(PINH, 4)
	#define PINH5_SET() SET_PIN(PINH, 5)
	#define PINH6_SET() SET_PIN(PINH, 6)
	#define PINH7_SET() SET_PIN(PINH, 7)


	#define DDRH0_CLEAR() CLEAR_PIN(DDRH, 0)
	#define DDRH1_CLEAR() CLEAR_PIN(DDRH, 1)
	#define DDRH2_CLEAR() CLEAR_PIN(DDRH, 2)
	#define DDRH3_CLEAR() CLEAR_PIN(DDRH, 3)
	#define DDRH4_CLEAR() CLEAR_PIN(DDRH, 4)
	#define DDRH5_CLEAR() CLEAR_PIN(DDRH, 5)
	#define DDRH6_CLEAR() CLEAR_PIN(DDRH, 6)
	#define DDRH7_CLEAR() CLEAR_PIN(DDRH, 7)

	#define PORTH0_CLEAR() CLEAR_PIN(PORTH, 0)
	#define PORTH1_CLEAR() CLEAR_PIN(PORTH, 1)
	#define PORTH2_CLEAR() CLEAR_PIN(PORTH, 2)
	#define PORTH3_CLEAR() CLEAR_PIN(PORTH, 3)
	#define PORTH4_CLEAR() CLEAR_PIN(PORTH, 4)
	#define PORTH5_CLEAR() CLEAR_PIN(PORTH, 5)
	#define PORTH6_CLEAR() CLEAR_PIN(PORTH, 6)
	#define PORTH7_CLEAR() CLEAR_PIN(PORTH, 7)

	#define PINH0_CLEAR() CLEAR_PIN(PINH, 0)
	#define PINH1_CLEAR() CLEAR_PIN(PINH, 1)
	#define PINH2_CLEAR() CLEAR_PIN(PINH, 2)
	#define PINH3_CLEAR() CLEAR_PIN(PINH, 3)
	#define PINH4_CLEAR() CLEAR_PIN(PINH, 4)
	#define PINH5_CLEAR() CLEAR_PIN(PINH, 5)
	#define PINH6_CLEAR() CLEAR_PIN(PINH, 6)
	#define PINH7_CLEAR() CLEAR_PIN(PINH, 7)


	#define DDRH0_TOGGLE() TOGGLE_PIN(DDRH, 0)
	#define DDRH1_TOGGLE() TOGGLE_PIN(DDRH, 1)
	#define DDRH2_TOGGLE() TOGGLE_PIN(DDRH, 2)
	#define DDRH3_TOGGLE() TOGGLE_PIN(DDRH, 3)
	#define DDRH4_TOGGLE() TOGGLE_PIN(DDRH, 4)
	#define DDRH5_TOGGLE() TOGGLE_PIN(DDRH, 5)
	#define DDRH6_TOGGLE() TOGGLE_PIN(DDRH, 6)
	#define DDRH7_TOGGLE() TOGGLE_PIN(DDRH, 7)

	#define PORTH0_TOGGLE() TOGGLE_PIN(PORTH, 0)
	#define PORTH1_TOGGLE() TOGGLE_PIN(PORTH, 1)
	#define PORTH2_TOGGLE() TOGGLE_PIN(PORTH, 2)
	#define PORTH3_TOGGLE() TOGGLE_PIN(PORTH, 3)
	#define PORTH4_TOGGLE() TOGGLE_PIN(PORTH, 4)
	#define PORTH5_TOGGLE() TOGGLE_PIN(PORTH, 5)
	#define PORTH6_TOGGLE() TOGGLE_PIN(PORTH, 6)
	#define PORTH7_TOGGLE() TOGGLE_PIN(PORTH, 7)

	#define PINH0_TOGGLE() TOGGLE_PIN(PINH, 0)
	#define PINH1_TOGGLE() TOGGLE_PIN(PINH, 1)
	#define PINH2_TOGGLE() TOGGLE_PIN(PINH, 2)
	#define PINH3_TOGGLE() TOGGLE_PIN(PINH, 3)
	#define PINH4_TOGGLE() TOGGLE_PIN(PINH, 4)
	#define PINH5_TOGGLE() TOGGLE_PIN(PINH, 5)
	#define PINH6_TOGGLE() TOGGLE_PIN(PINH, 6)
	#define PINH7_TOGGLE() TOGGLE_PIN(PINH, 7)


	#define PH0_CHECK() PIN_IN_CHECK(H, 0)
	#define PH1_CHECK() PIN_IN_CHECK(H, 1)
	#define PH2_CHECK() PIN_IN_CHECK(H, 2)
	#define PH3_CHECK() PIN_IN_CHECK(H, 3)
	#define PH4_CHECK() PIN_IN_CHECK(H, 4)
	#define PH5_CHECK() PIN_IN_CHECK(H, 5)
	#define PH6_CHECK() PIN_IN_CHECK(H, 6)
	#define PH7_CHECK() PIN_IN_CHECK(H, 7)

	#define PH0_IS_SET() PIN_IN_CHECK_SET(H, 0)
	#define PH1_IS_SET() PIN_IN_CHECK_SET(H, 1)
	#define PH2_IS_SET() PIN_IN_CHECK_SET(H, 2)
	#define PH3_IS_SET() PIN_IN_CHECK_SET(H, 3)
	#define PH4_IS_SET() PIN_IN_CHECK_SET(H, 4)
	#define PH5_IS_SET() PIN_IN_CHECK_SET(H, 5)
	#define PH6_IS_SET() PIN_IN_CHECK_SET(H, 6)
	#define PH7_IS_SET() PIN_IN_CHECK_SET(H, 7)

	#define PH0_IS_UNSET() PIN_IN_CHECK_UNSET(H, 0)
	#define PH1_IS_UNSET() PIN_IN_CHECK_UNSET(H, 1)
	#define PH2_IS_UNSET() PIN_IN_CHECK_UNSET(H, 2)
	#define PH3_IS_UNSET() PIN_IN_CHECK_UNSET(H, 3)
	#define PH4_IS_UNSET() PIN_IN_CHECK_UNSET(H, 4)
	#define PH5_IS_UNSET() PIN_IN_CHECK_UNSET(H, 5)
	#define PH6_IS_UNSET() PIN_IN_CHECK_UNSET(H, 6)
	#define PH7_IS_UNSET() PIN_IN_CHECK_UNSET(H, 7)


	#define PH0_SET() PIN_OUT_SET(H, 0)
	#define PH1_SET() PIN_OUT_SET(H, 1)
	#define PH2_SET() PIN_OUT_SET(H, 2)
	#define PH3_SET() PIN_OUT_SET(H, 3)
	#define PH4_SET() PIN_OUT_SET(H, 4)
	#define PH5_SET() PIN_OUT_SET(H, 5)
	#define PH6_SET() PIN_OUT_SET(H, 6)
	#define PH7_SET() PIN_OUT_SET(H, 7)

	#define PH0_CLEAR() PIN_OUT_CLEAR(H, 0)
	#define PH1_CLEAR() PIN_OUT_CLEAR(H, 1)
	#define PH2_CLEAR() PIN_OUT_CLEAR(H, 2)
	#define PH3_CLEAR() PIN_OUT_CLEAR(H, 3)
	#define PH4_CLEAR() PIN_OUT_CLEAR(H, 4)
	#define PH5_CLEAR() PIN_OUT_CLEAR(H, 5)
	#define PH6_CLEAR() PIN_OUT_CLEAR(H, 6)
	#define PH7_CLEAR() PIN_OUT_CLEAR(H, 7)

	#define PH0_TOGGLE() PIN_OUT_TOGGLE(H, 0)
	#define PH1_TOGGLE() PIN_OUT_TOGGLE(H, 1)
	#define PH2_TOGGLE() PIN_OUT_TOGGLE(H, 2)
	#define PH3_TOGGLE() PIN_OUT_TOGGLE(H, 3)
	#define PH4_TOGGLE() PIN_OUT_TOGGLE(H, 4)
	#define PH5_TOGGLE() PIN_OUT_TOGGLE(H, 5)
	#define PH6_TOGGLE() PIN_OUT_TOGGLE(H, 6)
	#define PH7_TOGGLE() PIN_OUT_TOGGLE(H, 7)

	#define PH0_RESET() PIN_RESET(H, 0)
	#define PH1_RESET() PIN_RESET(H, 1)
	#define PH2_RESET() PIN_RESET(H, 2)
	#define PH3_RESET() PIN_RESET(H, 3)
	#define PH4_RESET() PIN_RESET(H, 4)
	#define PH5_RESET() PIN_RESET(H, 5)
	#define PH6_RESET() PIN_RESET(H, 6)
	#define PH7_RESET() PIN_RESET(H, 7)

	#define PH0_GET_MODE() PIN_GET_MODE(H, 0)
	#define PH1_GET_MODE() PIN_GET_MODE(H, 1)
	#define PH2_GET_MODE() PIN_GET_MODE(H, 2)
	#define PH3_GET_MODE() PIN_GET_MODE(H, 3)
	#define PH4_GET_MODE() PIN_GET_MODE(H, 4)
	#define PH5_GET_MODE() PIN_GET_MODE(H, 5)
	#define PH6_GET_MODE() PIN_GET_MODE(H, 6)
	#define PH7_GET_MODE() PIN_GET_MODE(H, 7)

	#define PH0_SET_MODE(mode) PIN_SET_MODE(H, 0, mode)
	#define PH1_SET_MODE(mode) PIN_SET_MODE(H, 1, mode)
	#define PH2_SET_MODE(mode) PIN_SET_MODE(H, 2, mode)
	#define PH3_SET_MODE(mode) PIN_SET_MODE(H, 3, mode)
	#define PH4_SET_MODE(mode) PIN_SET_MODE(H, 4, mode)
	#define PH5_SET_MODE(mode) PIN_SET_MODE(H, 5, mode)
	#define PH6_SET_MODE(mode) PIN_SET_MODE(H, 6, mode)
	#define PH7_SET_MODE(mode) PIN_SET_MODE(H, 7, mode)

#endif


#ifdef PORTJ

	#define DDRJ_0 PIN_REF(DDRJ, 0)
	#define DDRJ_1 PIN_REF(DDRJ, 1)
	#define DDRJ_2 PIN_REF(DDRJ, 2)
	#define DDRJ_3 PIN_REF(DDRJ, 3)
	#define DDRJ_4 PIN_REF(DDRJ, 4)
	#define DDRJ_5 PIN_REF(DDRJ, 5)
	#define DDRJ_6 PIN_REF(DDRJ, 6)
	#define DDRJ_7 PIN_REF(DDRJ, 7)

	#define PORTJ_0 PIN_REF(PORTJ, 0)
	#define PORTJ_1 PIN_REF(PORTJ, 1)
	#define PORTJ_2 PIN_REF(PORTJ, 2)
	#define PORTJ_3 PIN_REF(PORTJ, 3)
	#define PORTJ_4 PIN_REF(PORTJ, 4)
	#define PORTJ_5 PIN_REF(PORTJ, 5)
	#define PORTJ_6 PIN_REF(PORTJ, 6)
	#define PORTJ_7 PIN_REF(PORTJ, 7)

	#define PINJ_0 PIN_REF(PINJ, 0)
	#define PINJ_1 PIN_REF(PINJ, 1)
	#define PINJ_2 PIN_REF(PINJ, 2)
	#define PINJ_3 PIN_REF(PINJ, 3)
	#define PINJ_4 PIN_REF(PINJ, 4)
	#define PINJ_5 PIN_REF(PINJ, 5)
	#define PINJ_6 PIN_REF(PINJ, 6)
	#define PINJ_7 PIN_REF(PINJ, 7)


	#define DDRJ0_PAIR() PORT_PIN_PAIR(DDRJ, 0)
	#define DDRJ1_PAIR() PORT_PIN_PAIR(DDRJ, 1)
	#define DDRJ2_PAIR() PORT_PIN_PAIR(DDRJ, 2)
	#define DDRJ3_PAIR() PORT_PIN_PAIR(DDRJ, 3)
	#define DDRJ4_PAIR() PORT_PIN_PAIR(DDRJ, 4)
	#define DDRJ5_PAIR() PORT_PIN_PAIR(DDRJ, 5)
	#define DDRJ6_PAIR() PORT_PIN_PAIR(DDRJ, 6)
	#define DDRJ7_PAIR() PORT_PIN_PAIR(DDRJ, 7)

	#define PORTJ0_PAIR() PORT_PIN_PAIR(PORTJ, 0)
	#define PORTJ1_PAIR() PORT_PIN_PAIR(PORTJ, 1)
	#define PORTJ2_PAIR() PORT_PIN_PAIR(PORTJ, 2)
	#define PORTJ3_PAIR() PORT_PIN_PAIR(PORTJ, 3)
	#define PORTJ4_PAIR() PORT_PIN_PAIR(PORTJ, 4)
	#define PORTJ5_PAIR() PORT_PIN_PAIR(PORTJ, 5)
	#define PORTJ6_PAIR() PORT_PIN_PAIR(PORTJ, 6)
	#define PORTJ7_PAIR() PORT_PIN_PAIR(PORTJ, 7)

	#define PINJ0_PAIR() PORT_PIN_PAIR(PINJ, 0)
	#define PINJ1_PAIR() PORT_PIN_PAIR(PINJ, 1)
	#define PINJ2_PAIR() PORT_PIN_PAIR(PINJ, 2)
	#define PINJ3_PAIR() PORT_PIN_PAIR(PINJ, 3)
	#define PINJ4_PAIR() PORT_PIN_PAIR(PINJ, 4)
	#define PINJ5_PAIR() PORT_PIN_PAIR(PINJ, 5)
	#define PINJ6_PAIR() PORT_PIN_PAIR(PINJ, 6)
	#define PINJ7_PAIR() PORT_PIN_PAIR(PINJ, 7)


	#define DDRJ0_CHECK() CHECK_PIN(DDRJ, 0)
	#define DDRJ1_CHECK() CHECK_PIN(DDRJ, 1)
	#define DDRJ2_CHECK() CHECK_PIN(DDRJ, 2)
	#define DDRJ3_CHECK() CHECK_PIN(DDRJ, 3)
	#define DDRJ4_CHECK() CHECK_PIN(DDRJ, 4)
	#define DDRJ5_CHECK() CHECK_PIN(DDRJ, 5)
	#define DDRJ6_CHECK() CHECK_PIN(DDRJ, 6)
	#define DDRJ7_CHECK() CHECK_PIN(DDRJ, 7)

	#define PORTJ0_CHECK() CHECK_PIN(PORTJ, 0)
	#define PORTJ1_CHECK() CHECK_PIN(PORTJ, 1)
	#define PORTJ2_CHECK() CHECK_PIN(PORTJ, 2)
	#define PORTJ3_CHECK() CHECK_PIN(PORTJ, 3)
	#define PORTJ4_CHECK() CHECK_PIN(PORTJ, 4)
	#define PORTJ5_CHECK() CHECK_PIN(PORTJ, 5)
	#define PORTJ6_CHECK() CHECK_PIN(PORTJ, 6)
	#define PORTJ7_CHECK() CHECK_PIN(PORTJ, 7)

	#define PINJ0_CHECK() CHECK_PIN(PINJ, 0)
	#define PINJ1_CHECK() CHECK_PIN(PINJ, 1)
	#define PINJ2_CHECK() CHECK_PIN(PINJ, 2)
	#define PINJ3_CHECK() CHECK_PIN(PINJ, 3)
	#define PINJ4_CHECK() CHECK_PIN(PINJ, 4)
	#define PINJ5_CHECK() CHECK_PIN(PINJ, 5)
	#define PINJ6_CHECK() CHECK_PIN(PINJ, 6)
	#define PINJ7_CHECK() CHECK_PIN(PINJ, 7)


	#define DDRJ0_IS_SET() CHECK_SET_PIN(DDRJ, 0)
	#define DDRJ1_IS_SET() CHECK_SET_PIN(DDRJ, 1)
	#define DDRJ2_IS_SET() CHECK_SET_PIN(DDRJ, 2)
	#define DDRJ3_IS_SET() CHECK_SET_PIN(DDRJ, 3)
	#define DDRJ4_IS_SET() CHECK_SET_PIN(DDRJ, 4)
	#define DDRJ5_IS_SET() CHECK_SET_PIN(DDRJ, 5)
	#define DDRJ6_IS_SET() CHECK_SET_PIN(DDRJ, 6)
	#define DDRJ7_IS_SET() CHECK_SET_PIN(DDRJ, 7)

	#define PORTJ0_IS_SET() CHECK_SET_PIN(PORTJ, 0)
	#define PORTJ1_IS_SET() CHECK_SET_PIN(PORTJ, 1)
	#define PORTJ2_IS_SET() CHECK_SET_PIN(PORTJ, 2)
	#define PORTJ3_IS_SET() CHECK_SET_PIN(PORTJ, 3)
	#define PORTJ4_IS_SET() CHECK_SET_PIN(PORTJ, 4)
	#define PORTJ5_IS_SET() CHECK_SET_PIN(PORTJ, 5)
	#define PORTJ6_IS_SET() CHECK_SET_PIN(PORTJ, 6)
	#define PORTJ7_IS_SET() CHECK_SET_PIN(PORTJ, 7)

	#define PINJ0_IS_SET() CHECK_SET_PIN(PINJ, 0)
	#define PINJ1_IS_SET() CHECK_SET_PIN(PINJ, 1)
	#define PINJ2_IS_SET() CHECK_SET_PIN(PINJ, 2)
	#define PINJ3_IS_SET() CHECK_SET_PIN(PINJ, 3)
	#define PINJ4_IS_SET() CHECK_SET_PIN(PINJ, 4)
	#define PINJ5_IS_SET() CHECK_SET_PIN(PINJ, 5)
	#define PINJ6_IS_SET() CHECK_SET_PIN(PINJ, 6)
	#define PINJ7_IS_SET() CHECK_SET_PIN(PINJ, 7)


	#define DDRJ0_IS_UNSET() CHECK_UNSET_PIN(DDRJ, 0)
	#define DDRJ1_IS_UNSET() CHECK_UNSET_PIN(DDRJ, 1)
	#define DDRJ2_IS_UNSET() CHECK_UNSET_PIN(DDRJ, 2)
	#define DDRJ3_IS_UNSET() CHECK_UNSET_PIN(DDRJ, 3)
	#define DDRJ4_IS_UNSET() CHECK_UNSET_PIN(DDRJ, 4)
	#define DDRJ5_IS_UNSET() CHECK_UNSET_PIN(DDRJ, 5)
	#define DDRJ6_IS_UNSET() CHECK_UNSET_PIN(DDRJ, 6)
	#define DDRJ7_IS_UNSET() CHECK_UNSET_PIN(DDRJ, 7)

	#define PORTJ0_IS_UNSET() CHECK_UNSET_PIN(PORTJ, 0)
	#define PORTJ1_IS_UNSET() CHECK_UNSET_PIN(PORTJ, 1)
	#define PORTJ2_IS_UNSET() CHECK_UNSET_PIN(PORTJ, 2)
	#define PORTJ3_IS_UNSET() CHECK_UNSET_PIN(PORTJ, 3)
	#define PORTJ4_IS_UNSET() CHECK_UNSET_PIN(PORTJ, 4)
	#define PORTJ5_IS_UNSET() CHECK_UNSET_PIN(PORTJ, 5)
	#define PORTJ6_IS_UNSET() CHECK_UNSET_PIN(PORTJ, 6)
	#define PORTJ7_IS_UNSET() CHECK_UNSET_PIN(PORTJ, 7)

	#define PINJ0_IS_UNSET() CHECK_UNSET_PIN(PINJ, 0)
	#define PINJ1_IS_UNSET() CHECK_UNSET_PIN(PINJ, 1)
	#define PINJ2_IS_UNSET() CHECK_UNSET_PIN(PINJ, 2)
	#define PINJ3_IS_UNSET() CHECK_UNSET_PIN(PINJ, 3)
	#define PINJ4_IS_UNSET() CHECK_UNSET_PIN(PINJ, 4)
	#define PINJ5_IS_UNSET() CHECK_UNSET_PIN(PINJ, 5)
	#define PINJ6_IS_UNSET() CHECK_UNSET_PIN(PINJ, 6)
	#define PINJ7_IS_UNSET() CHECK_UNSET_PIN(PINJ, 7)


	#define DDRJ0_SET() SET_PIN(DDRJ, 0)
	#define DDRJ1_SET() SET_PIN(DDRJ, 1)
	#define DDRJ2_SET() SET_PIN(DDRJ, 2)
	#define DDRJ3_SET() SET_PIN(DDRJ, 3)
	#define DDRJ4_SET() SET_PIN(DDRJ, 4)
	#define DDRJ5_SET() SET_PIN(DDRJ, 5)
	#define DDRJ6_SET() SET_PIN(DDRJ, 6)
	#define DDRJ7_SET() SET_PIN(DDRJ, 7)

	#define PORTJ0_SET() SET_PIN(PORTJ, 0)
	#define PORTJ1_SET() SET_PIN(PORTJ, 1)
	#define PORTJ2_SET() SET_PIN(PORTJ, 2)
	#define PORTJ3_SET() SET_PIN(PORTJ, 3)
	#define PORTJ4_SET() SET_PIN(PORTJ, 4)
	#define PORTJ5_SET() SET_PIN(PORTJ, 5)
	#define PORTJ6_SET() SET_PIN(PORTJ, 6)
	#define PORTJ7_SET() SET_PIN(PORTJ, 7)

	#define PINJ0_SET() SET_PIN(PINJ, 0)
	#define PINJ1_SET() SET_PIN(PINJ, 1)
	#define PINJ2_SET() SET_PIN(PINJ, 2)
	#define PINJ3_SET() SET_PIN(PINJ, 3)
	#define PINJ4_SET() SET_PIN(PINJ, 4)
	#define PINJ5_SET() SET_PIN(PINJ, 5)
	#define PINJ6_SET() SET_PIN(PINJ, 6)
	#define PINJ7_SET() SET_PIN(PINJ, 7)


	#define DDRJ0_CLEAR() CLEAR_PIN(DDRJ, 0)
	#define DDRJ1_CLEAR() CLEAR_PIN(DDRJ, 1)
	#define DDRJ2_CLEAR() CLEAR_PIN(DDRJ, 2)
	#define DDRJ3_CLEAR() CLEAR_PIN(DDRJ, 3)
	#define DDRJ4_CLEAR() CLEAR_PIN(DDRJ, 4)
	#define DDRJ5_CLEAR() CLEAR_PIN(DDRJ, 5)
	#define DDRJ6_CLEAR() CLEAR_PIN(DDRJ, 6)
	#define DDRJ7_CLEAR() CLEAR_PIN(DDRJ, 7)

	#define PORTJ0_CLEAR() CLEAR_PIN(PORTJ, 0)
	#define PORTJ1_CLEAR() CLEAR_PIN(PORTJ, 1)
	#define PORTJ2_CLEAR() CLEAR_PIN(PORTJ, 2)
	#define PORTJ3_CLEAR() CLEAR_PIN(PORTJ, 3)
	#define PORTJ4_CLEAR() CLEAR_PIN(PORTJ, 4)
	#define PORTJ5_CLEAR() CLEAR_PIN(PORTJ, 5)
	#define PORTJ6_CLEAR() CLEAR_PIN(PORTJ, 6)
	#define PORTJ7_CLEAR() CLEAR_PIN(PORTJ, 7)

	#define PINJ0_CLEAR() CLEAR_PIN(PINJ, 0)
	#define PINJ1_CLEAR() CLEAR_PIN(PINJ, 1)
	#define PINJ2_CLEAR() CLEAR_PIN(PINJ, 2)
	#define PINJ3_CLEAR() CLEAR_PIN(PINJ, 3)
	#define PINJ4_CLEAR() CLEAR_PIN(PINJ, 4)
	#define PINJ5_CLEAR() CLEAR_PIN(PINJ, 5)
	#define PINJ6_CLEAR() CLEAR_PIN(PINJ, 6)
	#define PINJ7_CLEAR() CLEAR_PIN(PINJ, 7)


	#define DDRJ0_TOGGLE() TOGGLE_PIN(DDRJ, 0)
	#define DDRJ1_TOGGLE() TOGGLE_PIN(DDRJ, 1)
	#define DDRJ2_TOGGLE() TOGGLE_PIN(DDRJ, 2)
	#define DDRJ3_TOGGLE() TOGGLE_PIN(DDRJ, 3)
	#define DDRJ4_TOGGLE() TOGGLE_PIN(DDRJ, 4)
	#define DDRJ5_TOGGLE() TOGGLE_PIN(DDRJ, 5)
	#define DDRJ6_TOGGLE() TOGGLE_PIN(DDRJ, 6)
	#define DDRJ7_TOGGLE() TOGGLE_PIN(DDRJ, 7)

	#define PORTJ0_TOGGLE() TOGGLE_PIN(PORTJ, 0)
	#define PORTJ1_TOGGLE() TOGGLE_PIN(PORTJ, 1)
	#define PORTJ2_TOGGLE() TOGGLE_PIN(PORTJ, 2)
	#define PORTJ3_TOGGLE() TOGGLE_PIN(PORTJ, 3)
	#define PORTJ4_TOGGLE() TOGGLE_PIN(PORTJ, 4)
	#define PORTJ5_TOGGLE() TOGGLE_PIN(PORTJ, 5)
	#define PORTJ6_TOGGLE() TOGGLE_PIN(PORTJ, 6)
	#define PORTJ7_TOGGLE() TOGGLE_PIN(PORTJ, 7)

	#define PINJ0_TOGGLE() TOGGLE_PIN(PINJ, 0)
	#define PINJ1_TOGGLE() TOGGLE_PIN(PINJ, 1)
	#define PINJ2_TOGGLE() TOGGLE_PIN(PINJ, 2)
	#define PINJ3_TOGGLE() TOGGLE_PIN(PINJ, 3)
	#define PINJ4_TOGGLE() TOGGLE_PIN(PINJ, 4)
	#define PINJ5_TOGGLE() TOGGLE_PIN(PINJ, 5)
	#define PINJ6_TOGGLE() TOGGLE_PIN(PINJ, 6)
	#define PINJ7_TOGGLE() TOGGLE_PIN(PINJ, 7)


	#define PJ0_CHECK() PIN_IN_CHECK(J, 0)
	#define PJ1_CHECK() PIN_IN_CHECK(J, 1)
	#define PJ2_CHECK() PIN_IN_CHECK(J, 2)
	#define PJ3_CHECK() PIN_IN_CHECK(J, 3)
	#define PJ4_CHECK() PIN_IN_CHECK(J, 4)
	#define PJ5_CHECK() PIN_IN_CHECK(J, 5)
	#define PJ6_CHECK() PIN_IN_CHECK(J, 6)
	#define PJ7_CHECK() PIN_IN_CHECK(J, 7)

	#define PJ0_IS_SET() PIN_IN_CHECK_SET(J, 0)
	#define PJ1_IS_SET() PIN_IN_CHECK_SET(J, 1)
	#define PJ2_IS_SET() PIN_IN_CHECK_SET(J, 2)
	#define PJ3_IS_SET() PIN_IN_CHECK_SET(J, 3)
	#define PJ4_IS_SET() PIN_IN_CHECK_SET(J, 4)
	#define PJ5_IS_SET() PIN_IN_CHECK_SET(J, 5)
	#define PJ6_IS_SET() PIN_IN_CHECK_SET(J, 6)
	#define PJ7_IS_SET() PIN_IN_CHECK_SET(J, 7)

	#define PJ0_IS_UNSET() PIN_IN_CHECK_UNSET(J, 0)
	#define PJ1_IS_UNSET() PIN_IN_CHECK_UNSET(J, 1)
	#define PJ2_IS_UNSET() PIN_IN_CHECK_UNSET(J, 2)
	#define PJ3_IS_UNSET() PIN_IN_CHECK_UNSET(J, 3)
	#define PJ4_IS_UNSET() PIN_IN_CHECK_UNSET(J, 4)
	#define PJ5_IS_UNSET() PIN_IN_CHECK_UNSET(J, 5)
	#define PJ6_IS_UNSET() PIN_IN_CHECK_UNSET(J, 6)
	#define PJ7_IS_UNSET() PIN_IN_CHECK_UNSET(J, 7)


	#define PJ0_SET() PIN_OUT_SET(J, 0)
	#define PJ1_SET() PIN_OUT_SET(J, 1)
	#define PJ2_SET() PIN_OUT_SET(J, 2)
	#define PJ3_SET() PIN_OUT_SET(J, 3)
	#define PJ4_SET() PIN_OUT_SET(J, 4)
	#define PJ5_SET() PIN_OUT_SET(J, 5)
	#define PJ6_SET() PIN_OUT_SET(J, 6)
	#define PJ7_SET() PIN_OUT_SET(J, 7)

	#define PJ0_CLEAR() PIN_OUT_CLEAR(J, 0)
	#define PJ1_CLEAR() PIN_OUT_CLEAR(J, 1)
	#define PJ2_CLEAR() PIN_OUT_CLEAR(J, 2)
	#define PJ3_CLEAR() PIN_OUT_CLEAR(J, 3)
	#define PJ4_CLEAR() PIN_OUT_CLEAR(J, 4)
	#define PJ5_CLEAR() PIN_OUT_CLEAR(J, 5)
	#define PJ6_CLEAR() PIN_OUT_CLEAR(J, 6)
	#define PJ7_CLEAR() PIN_OUT_CLEAR(J, 7)

	#define PJ0_TOGGLE() PIN_OUT_TOGGLE(J, 0)
	#define PJ1_TOGGLE() PIN_OUT_TOGGLE(J, 1)
	#define PJ2_TOGGLE() PIN_OUT_TOGGLE(J, 2)
	#define PJ3_TOGGLE() PIN_OUT_TOGGLE(J, 3)
	#define PJ4_TOGGLE() PIN_OUT_TOGGLE(J, 4)
	#define PJ5_TOGGLE() PIN_OUT_TOGGLE(J, 5)
	#define PJ6_TOGGLE() PIN_OUT_TOGGLE(J, 6)
	#define PJ7_TOGGLE() PIN_OUT_TOGGLE(J, 7)

	#define PJ0_RESET() PIN_RESET(J, 0)
	#define PJ1_RESET() PIN_RESET(J, 1)
	#define PJ2_RESET() PIN_RESET(J, 2)
	#define PJ3_RESET() PIN_RESET(J, 3)
	#define PJ4_RESET() PIN_RESET(J, 4)
	#define PJ5_RESET() PIN_RESET(J, 5)
	#define PJ6_RESET() PIN_RESET(J, 6)
	#define PJ7_RESET() PIN_RESET(J, 7)

	#define PJ0_GET_MODE() PIN_GET_MODE(J, 0)
	#define PJ1_GET_MODE() PIN_GET_MODE(J, 1)
	#define PJ2_GET_MODE() PIN_GET_MODE(J, 2)
	#define PJ3_GET_MODE() PIN_GET_MODE(J, 3)
	#define PJ4_GET_MODE() PIN_GET_MODE(J, 4)
	#define PJ5_GET_MODE() PIN_GET_MODE(J, 5)
	#define PJ6_GET_MODE() PIN_GET_MODE(J, 6)
	#define PJ7_GET_MODE() PIN_GET_MODE(J, 7)

	#define PJ0_SET_MODE(mode) PIN_SET_MODE(J, 0, mode)
	#define PJ1_SET_MODE(mode) PIN_SET_MODE(J, 1, mode)
	#define PJ2_SET_MODE(mode) PIN_SET_MODE(J, 2, mode)
	#define PJ3_SET_MODE(mode) PIN_SET_MODE(J, 3, mode)
	#define PJ4_SET_MODE(mode) PIN_SET_MODE(J, 4, mode)
	#define PJ5_SET_MODE(mode) PIN_SET_MODE(J, 5, mode)
	#define PJ6_SET_MODE(mode) PIN_SET_MODE(J, 6, mode)
	#define PJ7_SET_MODE(mode) PIN_SET_MODE(J, 7, mode)

#endif


#ifdef PORTK

	#define DDRK_0 PIN_REF(DDRK, 0)
	#define DDRK_1 PIN_REF(DDRK, 1)
	#define DDRK_2 PIN_REF(DDRK, 2)
	#define DDRK_3 PIN_REF(DDRK, 3)
	#define DDRK_4 PIN_REF(DDRK, 4)
	#define DDRK_5 PIN_REF(DDRK, 5)
	#define DDRK_6 PIN_REF(DDRK, 6)
	#define DDRK_7 PIN_REF(DDRK, 7)

	#define PORTK_0 PIN_REF(PORTK, 0)
	#define PORTK_1 PIN_REF(PORTK, 1)
	#define PORTK_2 PIN_REF(PORTK, 2)
	#define PORTK_3 PIN_REF(PORTK, 3)
	#define PORTK_4 PIN_REF(PORTK, 4)
	#define PORTK_5 PIN_REF(PORTK, 5)
	#define PORTK_6 PIN_REF(PORTK, 6)
	#define PORTK_7 PIN_REF(PORTK, 7)

	#define PINK_0 PIN_REF(PINK, 0)
	#define PINK_1 PIN_REF(PINK, 1)
	#define PINK_2 PIN_REF(PINK, 2)
	#define PINK_3 PIN_REF(PINK, 3)
	#define PINK_4 PIN_REF(PINK, 4)
	#define PINK_5 PIN_REF(PINK, 5)
	#define PINK_6 PIN_REF(PINK, 6)
	#define PINK_7 PIN_REF(PINK, 7)


	#define DDRK0_PAIR() PORT_PIN_PAIR(DDRK, 0)
	#define DDRK1_PAIR() PORT_PIN_PAIR(DDRK, 1)
	#define DDRK2_PAIR() PORT_PIN_PAIR(DDRK, 2)
	#define DDRK3_PAIR() PORT_PIN_PAIR(DDRK, 3)
	#define DDRK4_PAIR() PORT_PIN_PAIR(DDRK, 4)
	#define DDRK5_PAIR() PORT_PIN_PAIR(DDRK, 5)
	#define DDRK6_PAIR() PORT_PIN_PAIR(DDRK, 6)
	#define DDRK7_PAIR() PORT_PIN_PAIR(DDRK, 7)

	#define PORTK0_PAIR() PORT_PIN_PAIR(PORTK, 0)
	#define PORTK1_PAIR() PORT_PIN_PAIR(PORTK, 1)
	#define PORTK2_PAIR() PORT_PIN_PAIR(PORTK, 2)
	#define PORTK3_PAIR() PORT_PIN_PAIR(PORTK, 3)
	#define PORTK4_PAIR() PORT_PIN_PAIR(PORTK, 4)
	#define PORTK5_PAIR() PORT_PIN_PAIR(PORTK, 5)
	#define PORTK6_PAIR() PORT_PIN_PAIR(PORTK, 6)
	#define PORTK7_PAIR() PORT_PIN_PAIR(PORTK, 7)

	#define PINK0_PAIR() PORT_PIN_PAIR(PINK, 0)
	#define PINK1_PAIR() PORT_PIN_PAIR(PINK, 1)
	#define PINK2_PAIR() PORT_PIN_PAIR(PINK, 2)
	#define PINK3_PAIR() PORT_PIN_PAIR(PINK, 3)
	#define PINK4_PAIR() PORT_PIN_PAIR(PINK, 4)
	#define PINK5_PAIR() PORT_PIN_PAIR(PINK, 5)
	#define PINK6_PAIR() PORT_PIN_PAIR(PINK, 6)
	#define PINK7_PAIR() PORT_PIN_PAIR(PINK, 7)


	#define DDRK0_CHECK() CHECK_PIN(DDRK, 0)
	#define DDRK1_CHECK() CHECK_PIN(DDRK, 1)
	#define DDRK2_CHECK() CHECK_PIN(DDRK, 2)
	#define DDRK3_CHECK() CHECK_PIN(DDRK, 3)
	#define DDRK4_CHECK() CHECK_PIN(DDRK, 4)
	#define DDRK5_CHECK() CHECK_PIN(DDRK, 5)
	#define DDRK6_CHECK() CHECK_PIN(DDRK, 6)
	#define DDRK7_CHECK() CHECK_PIN(DDRK, 7)

	#define PORTK0_CHECK() CHECK_PIN(PORTK, 0)
	#define PORTK1_CHECK() CHECK_PIN(PORTK, 1)
	#define PORTK2_CHECK() CHECK_PIN(PORTK, 2)
	#define PORTK3_CHECK() CHECK_PIN(PORTK, 3)
	#define PORTK4_CHECK() CHECK_PIN(PORTK, 4)
	#define PORTK5_CHECK() CHECK_PIN(PORTK, 5)
	#define PORTK6_CHECK() CHECK_PIN(PORTK, 6)
	#define PORTK7_CHECK() CHECK_PIN(PORTK, 7)

	#define PINK0_CHECK() CHECK_PIN(PINK, 0)
	#define PINK1_CHECK() CHECK_PIN(PINK, 1)
	#define PINK2_CHECK() CHECK_PIN(PINK, 2)
	#define PINK3_CHECK() CHECK_PIN(PINK, 3)
	#define PINK4_CHECK() CHECK_PIN(PINK, 4)
	#define PINK5_CHECK() CHECK_PIN(PINK, 5)
	#define PINK6_CHECK() CHECK_PIN(PINK, 6)
	#define PINK7_CHECK() CHECK_PIN(PINK, 7)


	#define DDRK0_IS_SET() CHECK_SET_PIN(DDRK, 0)
	#define DDRK1_IS_SET() CHECK_SET_PIN(DDRK, 1)
	#define DDRK2_IS_SET() CHECK_SET_PIN(DDRK, 2)
	#define DDRK3_IS_SET() CHECK_SET_PIN(DDRK, 3)
	#define DDRK4_IS_SET() CHECK_SET_PIN(DDRK, 4)
	#define DDRK5_IS_SET() CHECK_SET_PIN(DDRK, 5)
	#define DDRK6_IS_SET() CHECK_SET_PIN(DDRK, 6)
	#define DDRK7_IS_SET() CHECK_SET_PIN(DDRK, 7)

	#define PORTK0_IS_SET() CHECK_SET_PIN(PORTK, 0)
	#define PORTK1_IS_SET() CHECK_SET_PIN(PORTK, 1)
	#define PORTK2_IS_SET() CHECK_SET_PIN(PORTK, 2)
	#define PORTK3_IS_SET() CHECK_SET_PIN(PORTK, 3)
	#define PORTK4_IS_SET() CHECK_SET_PIN(PORTK, 4)
	#define PORTK5_IS_SET() CHECK_SET_PIN(PORTK, 5)
	#define PORTK6_IS_SET() CHECK_SET_PIN(PORTK, 6)
	#define PORTK7_IS_SET() CHECK_SET_PIN(PORTK, 7)

	#define PINK0_IS_SET() CHECK_SET_PIN(PINK, 0)
	#define PINK1_IS_SET() CHECK_SET_PIN(PINK, 1)
	#define PINK2_IS_SET() CHECK_SET_PIN(PINK, 2)
	#define PINK3_IS_SET() CHECK_SET_PIN(PINK, 3)
	#define PINK4_IS_SET() CHECK_SET_PIN(PINK, 4)
	#define PINK5_IS_SET() CHECK_SET_PIN(PINK, 5)
	#define PINK6_IS_SET() CHECK_SET_PIN(PINK, 6)
	#define PINK7_IS_SET() CHECK_SET_PIN(PINK, 7)


	#define DDRK0_IS_UNSET() CHECK_UNSET_PIN(DDRK, 0)
	#define DDRK1_IS_UNSET() CHECK_UNSET_PIN(DDRK, 1)
	#define DDRK2_IS_UNSET() CHECK_UNSET_PIN(DDRK, 2)
	#define DDRK3_IS_UNSET() CHECK_UNSET_PIN(DDRK, 3)
	#define DDRK4_IS_UNSET() CHECK_UNSET_PIN(DDRK, 4)
	#define DDRK5_IS_UNSET() CHECK_UNSET_PIN(DDRK, 5)
	#define DDRK6_IS_UNSET() CHECK_UNSET_PIN(DDRK, 6)
	#define DDRK7_IS_UNSET() CHECK_UNSET_PIN(DDRK, 7)

	#define PORTK0_IS_UNSET() CHECK_UNSET_PIN(PORTK, 0)
	#define PORTK1_IS_UNSET() CHECK_UNSET_PIN(PORTK, 1)
	#define PORTK2_IS_UNSET() CHECK_UNSET_PIN(PORTK, 2)
	#define PORTK3_IS_UNSET() CHECK_UNSET_PIN(PORTK, 3)
	#define PORTK4_IS_UNSET() CHECK_UNSET_PIN(PORTK, 4)
	#define PORTK5_IS_UNSET() CHECK_UNSET_PIN(PORTK, 5)
	#define PORTK6_IS_UNSET() CHECK_UNSET_PIN(PORTK, 6)
	#define PORTK7_IS_UNSET() CHECK_UNSET_PIN(PORTK, 7)

	#define PINK0_IS_UNSET() CHECK_UNSET_PIN(PINK, 0)
	#define PINK1_IS_UNSET() CHECK_UNSET_PIN(PINK, 1)
	#define PINK2_IS_UNSET() CHECK_UNSET_PIN(PINK, 2)
	#define PINK3_IS_UNSET() CHECK_UNSET_PIN(PINK, 3)
	#define PINK4_IS_UNSET() CHECK_UNSET_PIN(PINK, 4)
	#define PINK5_IS_UNSET() CHECK_UNSET_PIN(PINK, 5)
	#define PINK6_IS_UNSET() CHECK_UNSET_PIN(PINK, 6)
	#define PINK7_IS_UNSET() CHECK_UNSET_PIN(PINK, 7)


	#define DDRK0_SET() SET_PIN(DDRK, 0)
	#define DDRK1_SET() SET_PIN(DDRK, 1)
	#define DDRK2_SET() SET_PIN(DDRK, 2)
	#define DDRK3_SET() SET_PIN(DDRK, 3)
	#define DDRK4_SET() SET_PIN(DDRK, 4)
	#define DDRK5_SET() SET_PIN(DDRK, 5)
	#define DDRK6_SET() SET_PIN(DDRK, 6)
	#define DDRK7_SET() SET_PIN(DDRK, 7)

	#define PORTK0_SET() SET_PIN(PORTK, 0)
	#define PORTK1_SET() SET_PIN(PORTK, 1)
	#define PORTK2_SET() SET_PIN(PORTK, 2)
	#define PORTK3_SET() SET_PIN(PORTK, 3)
	#define PORTK4_SET() SET_PIN(PORTK, 4)
	#define PORTK5_SET() SET_PIN(PORTK, 5)
	#define PORTK6_SET() SET_PIN(PORTK, 6)
	#define PORTK7_SET() SET_PIN(PORTK, 7)

	#define PINK0_SET() SET_PIN(PINK, 0)
	#define PINK1_SET() SET_PIN(PINK, 1)
	#define PINK2_SET() SET_PIN(PINK, 2)
	#define PINK3_SET() SET_PIN(PINK, 3)
	#define PINK4_SET() SET_PIN(PINK, 4)
	#define PINK5_SET() SET_PIN(PINK, 5)
	#define PINK6_SET() SET_PIN(PINK, 6)
	#define PINK7_SET() SET_PIN(PINK, 7)


	#define DDRK0_CLEAR() CLEAR_PIN(DDRK, 0)
	#define DDRK1_CLEAR() CLEAR_PIN(DDRK, 1)
	#define DDRK2_CLEAR() CLEAR_PIN(DDRK, 2)
	#define DDRK3_CLEAR() CLEAR_PIN(DDRK, 3)
	#define DDRK4_CLEAR() CLEAR_PIN(DDRK, 4)
	#define DDRK5_CLEAR() CLEAR_PIN(DDRK, 5)
	#define DDRK6_CLEAR() CLEAR_PIN(DDRK, 6)
	#define DDRK7_CLEAR() CLEAR_PIN(DDRK, 7)

	#define PORTK0_CLEAR() CLEAR_PIN(PORTK, 0)
	#define PORTK1_CLEAR() CLEAR_PIN(PORTK, 1)
	#define PORTK2_CLEAR() CLEAR_PIN(PORTK, 2)
	#define PORTK3_CLEAR() CLEAR_PIN(PORTK, 3)
	#define PORTK4_CLEAR() CLEAR_PIN(PORTK, 4)
	#define PORTK5_CLEAR() CLEAR_PIN(PORTK, 5)
	#define PORTK6_CLEAR() CLEAR_PIN(PORTK, 6)
	#define PORTK7_CLEAR() CLEAR_PIN(PORTK, 7)

	#define PINK0_CLEAR() CLEAR_PIN(PINK, 0)
	#define PINK1_CLEAR() CLEAR_PIN(PINK, 1)
	#define PINK2_CLEAR() CLEAR_PIN(PINK, 2)
	#define PINK3_CLEAR() CLEAR_PIN(PINK, 3)
	#define PINK4_CLEAR() CLEAR_PIN(PINK, 4)
	#define PINK5_CLEAR() CLEAR_PIN(PINK, 5)
	#define PINK6_CLEAR() CLEAR_PIN(PINK, 6)
	#define PINK7_CLEAR() CLEAR_PIN(PINK, 7)


	#define DDRK0_TOGGLE() TOGGLE_PIN(DDRK, 0)
	#define DDRK1_TOGGLE() TOGGLE_PIN(DDRK, 1)
	#define DDRK2_TOGGLE() TOGGLE_PIN(DDRK, 2)
	#define DDRK3_TOGGLE() TOGGLE_PIN(DDRK, 3)
	#define DDRK4_TOGGLE() TOGGLE_PIN(DDRK, 4)
	#define DDRK5_TOGGLE() TOGGLE_PIN(DDRK, 5)
	#define DDRK6_TOGGLE() TOGGLE_PIN(DDRK, 6)
	#define DDRK7_TOGGLE() TOGGLE_PIN(DDRK, 7)

	#define PORTK0_TOGGLE() TOGGLE_PIN(PORTK, 0)
	#define PORTK1_TOGGLE() TOGGLE_PIN(PORTK, 1)
	#define PORTK2_TOGGLE() TOGGLE_PIN(PORTK, 2)
	#define PORTK3_TOGGLE() TOGGLE_PIN(PORTK, 3)
	#define PORTK4_TOGGLE() TOGGLE_PIN(PORTK, 4)
	#define PORTK5_TOGGLE() TOGGLE_PIN(PORTK, 5)
	#define PORTK6_TOGGLE() TOGGLE_PIN(PORTK, 6)
	#define PORTK7_TOGGLE() TOGGLE_PIN(PORTK, 7)

	#define PINK0_TOGGLE() TOGGLE_PIN(PINK, 0)
	#define PINK1_TOGGLE() TOGGLE_PIN(PINK, 1)
	#define PINK2_TOGGLE() TOGGLE_PIN(PINK, 2)
	#define PINK3_TOGGLE() TOGGLE_PIN(PINK, 3)
	#define PINK4_TOGGLE() TOGGLE_PIN(PINK, 4)
	#define PINK5_TOGGLE() TOGGLE_PIN(PINK, 5)
	#define PINK6_TOGGLE() TOGGLE_PIN(PINK, 6)
	#define PINK7_TOGGLE() TOGGLE_PIN(PINK, 7)


	#define PK0_CHECK() PIN_IN_CHECK(K, 0)
	#define PK1_CHECK() PIN_IN_CHECK(K, 1)
	#define PK2_CHECK() PIN_IN_CHECK(K, 2)
	#define PK3_CHECK() PIN_IN_CHECK(K, 3)
	#define PK4_CHECK() PIN_IN_CHECK(K, 4)
	#define PK5_CHECK() PIN_IN_CHECK(K, 5)
	#define PK6_CHECK() PIN_IN_CHECK(K, 6)
	#define PK7_CHECK() PIN_IN_CHECK(K, 7)

	#define PK0_IS_SET() PIN_IN_CHECK_SET(K, 0)
	#define PK1_IS_SET() PIN_IN_CHECK_SET(K, 1)
	#define PK2_IS_SET() PIN_IN_CHECK_SET(K, 2)
	#define PK3_IS_SET() PIN_IN_CHECK_SET(K, 3)
	#define PK4_IS_SET() PIN_IN_CHECK_SET(K, 4)
	#define PK5_IS_SET() PIN_IN_CHECK_SET(K, 5)
	#define PK6_IS_SET() PIN_IN_CHECK_SET(K, 6)
	#define PK7_IS_SET() PIN_IN_CHECK_SET(K, 7)

	#define PK0_IS_UNSET() PIN_IN_CHECK_UNSET(K, 0)
	#define PK1_IS_UNSET() PIN_IN_CHECK_UNSET(K, 1)
	#define PK2_IS_UNSET() PIN_IN_CHECK_UNSET(K, 2)
	#define PK3_IS_UNSET() PIN_IN_CHECK_UNSET(K, 3)
	#define PK4_IS_UNSET() PIN_IN_CHECK_UNSET(K, 4)
	#define PK5_IS_UNSET() PIN_IN_CHECK_UNSET(K, 5)
	#define PK6_IS_UNSET() PIN_IN_CHECK_UNSET(K, 6)
	#define PK7_IS_UNSET() PIN_IN_CHECK_UNSET(K, 7)


	#define PK0_SET() PIN_OUT_SET(K, 0)
	#define PK1_SET() PIN_OUT_SET(K, 1)
	#define PK2_SET() PIN_OUT_SET(K, 2)
	#define PK3_SET() PIN_OUT_SET(K, 3)
	#define PK4_SET() PIN_OUT_SET(K, 4)
	#define PK5_SET() PIN_OUT_SET(K, 5)
	#define PK6_SET() PIN_OUT_SET(K, 6)
	#define PK7_SET() PIN_OUT_SET(K, 7)

	#define PK0_CLEAR() PIN_OUT_CLEAR(K, 0)
	#define PK1_CLEAR() PIN_OUT_CLEAR(K, 1)
	#define PK2_CLEAR() PIN_OUT_CLEAR(K, 2)
	#define PK3_CLEAR() PIN_OUT_CLEAR(K, 3)
	#define PK4_CLEAR() PIN_OUT_CLEAR(K, 4)
	#define PK5_CLEAR() PIN_OUT_CLEAR(K, 5)
	#define PK6_CLEAR() PIN_OUT_CLEAR(K, 6)
	#define PK7_CLEAR() PIN_OUT_CLEAR(K, 7)

	#define PK0_TOGGLE() PIN_OUT_TOGGLE(K, 0)
	#define PK1_TOGGLE() PIN_OUT_TOGGLE(K, 1)
	#define PK2_TOGGLE() PIN_OUT_TOGGLE(K, 2)
	#define PK3_TOGGLE() PIN_OUT_TOGGLE(K, 3)
	#define PK4_TOGGLE() PIN_OUT_TOGGLE(K, 4)
	#define PK5_TOGGLE() PIN_OUT_TOGGLE(K, 5)
	#define PK6_TOGGLE() PIN_OUT_TOGGLE(K, 6)
	#define PK7_TOGGLE() PIN_OUT_TOGGLE(K, 7)

	#define PK0_RESET() PIN_RESET(K, 0)
	#define PK1_RESET() PIN_RESET(K, 1)
	#define PK2_RESET() PIN_RESET(K, 2)
	#define PK3_RESET() PIN_RESET(K, 3)
	#define PK4_RESET() PIN_RESET(K, 4)
	#define PK5_RESET() PIN_RESET(K, 5)
	#define PK6_RESET() PIN_RESET(K, 6)
	#define PK7_RESET() PIN_RESET(K, 7)

	#define PK0_GET_MODE() PIN_GET_MODE(K, 0)
	#define PK1_GET_MODE() PIN_GET_MODE(K, 1)
	#define PK2_GET_MODE() PIN_GET_MODE(K, 2)
	#define PK3_GET_MODE() PIN_GET_MODE(K, 3)
	#define PK4_GET_MODE() PIN_GET_MODE(K, 4)
	#define PK5_GET_MODE() PIN_GET_MODE(K, 5)
	#define PK6_GET_MODE() PIN_GET_MODE(K, 6)
	#define PK7_GET_MODE() PIN_GET_MODE(K, 7)

	#define PK0_SET_MODE(mode) PIN_SET_MODE(K, 0, mode)
	#define PK1_SET_MODE(mode) PIN_SET_MODE(K, 1, mode)
	#define PK2_SET_MODE(mode) PIN_SET_MODE(K, 2, mode)
	#define PK3_SET_MODE(mode) PIN_SET_MODE(K, 3, mode)
	#define PK4_SET_MODE(mode) PIN_SET_MODE(K, 4, mode)
	#define PK5_SET_MODE(mode) PIN_SET_MODE(K, 5, mode)
	#define PK6_SET_MODE(mode) PIN_SET_MODE(K, 6, mode)
	#define PK7_SET_MODE(mode) PIN_SET_MODE(K, 7, mode)

#endif


#ifdef PORTL

	#define DDRL_0 PIN_REF(DDRL, 0)
	#define DDRL_1 PIN_REF(DDRL, 1)
	#define DDRL_2 PIN_REF(DDRL, 2)
	#define DDRL_3 PIN_REF(DDRL, 3)
	#define DDRL_4 PIN_REF(DDRL, 4)
	#define DDRL_5 PIN_REF(DDRL, 5)
	#define DDRL_6 PIN_REF(DDRL, 6)
	#define DDRL_7 PIN_REF(DDRL, 7)

	#define PORTL_0 PIN_REF(PORTL, 0)
	#define PORTL_1 PIN_REF(PORTL, 1)
	#define PORTL_2 PIN_REF(PORTL, 2)
	#define PORTL_3 PIN_REF(PORTL, 3)
	#define PORTL_4 PIN_REF(PORTL, 4)
	#define PORTL_5 PIN_REF(PORTL, 5)
	#define PORTL_6 PIN_REF(PORTL, 6)
	#define PORTL_7 PIN_REF(PORTL, 7)

	#define PINL_0 PIN_REF(PINL, 0)
	#define PINL_1 PIN_REF(PINL, 1)
	#define PINL_2 PIN_REF(PINL, 2)
	#define PINL_3 PIN_REF(PINL, 3)
	#define PINL_4 PIN_REF(PINL, 4)
	#define PINL_5 PIN_REF(PINL, 5)
	#define PINL_6 PIN_REF(PINL, 6)
	#define PINL_7 PIN_REF(PINL, 7)


	#define DDRL0_PAIR() PORT_PIN_PAIR(DDRL, 0)
	#define DDRL1_PAIR() PORT_PIN_PAIR(DDRL, 1)
	#define DDRL2_PAIR() PORT_PIN_PAIR(DDRL, 2)
	#define DDRL3_PAIR() PORT_PIN_PAIR(DDRL, 3)
	#define DDRL4_PAIR() PORT_PIN_PAIR(DDRL, 4)
	#define DDRL5_PAIR() PORT_PIN_PAIR(DDRL, 5)
	#define DDRL6_PAIR() PORT_PIN_PAIR(DDRL, 6)
	#define DDRL7_PAIR() PORT_PIN_PAIR(DDRL, 7)

	#define PORTL0_PAIR() PORT_PIN_PAIR(PORTL, 0)
	#define PORTL1_PAIR() PORT_PIN_PAIR(PORTL, 1)
	#define PORTL2_PAIR() PORT_PIN_PAIR(PORTL, 2)
	#define PORTL3_PAIR() PORT_PIN_PAIR(PORTL, 3)
	#define PORTL4_PAIR() PORT_PIN_PAIR(PORTL, 4)
	#define PORTL5_PAIR() PORT_PIN_PAIR(PORTL, 5)
	#define PORTL6_PAIR() PORT_PIN_PAIR(PORTL, 6)
	#define PORTL7_PAIR() PORT_PIN_PAIR(PORTL, 7)

	#define PINL0_PAIR() PORT_PIN_PAIR(PINL, 0)
	#define PINL1_PAIR() PORT_PIN_PAIR(PINL, 1)
	#define PINL2_PAIR() PORT_PIN_PAIR(PINL, 2)
	#define PINL3_PAIR() PORT_PIN_PAIR(PINL, 3)
	#define PINL4_PAIR() PORT_PIN_PAIR(PINL, 4)
	#define PINL5_PAIR() PORT_PIN_PAIR(PINL, 5)
	#define PINL6_PAIR() PORT_PIN_PAIR(PINL, 6)
	#define PINL7_PAIR() PORT_PIN_PAIR(PINL, 7)


	#define DDRL0_CHECK() CHECK_PIN(DDRL, 0)
	#define DDRL1_CHECK() CHECK_PIN(DDRL, 1)
	#define DDRL2_CHECK() CHECK_PIN(DDRL, 2)
	#define DDRL3_CHECK() CHECK_PIN(DDRL, 3)
	#define DDRL4_CHECK() CHECK_PIN(DDRL, 4)
	#define DDRL5_CHECK() CHECK_PIN(DDRL, 5)
	#define DDRL6_CHECK() CHECK_PIN(DDRL, 6)
	#define DDRL7_CHECK() CHECK_PIN(DDRL, 7)

	#define PORTL0_CHECK() CHECK_PIN(PORTL, 0)
	#define PORTL1_CHECK() CHECK_PIN(PORTL, 1)
	#define PORTL2_CHECK() CHECK_PIN(PORTL, 2)
	#define PORTL3_CHECK() CHECK_PIN(PORTL, 3)
	#define PORTL4_CHECK() CHECK_PIN(PORTL, 4)
	#define PORTL5_CHECK() CHECK_PIN(PORTL, 5)
	#define PORTL6_CHECK() CHECK_PIN(PORTL, 6)
	#define PORTL7_CHECK() CHECK_PIN(PORTL, 7)

	#define PINL0_CHECK() CHECK_PIN(PINL, 0)
	#define PINL1_CHECK() CHECK_PIN(PINL, 1)
	#define PINL2_CHECK() CHECK_PIN(PINL, 2)
	#define PINL3_CHECK() CHECK_PIN(PINL, 3)
	#define PINL4_CHECK() CHECK_PIN(PINL, 4)
	#define PINL5_CHECK() CHECK_PIN(PINL, 5)
	#define PINL6_CHECK() CHECK_PIN(PINL, 6)
	#define PINL7_CHECK() CHECK_PIN(PINL, 7)


	#define DDRL0_IS_SET() CHECK_SET_PIN(DDRL, 0)
	#define DDRL1_IS_SET() CHECK_SET_PIN(DDRL, 1)
	#define DDRL2_IS_SET() CHECK_SET_PIN(DDRL, 2)
	#define DDRL3_IS_SET() CHECK_SET_PIN(DDRL, 3)
	#define DDRL4_IS_SET() CHECK_SET_PIN(DDRL, 4)
	#define DDRL5_IS_SET() CHECK_SET_PIN(DDRL, 5)
	#define DDRL6_IS_SET() CHECK_SET_PIN(DDRL, 6)
	#define DDRL7_IS_SET() CHECK_SET_PIN(DDRL, 7)

	#define PORTL0_IS_SET() CHECK_SET_PIN(PORTL, 0)
	#define PORTL1_IS_SET() CHECK_SET_PIN(PORTL, 1)
	#define PORTL2_IS_SET() CHECK_SET_PIN(PORTL, 2)
	#define PORTL3_IS_SET() CHECK_SET_PIN(PORTL, 3)
	#define PORTL4_IS_SET() CHECK_SET_PIN(PORTL, 4)
	#define PORTL5_IS_SET() CHECK_SET_PIN(PORTL, 5)
	#define PORTL6_IS_SET() CHECK_SET_PIN(PORTL, 6)
	#define PORTL7_IS_SET() CHECK_SET_PIN(PORTL, 7)

	#define PINL0_IS_SET() CHECK_SET_PIN(PINL, 0)
	#define PINL1_IS_SET() CHECK_SET_PIN(PINL, 1)
	#define PINL2_IS_SET() CHECK_SET_PIN(PINL, 2)
	#define PINL3_IS_SET() CHECK_SET_PIN(PINL, 3)
	#define PINL4_IS_SET() CHECK_SET_PIN(PINL, 4)
	#define PINL5_IS_SET() CHECK_SET_PIN(PINL, 5)
	#define PINL6_IS_SET() CHECK_SET_PIN(PINL, 6)
	#define PINL7_IS_SET() CHECK_SET_PIN(PINL, 7)


	#define DDRL0_IS_UNSET() CHECK_UNSET_PIN(DDRL, 0)
	#define DDRL1_IS_UNSET() CHECK_UNSET_PIN(DDRL, 1)
	#define DDRL2_IS_UNSET() CHECK_UNSET_PIN(DDRL, 2)
	#define DDRL3_IS_UNSET() CHECK_UNSET_PIN(DDRL, 3)
	#define DDRL4_IS_UNSET() CHECK_UNSET_PIN(DDRL, 4)
	#define DDRL5_IS_UNSET() CHECK_UNSET_PIN(DDRL, 5)
	#define DDRL6_IS_UNSET() CHECK_UNSET_PIN(DDRL, 6)
	#define DDRL7_IS_UNSET() CHECK_UNSET_PIN(DDRL, 7)

	#define PORTL0_IS_UNSET() CHECK_UNSET_PIN(PORTL, 0)
	#define PORTL1_IS_UNSET() CHECK_UNSET_PIN(PORTL, 1)
	#define PORTL2_IS_UNSET() CHECK_UNSET_PIN(PORTL, 2)
	#define PORTL3_IS_UNSET() CHECK_UNSET_PIN(PORTL, 3)
	#define PORTL4_IS_UNSET() CHECK_UNSET_PIN(PORTL, 4)
	#define PORTL5_IS_UNSET() CHECK_UNSET_PIN(PORTL, 5)
	#define PORTL6_IS_UNSET() CHECK_UNSET_PIN(PORTL, 6)
	#define PORTL7_IS_UNSET() CHECK_UNSET_PIN(PORTL, 7)

	#define PINL0_IS_UNSET() CHECK_UNSET_PIN(PINL, 0)
	#define PINL1_IS_UNSET() CHECK_UNSET_PIN(PINL, 1)
	#define PINL2_IS_UNSET() CHECK_UNSET_PIN(PINL, 2)
	#define PINL3_IS_UNSET() CHECK_UNSET_PIN(PINL, 3)
	#define PINL4_IS_UNSET() CHECK_UNSET_PIN(PINL, 4)
	#define PINL5_IS_UNSET() CHECK_UNSET_PIN(PINL, 5)
	#define PINL6_IS_UNSET() CHECK_UNSET_PIN(PINL, 6)
	#define PINL7_IS_UNSET() CHECK_UNSET_PIN(PINL, 7)


	#define DDRL0_SET() SET_PIN(DDRL, 0)
	#define DDRL1_SET() SET_PIN(DDRL, 1)
	#define DDRL2_SET() SET_PIN(DDRL, 2)
	#define DDRL3_SET() SET_PIN(DDRL, 3)
	#define DDRL4_SET() SET_PIN(DDRL, 4)
	#define DDRL5_SET() SET_PIN(DDRL, 5)
	#define DDRL6_SET() SET_PIN(DDRL, 6)
	#define DDRL7_SET() SET_PIN(DDRL, 7)

	#define PORTL0_SET() SET_PIN(PORTL, 0)
	#define PORTL1_SET() SET_PIN(PORTL, 1)
	#define PORTL2_SET() SET_PIN(PORTL, 2)
	#define PORTL3_SET() SET_PIN(PORTL, 3)
	#define PORTL4_SET() SET_PIN(PORTL, 4)
	#define PORTL5_SET() SET_PIN(PORTL, 5)
	#define PORTL6_SET() SET_PIN(PORTL, 6)
	#define PORTL7_SET() SET_PIN(PORTL, 7)

	#define PINL0_SET() SET_PIN(PINL, 0)
	#define PINL1_SET() SET_PIN(PINL, 1)
	#define PINL2_SET() SET_PIN(PINL, 2)
	#define PINL3_SET() SET_PIN(PINL, 3)
	#define PINL4_SET() SET_PIN(PINL, 4)
	#define PINL5_SET() SET_PIN(PINL, 5)
	#define PINL6_SET() SET_PIN(PINL, 6)
	#define PINL7_SET() SET_PIN(PINL, 7)


	#define DDRL0_CLEAR() CLEAR_PIN(DDRL, 0)
	#define DDRL1_CLEAR() CLEAR_PIN(DDRL, 1)
	#define DDRL2_CLEAR() CLEAR_PIN(DDRL, 2)
	#define DDRL3_CLEAR() CLEAR_PIN(DDRL, 3)
	#define DDRL4_CLEAR() CLEAR_PIN(DDRL, 4)
	#define DDRL5_CLEAR() CLEAR_PIN(DDRL, 5)
	#define DDRL6_CLEAR() CLEAR_PIN(DDRL, 6)
	#define DDRL7_CLEAR() CLEAR_PIN(DDRL, 7)

	#define PORTL0_CLEAR() CLEAR_PIN(PORTL, 0)
	#define PORTL1_CLEAR() CLEAR_PIN(PORTL, 1)
	#define PORTL2_CLEAR() CLEAR_PIN(PORTL, 2)
	#define PORTL3_CLEAR() CLEAR_PIN(PORTL, 3)
	#define PORTL4_CLEAR() CLEAR_PIN(PORTL, 4)
	#define PORTL5_CLEAR() CLEAR_PIN(PORTL, 5)
	#define PORTL6_CLEAR() CLEAR_PIN(PORTL, 6)
	#define PORTL7_CLEAR() CLEAR_PIN(PORTL, 7)

	#define PINL0_CLEAR() CLEAR_PIN(PINL, 0)
	#define PINL1_CLEAR() CLEAR_PIN(PINL, 1)
	#define PINL2_CLEAR() CLEAR_PIN(PINL, 2)
	#define PINL3_CLEAR() CLEAR_PIN(PINL, 3)
	#define PINL4_CLEAR() CLEAR_PIN(PINL, 4)
	#define PINL5_CLEAR() CLEAR_PIN(PINL, 5)
	#define PINL6_CLEAR() CLEAR_PIN(PINL, 6)
	#define PINL7_CLEAR() CLEAR_PIN(PINL, 7)


	#define DDRL0_TOGGLE() TOGGLE_PIN(DDRL, 0)
	#define DDRL1_TOGGLE() TOGGLE_PIN(DDRL, 1)
	#define DDRL2_TOGGLE() TOGGLE_PIN(DDRL, 2)
	#define DDRL3_TOGGLE() TOGGLE_PIN(DDRL, 3)
	#define DDRL4_TOGGLE() TOGGLE_PIN(DDRL, 4)
	#define DDRL5_TOGGLE() TOGGLE_PIN(DDRL, 5)
	#define DDRL6_TOGGLE() TOGGLE_PIN(DDRL, 6)
	#define DDRL7_TOGGLE() TOGGLE_PIN(DDRL, 7)

	#define PORTL0_TOGGLE() TOGGLE_PIN(PORTL, 0)
	#define PORTL1_TOGGLE() TOGGLE_PIN(PORTL, 1)
	#define PORTL2_TOGGLE() TOGGLE_PIN(PORTL, 2)
	#define PORTL3_TOGGLE() TOGGLE_PIN(PORTL, 3)
	#define PORTL4_TOGGLE() TOGGLE_PIN(PORTL, 4)
	#define PORTL5_TOGGLE() TOGGLE_PIN(PORTL, 5)
	#define PORTL6_TOGGLE() TOGGLE_PIN(PORTL, 6)
	#define PORTL7_TOGGLE() TOGGLE_PIN(PORTL, 7)

	#define PINL0_TOGGLE() TOGGLE_PIN(PINL, 0)
	#define PINL1_TOGGLE() TOGGLE_PIN(PINL, 1)
	#define PINL2_TOGGLE() TOGGLE_PIN(PINL, 2)
	#define PINL3_TOGGLE() TOGGLE_PIN(PINL, 3)
	#define PINL4_TOGGLE() TOGGLE_PIN(PINL, 4)
	#define PINL5_TOGGLE() TOGGLE_PIN(PINL, 5)
	#define PINL6_TOGGLE() TOGGLE_PIN(PINL, 6)
	#define PINL7_TOGGLE() TOGGLE_PIN(PINL, 7)


	#define PL0_CHECK() PIN_IN_CHECK(L, 0)
	#define PL1_CHECK() PIN_IN_CHECK(L, 1)
	#define PL2_CHECK() PIN_IN_CHECK(L, 2)
	#define PL3_CHECK() PIN_IN_CHECK(L, 3)
	#define PL4_CHECK() PIN_IN_CHECK(L, 4)
	#define PL5_CHECK() PIN_IN_CHECK(L, 5)
	#define PL6_CHECK() PIN_IN_CHECK(L, 6)
	#define PL7_CHECK() PIN_IN_CHECK(L, 7)

	#define PL0_IS_SET() PIN_IN_CHECK_SET(L, 0)
	#define PL1_IS_SET() PIN_IN_CHECK_SET(L, 1)
	#define PL2_IS_SET() PIN_IN_CHECK_SET(L, 2)
	#define PL3_IS_SET() PIN_IN_CHECK_SET(L, 3)
	#define PL4_IS_SET() PIN_IN_CHECK_SET(L, 4)
	#define PL5_IS_SET() PIN_IN_CHECK_SET(L, 5)
	#define PL6_IS_SET() PIN_IN_CHECK_SET(L, 6)
	#define PL7_IS_SET() PIN_IN_CHECK_SET(L, 7)

	#define PL0_IS_UNSET() PIN_IN_CHECK_UNSET(L, 0)
	#define PL1_IS_UNSET() PIN_IN_CHECK_UNSET(L, 1)
	#define PL2_IS_UNSET() PIN_IN_CHECK_UNSET(L, 2)
	#define PL3_IS_UNSET() PIN_IN_CHECK_UNSET(L, 3)
	#define PL4_IS_UNSET() PIN_IN_CHECK_UNSET(L, 4)
	#define PL5_IS_UNSET() PIN_IN_CHECK_UNSET(L, 5)
	#define PL6_IS_UNSET() PIN_IN_CHECK_UNSET(L, 6)
	#define PL7_IS_UNSET() PIN_IN_CHECK_UNSET(L, 7)


	#define PL0_SET() PIN_OUT_SET(L, 0)
	#define PL1_SET() PIN_OUT_SET(L, 1)
	#define PL2_SET() PIN_OUT_SET(L, 2)
	#define PL3_SET() PIN_OUT_SET(L, 3)
	#define PL4_SET() PIN_OUT_SET(L, 4)
	#define PL5_SET() PIN_OUT_SET(L, 5)
	#define PL6_SET() PIN_OUT_SET(L, 6)
	#define PL7_SET() PIN_OUT_SET(L, 7)

	#define PL0_CLEAR() PIN_OUT_CLEAR(L, 0)
	#define PL1_CLEAR() PIN_OUT_CLEAR(L, 1)
	#define PL2_CLEAR() PIN_OUT_CLEAR(L, 2)
	#define PL3_CLEAR() PIN_OUT_CLEAR(L, 3)
	#define PL4_CLEAR() PIN_OUT_CLEAR(L, 4)
	#define PL5_CLEAR() PIN_OUT_CLEAR(L, 5)
	#define PL6_CLEAR() PIN_OUT_CLEAR(L, 6)
	#define PL7_CLEAR() PIN_OUT_CLEAR(L, 7)

	#define PL0_TOGGLE() PIN_OUT_TOGGLE(L, 0)
	#define PL1_TOGGLE() PIN_OUT_TOGGLE(L, 1)
	#define PL2_TOGGLE() PIN_OUT_TOGGLE(L, 2)
	#define PL3_TOGGLE() PIN_OUT_TOGGLE(L, 3)
	#define PL4_TOGGLE() PIN_OUT_TOGGLE(L, 4)
	#define PL5_TOGGLE() PIN_OUT_TOGGLE(L, 5)
	#define PL6_TOGGLE() PIN_OUT_TOGGLE(L, 6)
	#define PL7_TOGGLE() PIN_OUT_TOGGLE(L, 7)

	#define PL0_RESET() PIN_RESET(L, 0)
	#define PL1_RESET() PIN_RESET(L, 1)
	#define PL2_RESET() PIN_RESET(L, 2)
	#define PL3_RESET() PIN_RESET(L, 3)
	#define PL4_RESET() PIN_RESET(L, 4)
	#define PL5_RESET() PIN_RESET(L, 5)
	#define PL6_RESET() PIN_RESET(L, 6)
	#define PL7_RESET() PIN_RESET(L, 7)

	#define PL0_GET_MODE() PIN_GET_MODE(L, 0)
	#define PL1_GET_MODE() PIN_GET_MODE(L, 1)
	#define PL2_GET_MODE() PIN_GET_MODE(L, 2)
	#define PL3_GET_MODE() PIN_GET_MODE(L, 3)
	#define PL4_GET_MODE() PIN_GET_MODE(L, 4)
	#define PL5_GET_MODE() PIN_GET_MODE(L, 5)
	#define PL6_GET_MODE() PIN_GET_MODE(L, 6)
	#define PL7_GET_MODE() PIN_GET_MODE(L, 7)

	#define PL0_SET_MODE(mode) PIN_SET_MODE(L, 0, mode)
	#define PL1_SET_MODE(mode) PIN_SET_MODE(L, 1, mode)
	#define PL2_SET_MODE(mode) PIN_SET_MODE(L, 2, mode)
	#define PL3_SET_MODE(mode) PIN_SET_MODE(L, 3, mode)
	#define PL4_SET_MODE(mode) PIN_SET_MODE(L, 4, mode)
	#define PL5_SET_MODE(mode) PIN_SET_MODE(L, 5, mode)
	#define PL6_SET_MODE(mode) PIN_SET_MODE(L, 6, mode)
	#define PL7_SET_MODE(mode) PIN_SET_MODE(L, 7, mode)

#endif

#pragma endregion

#endif
